with unchecked_deallocation;
with integer_io;                        use integer_io;
with Symbol_Table_io;
with Permute_Operations;                use Permute_Operations;

package body Extrinsic_Diagonal_Homotopies_io is

  procedure Write ( sa : in Symbol_Array ) is
  begin
    Write(Standard_Output,sa);
  end Write;

  procedure Write ( file : in file_type; sa : in Symbol_Array ) is
  begin
    for i in sa'range loop
      put(file," "); Symbol_Table_io.put(file,sa(i));
    end loop;
    new_line(file);
  end Write;

  function Add_Suffix ( sb : Symbol; c : character ) return Symbol is

    res : Symbol := sb;
    ind : natural := sb'first;

  begin
    while sb(ind) /= ' ' loop
      ind := ind + 1;
    end loop;
    res(ind) := c;
    return res;
  end Add_Suffix;

  function Add_Suffix ( sa : Symbol_Array; c : character )
                      return Symbol_Array is

    res : Symbol_Array(sa'range);
 
  begin
    for i in sa'range loop
      res(i) := Add_Suffix(sa(i),c);
    end loop;
    return res;
  end Add_Suffix;

  function Remove_Suffix ( sb : Symbol ) return Symbol is

    res : Symbol := sb;
    ind : natural := sb'first;

  begin
    while sb(ind) /= ' ' loop
      ind := ind + 1;
    end loop;
    res(ind-1) := ' ';
    return res;
  end Remove_Suffix;

  function Get_Symbols return Symbol_Array is

    res : Symbol_Array(1..Symbol_Table.Number);

  begin
    for i in 1..Symbol_Table.Number loop
      res(i) := Symbol_Table.Get(i);
    end loop;
    return res;
  end Get_Symbols;

  function Get_Link_to_Symbols return Link_to_Symbol_Array is

    res : Link_to_Symbol_Array;
    res_rep : constant Symbol_Array := Get_Symbols;

  begin
    res := new Symbol_Array'(res_rep);
    return res;
  end Get_Link_to_Symbols;

  function Suffixed_Symbols ( c : character ) return Symbol_Array is

    res : Symbol_Array(1..Symbol_Table.Number);

  begin
    for i in 1..Symbol_Table.Number loop 
      declare
        sb : Symbol := Symbol_Table.get(i);
      begin
        res(i) := Add_Suffix(sb,c);
      end;
    end loop;
    return res;
  end Suffixed_Symbols;

  function Is_Embed_Symbol ( sb : Symbol ) return boolean is
  begin
    if sb(1) /= 'z'  
     then return false;
     elsif sb(2) /= 'z'
         then return false;
         else return true;
    end if;
  end Is_Embed_Symbol;

  function Remove_Embed_Symbols ( sa : Symbol_Array ) return Symbol_Array is

    res : Symbol_Array(sa'range);
    ind : natural := res'first-1;

  begin
    for i in sa'range loop
      if not Is_Embed_Symbol(sa(i))
       then ind := ind + 1;
            res(ind) := sa(i);
      end if;
    end loop;
    return res(res'first..ind);
  end Remove_Embed_Symbols;

  function Look_for_Position 
              ( sa : Symbol_Array; sb : Symbol ) return natural is
  begin
    for i in sa'range loop
      if Equal(sa(i),sb)
       then return i;
      end if;
    end loop;
    return 0;
  end Look_for_Position;

  function Match_Symbols
              ( s1,s2 : in Symbol_Array ) return Permutation is

    prm : Permutation(s2'range);

  begin
    for i in s2'range loop
      prm(i) := Look_for_Position(s1,s2(i));
    end loop;
    return prm;
  end Match_Symbols;

  function Equals_mod_Suffix ( sb1,sb2 : Symbol ) return boolean is

    ind : natural := sb1'first;

  begin
    while sb1(ind) /= ' ' loop
      ind := ind + 1;
    end loop;
    ind := ind - 1;
    for i in sb1'first..(ind-1) loop
      if sb1(i) /= sb2(i)
       then return false;
      end if;
    end loop;
    return true;
  end Equals_mod_Suffix;

  function Search_Position 
              ( sa : Symbol_Array; sb : Symbol ) return natural is
  begin
    for i in sa'range loop
      if Equals_mod_Suffix(sa(i),sb)
       then return i;
      end if;
    end loop;
    return 0;
  end Search_Position;

  function Matching_Permutation
              ( s1,s2 : in Symbol_Array ) return Permutation is

    prm : Permutation(s2'range);

  begin
    for i in s2'range loop
     -- put("Matching the "); put(i,1); put("-th symbol ");
     -- Symbol_Table_io.put(s2(i)); new_line;
      prm(i) := Search_Position(s1,s2(i));
    end loop;
    return prm;
  end Matching_Permutation;

  procedure Permute ( prm : in Permutation; s : in out Symbol_Array ) is

    res : Symbol_Array(s'range) := s;

  begin
    for i in prm'range loop
      res(prm(i)) := s(i);
    end loop;
    s := res;
  end Permute;

  procedure Write_Symbol_Table is
  begin
    put_line("The symbol table :");
    for i in 1..Symbol_Table.Number loop
      put(" ");
      Symbol_Table_io.put(Symbol_Table.get(i));
    end loop;
    new_line;
  end Write_Symbol_Table;

  procedure Assign_Symbol_Table ( sa : in Symbol_Array ) is
  begin
    Symbol_Table.Clear;
    Symbol_Table.Init(sa'length);
    for i in sa'range loop
      Symbol_Table.Add(sa(i));
    end loop;
  end Assign_Symbol_Table;

  procedure Assign_Symbol_Table ( s1,s2 : in Symbol_Array ) is
  begin
    Symbol_Table.Clear;
    Symbol_Table.Init(s1'length+s2'length);
    for i in s1'range loop
      Symbol_Table.Add(s1(i));
    end loop;
    for i in s2'range loop
      Symbol_Table.Add(s2(i));
    end loop;
  end Assign_Symbol_Table;

  procedure Clear ( ls : in out Link_to_Symbol_Array ) is

    procedure free is
      new unchecked_deallocation(Symbol_Array,Link_to_Symbol_Array);

  begin
    free(ls);
  end Clear;

end Extrinsic_Diagonal_Homotopies_io;
