with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Integer_Vectors;          use Standard_Integer_Vectors;
with Standard_Integer_Vectors_io;       use Standard_Integer_Vectors_io;
with Standard_Complex_VecVecs;          use Standard_Complex_VecVecs;
with Symbol_Table,Symbol_Table_io;      use Symbol_Table;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Permutations,Permute_Operations;   use Permutations,Permute_Operations;
with Witness_Points;                    use Witness_Points;
with Witness_Points_io;                 use Witness_Points_io;
with Planes_and_Polynomials;            use Planes_and_Polynomials;
with Extrinsic_Diagonal_Homotopies;     use Extrinsic_Diagonal_Homotopies;
with Extrinsic_Diagonal_Homotopies_io;  use Extrinsic_Diagonal_Homotopies_io;

package body Extrinsic_Diagonal_Solvers is

-- UTILITIES TO SET UP THE DIAGONAL HOMOTOPIES :

  function Is_Dummy ( p : Poly; k : natural ) return boolean is

  -- DESCRIPTION :
  --   Returns true if the polynomial has only one term that is linear
  --   and where the occurring variables are among the k last ones.

    n : constant natural := Number_of_Unknowns(p);

  begin
    if Number_of_Terms(p) > 1
     then return false;
     elsif Degree(p) > 1
         then return false;
         else for i in 1..n-k loop
                if Degree(p,i) = 1
                 then return false;
                end if;
              end loop;
              return true;
    end if;
  end Is_Dummy;

  function Number_of_Dummies ( p : Poly_Sys; k : natural ) return natural is

  -- DESCRIPTION :
  --   Returns the number of equations that turn the slack variables
  --   into dummies.

    res : natural := 0;

  begin
    for i in p'range loop
      if Is_Dummy(p(i),k)
       then res := res + 1;
      end if;
    end loop;
    return res;
  end Number_of_Dummies;
 
-- UTILITIES TO SAVE THE RESULTS :

  procedure Save_Start_System ( p : in Poly_Sys; s : in Solution_List ) is

  -- DESCRIPTION :
  --   The user is prompted for a file name to save the start system
  --   in the homotopy to start the diagonal cascade.

    file : file_type;

  begin
    new_line;
    put_line("Reading the name of the file to save the start system.");
    Read_Name_and_Create_File(file);
    put_line(file,p);
    new_line(file);
    put_line(file,"THE SOLUTIONS :");
    put(file,Length_Of(s),p'last,s);
  end Save_Start_System;

  procedure Save_Target_System ( p : in Poly_Sys ) is

  -- DESCRIPTION :
  --   The user is prompted for a file name to save the target system
  --   in the homotopy to start the diagonal cascade.

    file : file_type;

  begin
    new_line;
    put_line("Reading the name of the file to save the target system.");
    Read_Name_and_Create_File(file);
    put_line(file,p);
  end Save_Target_System;

  procedure Save_Collapsed_System ( p : in Poly_Sys; s : in Solution_List ) is

  -- DESCRIPTION :
  --   The user is prompted for a file name to save the system and its
  --   solutions after collapsing from the diagonal to the original space.

    file : file_type;

  begin
    new_line;
    put_line("Reading the name of the file to save the collapsed system.");
    Read_Name_and_Create_File(file);
    put_line(file,p);
    new_line(file);
    put_line(file,"THE SOLUTIONS :");
    put(file,Length_Of(s),p'last,s);
  end Save_Collapsed_System;

-- MAIN DRIVERS :

  procedure Randomize_System is

    lp : Link_to_Poly_Sys;
    k : natural;
 
  begin
    new_line;
    put_line("Reading the polynomial system...");
    get(lp);
    new_line;
    put("Give the dimension of the solution component : ");
    get(k); skip_line;
    declare 
      n : constant natural := Number_of_Unknowns(lp(lp'first));
      rp : constant Poly_Sys := Complete(n,k,lp.all);
    begin
      Save_Target_System(rp);
    end;
  end Randomize_System;

  procedure Build_Cascade_Homotopy
              ( file : in file_type;
                p1e,p2e : in Poly_Sys; dim1,dim2 : in natural;
                sols1e,sols2e : in Solution_List;
                s1e,s2e : in Symbol_Array ) is

  -- DESCRIPTION :
  --   Builds the homotopy to start the cascade for all components of
  --   the intersection of two solution components (of dimensions dim1
  --   and dim2) defined by the embedded systems ep1 and ep2.

  -- ON ENTRY :
  --   p1e      1st embedded polynomial system;
  --   p2e      2nd embedded polynomial system;
  --   dim1     dimension of the component defined by the 1st system;
  --   dim2     dimension of the component defined by the 2nd system;
  --   sols1e   witness points on the 1st solution component;
  --   sols1e   witness points on the 2nd solution component;
  --   s1e      symbols used in the 1st polynomial system;
  --   s2e      symbols used in the 2nd polynomial system.

    dim : constant natural := Cascade_Dimension(p1e,p2e,dim1,dim2);
    k : constant natural := Number_of_Unknowns(p1e(p1e'first))-dim1;
    ch_start,ch_target : Poly_Sys(1..dim);
    sols1 : Solution_List := Remove_Embedding(sols1e,dim1);
    sols2 : Solution_List := Remove_Embedding(sols2e,dim2);
    sols : Solution_List := Product(sols1,sols2);
    embsols : Solution_List;
    s1 : constant Symbol_Array := Remove_Embed_Symbols(s1e);
    s11 : Symbol_Array(s1'range) := Add_Suffix(s1,'1');
    s2 : constant Symbol_Array := Remove_Embed_Symbols(s2e);
    s22 : Symbol_Array(s1'range) := Add_Suffix(s2,'2');
 
  begin
    if dim1+dim2 < k
     then Cascade1(p1e,p2e,dim1,dim2,ch_start,ch_target);
          embsols := Add_Embedding(sols,dim2);
     else Cascade2(p1e,p2e,dim1,dim2,ch_start,ch_target);
          embsols := Add_Embedding(sols,k-dim1);
    end if;
    Symbol_Table.Clear;
    Assign_Symbol_Table(s11,s22);
    Add_Embed_Symbols(dim2);
    Save_Target_System(ch_target);
    Save_Start_System(ch_start,embsols);
  end Build_Cascade_Homotopy;

  procedure Permute ( p : in Permutation; sols : in out Solution_List ) is

  -- DESCRIPTION :
  --   Permutes the vectors in the solution list with the permutation p.

    tmp : Solution_List := sols;

  begin
    while not Is_Null(tmp) loop
      declare
        ls : Link_to_Solution := Head_Of(tmp);
      begin
        ls.v := p*ls.v;
        Set_Head(tmp,ls);
      end;
      tmp := Tail_Of(tmp);
    end loop;
  end Permute;

  procedure Permute_by_Symbols
              ( file : in file_type; p2e : in out Poly_Sys;
                sols2e : in out Solution_List;
                s1,s2 : in out Symbol_Array; dim1,dim2 : in natural ) is

  -- DESCRIPTION :
  --   While the polynomial systems should have all their variables
  --   in common, they may have occurred at different places.
  --   This procedure permutes the variables in the system if needed.

    os1 : Symbol_Array(s1'first..s1'last-dim1) := s1(s1'first..s1'last-dim1);
    os2 : Symbol_Array(s2'first..s2'last-dim2) := s2(s2'first..s2'last-dim2);
    prm : Permutation(os1'range) := Match_Symbols(os1,os2);
    full_prm,inv_prm : Permutation(s2'range);
    tmp : Solution_List;

  begin
    put(file,"The symbols from the 1st system : "); Write(file,s1);
    put(file,"The symbols from the 2nd system : "); Write(file,s2);
    put(file,"The original first symbols  : "); Write(file,os1);
    put(file,"The original second symbols : "); Write(file,os2);
    put(file,"The matching permutation : ");
    put(file,Standard_Integer_Vectors.Vector(prm)); new_line(file);
    Permute(prm,os2);
    s2(os2'range) := os2;
    put(file,"The permuted second symbols : "); Write(file,os2);
    full_prm(prm'range) := prm;
    for i in prm'last+1..full_prm'last loop
      full_prm(i) := i;
    end loop;
    put(file,"The full matching permutation : ");
    put(file,Standard_Integer_Vectors.Vector(full_prm)); new_line(file);
    inv_prm := inv(full_prm);
    put(file,"Inverse of full permutation   : ");
    put(file,Standard_Integer_Vectors.Vector(inv_prm)); new_line(file);
    p2e := p2e*inv_prm;
    Assign_Symbol_Table(s2);
    put(file,"The permuted second system : "); put(file,p2e);
    Permute(inv_prm,sols2e);
  end Permute_by_Symbols;

  procedure Build_Diagonal_Cascade is
 
    file : file_type;
    lp1,lp2 : Link_to_Poly_Sys;
    sols1e,sols2e : Solution_List;
    dim1,dum1,dim2,dum2 : natural;
    lsym1,lsym2 : Link_to_Symbol_Array;

  begin
    new_line;
    put_line("Reading the first embedded polynomial system...");
    Standard_Read_Embedding(lp1,sols1e,dim1);
    dum1 := Number_of_Dummies(lp1.all,dim1);
    lsym1 := Get_Link_to_Symbols;
    new_line;
    put_line("Reading the second embedded polynomial system...");
    Standard_Read_Embedding(lp2,sols2e,dim2);
    dum2 := Number_of_Dummies(lp2.all,dim2);
    if dim1 < dim2
     then new_line;
          put("WARNING:");
          put(" dim(component-1) = "); put(dim1,1);
          put(", dim(component-2) = "); put(dim2,1);
          put(", and "); put(dim1,1); put("<"); put(dim2,1); put_line(".");
          put_line("This will cause too many variables in the homotopies.");
          put_line("Be advised to stop program and switch order of systems.");
    end if;
    new_line;
    put_line("Reading the name of the output file...");
    Read_Name_and_Create_File(file);
    put_line(file,"The first polynomial system :"); put(file,lp1.all);
    put(file,"Dimension of component 1  : ");
    put(file,dim1,1); new_line(file);
    put(file,"Number of dummy slackvars : ");
    put(file,dum1,1); new_line(file);
    put_line(file,"The second polynomial system :"); put(file,lp2.all);
    put(file,"Dimension of component 2  : ");
    put(file,dim2,1); new_line(file);
    put(file,"Number of dummy slackvars : ");
    put(file,dum2,1); new_line(file);
    lsym2 := Get_Link_to_Symbols;
    put(file,"2nd Symbols :"); Write(file,lsym2.all);
    Permute_by_Symbols(file,lp2.all,sols2e,lsym1.all,lsym2.all,dim1,dim2);
    put(file,"1st Symbols :"); Write(file,lsym1.all);
    put(file,"2nd Symbols :"); Write(file,lsym2.all);
   -- Assign_Symbol_Table(lsym2.all);
    Build_Cascade_Homotopy
      (file,lp1.all,lp2.all,dim1,dim2,sols1e,sols2e,lsym1.all,lsym2.all);
  end Build_Diagonal_Cascade;

  procedure Replace_with_Dummies 
               ( p : in out Poly_Sys; n,dim : in natural ) is

  -- DESCRIPTION :
  --   Equations which collapsed into constant = 0 equations are replaced
  --   with equations setting the dummy embedding variables to zero.

  -- REQUIRED : dim > 0.

    nd : natural := 0;
    dt : Term;

  begin
    dt.cf := Create(1.0);
    dt.dg := new Standard_Natural_Vectors.Vector'(1..n+dim => 0);
    for i in p'range loop
      if Number_of_Terms(p(i)) <= 1
       then nd := nd + 1;
            dt.dg(n+nd) := 1;
            Clear(p(i));
            p(i) := Create(dt);
      end if;
      exit when (nd = dim);
    end loop;
    Clear(dt);
  end Replace_with_Dummies;

  procedure Collapsed_Slices ( p : in Poly_Sys; n,dim : in natural;
                               cp : in out Poly_Sys ) is     

  -- DESCRIPTION :
  --   Collapses the last dim equations of p to the end of cp.

  -- REQUIRED : dim > 0.

    ind : natural := p'last-dim+1;

  begin
    for i in cp'last-dim+1..cp'last loop
      Clear(cp(i));
      cp(i) := Collapse(p(ind),n);
      ind := ind + 1;
    end loop;
  end Collapsed_Slices;

  procedure Collapse_Diagonal_System is

    lp : Link_to_Poly_Sys;
    sols : Solution_List;
    dim : natural;

  begin
    new_line;
    put("Reading the diagonal system...");
    Standard_Read_Embedding(lp,sols,dim);
    new_line;
    Write_Symbol_Table;
    declare
      sa : constant Symbol_Array := Get_Symbols;
      n : constant natural := (sa'last-dim)/2;
      s1 : Symbol_Array(1..n) := sa(1..n);
      s2 : Symbol_Array(1..n) := sa(n+1..sa'last);
      prm : Permutation(1..n) := Matching_Permutation(s1,s2);
      full_prm : Permutation(lp'range);
      pp2 : Poly_Sys(lp'range);
      cp : Poly_Sys(1..n+dim);
      tsols : Solution_List;
    begin
      put("The original symbols : "); Write(sa(1..sa'last-dim)); new_line;
      put("Symbols of 1st system : "); Write(s1); new_line;
      put("Symbols of 2nd system : "); Write(s2); new_line;
      put("The permutation : ");
      put(Standard_Integer_Vectors.Vector(prm)); new_line;
      put("The dimension is "); put(dim,1); new_line;
      for i in 1..n loop
        full_prm(i) := i;
        full_prm(i+n) := prm(i)+n;
      end loop;
      for i in 2*n+1..full_prm'last loop
        full_prm(i) := i;
      end loop;
      pp2 := lp.all*full_prm;
      put_line("The permuted system : "); put(pp2);
      Permute(full_prm,sols);
      tsols := Truncate(sols,n);
      for i in cp'range loop
        cp(i) := Collapse(pp2(i),n);
      end loop;
      for i in s1'range loop
        s1(i) := Remove_Suffix(s1(i));
      end loop;
      Assign_Symbol_Table(s1);
      Add_Embed_Symbols(dim);
      put_line("The system after first collapse :"); put(cp);
      if dim > 0
       then Replace_with_Dummies(cp,n,dim);
            put_line("After replace with dummies : "); put(cp);
            Collapsed_Slices(pp2,n,dim,cp);
            put_line("After collapsing slices : "); put(cp);
            for i in cp'range loop
              declare
                newcp : Poly;
              begin
                if Number_of_Unknowns(cp(i)) < n+dim
                 then newcp := Add_Embedding(cp(i),dim);
                      Copy(newcp,cp(i)); Clear(newcp);
                end if;
              end;
            end loop;
            Clear(sols);
            sols := Add_Embedding(tsols,dim);
            tsols := sols;
      end if;
      put_line("The collapsed system : "); put(cp);
      Save_Collapsed_System(cp,tsols);
    end;
  end Collapse_Diagonal_System;

end Extrinsic_Diagonal_Solvers;
