with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;

package Extrinsic_Diagonal_Solvers is

-- DESCRIPTION :
--   This package offers drivers to compute witness sets on the intersection
--   of positive dimensional components by extrinsic diagonal homotopies.
--   Note that the routines in this package do not solve, but provide the
--   homotopies on which standard continuation can be applied directly.

  procedure Randomize_System;

  -- DESCRIPTION :
  --   Interactive driver to randomize a system so that the number
  --   of equations equals the co-dimension of the solution component
  --   which is to be sampled.

  procedure Build_Diagonal_Cascade;

  -- DESCRIPTION :
  --   Interactive driver to build a homotopy to start a cascade
  --   of homotopies to compute all positive dimensional components
  --   of the intersection of two components.

  procedure Collapse_Diagonal_System;

  -- DESCRIPTION :
  --   Once the target system in the diagonal homotopy has been solved,
  --   we want to return to the original set of coordinates.
  --   This is what this interactive driver does.

end Extrinsic_Diagonal_Solvers;
