with text_io,integer_io;                 use text_io,integer_io;
with Communications_with_User;           use Communications_with_User;
with Standard_Complex_Solutions;         use Standard_Complex_Solutions;
with Standard_Complex_Polynomials;       use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems;      use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;   use Standard_Complex_Poly_Systems_io;
with Drivers_for_Poly_Continuation;      use Drivers_for_Poly_Continuation;
with Witness_Points,Witness_Points_io;   use Witness_Points,Witness_Points_io;
with Diagonal_Homotopy_Continuation;     use Diagonal_Homotopy_Continuation;
with Drivers_to_Cascade_Filtering;       use Drivers_to_Cascade_Filtering;
with Extrinsic_Diagonal_Solvers;         use Extrinsic_Diagonal_Solvers;
with Drivers_to_Intersect_Varieties;     use Drivers_to_Intersect_Varieties;

procedure maindeco ( infilename,outfilename : in string ) is

  procedure Read_System
               ( filename : in string; lp : out Link_to_Poly_Sys ) is

  -- DESCRIPTION :
  --   Attempts to open the file with name in the string filename
  --   in order to read the polynomial system on it.

  -- ON ENTRY :
  --   filename  name of file supplied by user from dispatcher.

  -- ON RETURN :
  --   lp        a polynomial system.

    file : file_type;

  begin
    Open(file,in_file,filename);
    get(file,lp);
    Close(file);
  exception
    when others => new_line;
                   put("Could not open file with name ");
                   put(filename); put_line(".");
                   lp := null; return;
  end Read_System;

  procedure Read_Two_Witness_Sets
              ( lp1,lp2 : out Link_to_Poly_Sys;
                sols1,sols2 : out Solution_List;
                dim1,dim2 : out natural ) is

  -- DESCRIPTION :
  --   Prompts the user for two files for two witness sets.

  begin
    new_line;
    put_line("Reading the first embedded polynomial system.");
    Standard_Read_Embedding(lp1,sols1,dim1);
    new_line;
    put_line("Reading the second embedded polynomial system.");
    Standard_Read_Embedding(lp2,sols2,dim2);
  end Read_Two_Witness_Sets;

  procedure Call_Extrinsic_Diagonal_Homotopies is

    lp1,lp2 : Link_to_Poly_Sys;
    esols1,esols2,sols1,sols2 : Solution_List;
    dim1,dim2,oc : natural;
    file : file_type;
    report : boolean;

  begin
    new_line;
    put_line("Executing diagonal homotopies extrinsically...");
    Read_Two_Witness_Sets(lp1,lp2,esols1,esols2,dim1,dim2);
    sols1 := Remove_Embedding(esols1,dim1);
    sols2 := Remove_Embedding(esols2,dim2);
    new_line;
    put_line("Reading the name of the output file.");
    Read_Name_and_Create_File(file);
    new_line;
    Driver_for_Continuation_Parameters(file);
    new_line;
    Driver_for_Process_io(file,oc);
    report := (oc /= 0);
    new_line;
    put_line("Running continuation, see the output file for results...");
    new_line;
    if dim1 >= dim2
     then Extrinsic_Diagonal_Homotopy
            (file,report,lp1.all,lp2.all,dim1,dim2,sols1,sols2);
     else Extrinsic_Diagonal_Homotopy
            (file,report,lp2.all,lp1.all,dim2,dim1,sols2,sols1);
    end if;
  end Call_Extrinsic_Diagonal_Homotopies;

  procedure Call_Intrinsic_Diagonal_Homotopies is

    lp1,lp2 : Link_to_Poly_Sys;
    sols1,sols2 : Solution_List;
    dim1,dim2,oc : natural;
    file : file_type;
    ans : character;
    generic_version,report : boolean;

  begin
    new_line;
    put_line("Executing diagonal homotopies intrinsically...");
    Read_Two_Witness_Sets(lp1,lp2,sols1,sols2,dim1,dim2);
    new_line;
    put_line("Reading the name of the output file.");
    Read_Name_and_Create_File(file);
    new_line;
    put("Do you want the version using generics ? (y/n) ");
    Ask_Yes_or_No(ans);
    generic_version := (ans = 'y');
    Driver_for_Continuation_Parameters(file);
    new_line;
    Driver_for_Process_io(file,oc);
    report := (oc /= 0);
    new_line;
    put_line("Running continuation, see the output file for results...");
    new_line;
    if generic_version
     then if dim1 >= dim2
           then A_Call_Generic_Diagonal_Homotopy
                  (file,lp1.all,lp2.all,sols1,sols2,dim1,dim2);
           else A_Call_Generic_Diagonal_Homotopy
                  (file,lp2.all,lp1.all,sols2,sols1,dim2,dim1);
          end if;
     else if dim1 >= dim2
           then Intrinsic_Diagonal_Homotopy
                  (file,report,lp1.all,lp2.all,sols1,sols2,dim1,dim2);
           else Intrinsic_Diagonal_Homotopy
                  (file,report,lp2.all,lp1.all,sols2,sols1,dim2,dim1);
          end if;
    end if;
  end Call_Intrinsic_Diagonal_Homotopies;

  procedure Main is

    ans : character;
    file : file_type;
    lp : Link_to_Poly_Sys;
    k : natural;

  begin
    new_line;
    put_line("MENU to generate and classify witness points :");
    put_line("  1. generate an embedding for the top dimensional component;");
    put_line("  2. given the top embedding, compute all witness points;");
    put_line("  3. perform steps 1, 2, and classify the witness points.");
    put_line("MENU to intersect positive dimensional solution components :");
    put_line("  4. build extrinsic diagonal cascade from 2 embedded systems;");
    put_line("  5. eliminate the diagonal, go to original at end of cascade;");
    put_line("  6. intersect varieties with extrinsic diagonal homotopies;");
    put_line("  7.                          intrinsic diagonal homotopies.");
    put("Type 1, 2, 3, 4, 5, 6, or 7 to choose : ");
    Ask_Alternative(ans,"1234567");
    case ans is
      when '1' => Driver_to_Square_and_Embed;
      when '2' => Driver_to_Witness_Generate;
      when '3' => if infilename /= ""
                   then Read_System(infilename,lp);
                  end if;
                  if lp = null
                   then new_line;
                        put_line("Reading the polynomial system.");
                        get(lp);
                  end if;
                  Create_Output_File(file,outfilename);
                  new_line;
                  put("Give the top dimension : "); get(k); skip_line;
                  Driver_for_Cascade_Filter(file,lp.all,k);
      when '4' => Build_Diagonal_Cascade;
      when '5' => Collapse_Diagonal_System;
      when '6' => Call_Extrinsic_Diagonal_Homotopies;
      when '7' => Call_Intrinsic_Diagonal_Homotopies;
      when others => null;
    end case;
  end Main;

begin
  Main;
end maindeco;
