with integer_io;                        use integer_io;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Multprec_Floating_Numbers;         use Multprec_Floating_Numbers;
with Multprec_Floating_Numbers_io;      use Multprec_Floating_Numbers_io;
with Multprec_Complex_Numbers;          use Multprec_Complex_Numbers;
with Multprec_Complex_Numbers_io;       use Multprec_Complex_Numbers_io;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Multprec_Complex_Vectors_io;       use Multprec_Complex_Vectors_io;
with Multprec_Complex_Norms_Equals;     use Multprec_Complex_Norms_Equals;
with Multprec_Complex_Vector_Tools;     use Multprec_Complex_Vector_Tools;
with Standard_Complex_VecVecs;
with Multprec_Complex_Polynomials;      use Multprec_Complex_Polynomials;
with Multprec_Complex_Polynomials_io;   use Multprec_Complex_Polynomials_io;
with Multprec_Complex_Poly_Functions;   use Multprec_Complex_Poly_Functions;
with Multprec_Complex_Poly_SysFun;      use Multprec_Complex_Poly_SysFun;
with Standard_Complex_Poly_SysFun;      use Standard_Complex_Poly_SysFun;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Multprec_Complex_Poly_Systems_io;  use Multprec_Complex_Poly_Systems_io;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Planes_and_Polynomials;            use Planes_and_Polynomials;
with Multprec_Linear_Projections;       use Multprec_Linear_Projections;
with Multprec_Central_Projections;      use Multprec_Central_Projections;

package body Multprec_Membership_Tests is

  function In_Subspace
               ( file : in file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys;
                 ind : natural; x : Standard_Complex_Vectors.Vector;
                 tol : double_float ) return boolean is

    mpx : Multprec_Complex_Vectors.Vector(x'range) := Create(x);
    res : boolean := In_Subspace(file,p,ind,mpx,tol);

  begin
    Multprec_Complex_Vectors.Clear(mpx);
    return res;
  end In_Subspace;

  function In_Subspace
               ( file : in file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys;
                 ind : natural; x : Multprec_Complex_Vectors.Vector;
                 tol : double_float ) return boolean is

    eva : Multprec_Complex_Vectors.Vector(p'range) := Eval(p,x);
    val : Floating_Number := Max_Norm(eva);
    fltval : double_float := Trunc(val);
    res : boolean := (fltval <= tol);

  begin
    put(file,"Residual in span of component "); put(file,ind,1);
    put(file," : "); put(file,fltval,3);
    if res
     then put_line(file,"  success");
     else put_line(file,"  failure");
    end if;
    Multprec_Complex_Vectors.Clear(eva);
    Clear(val);
    return res;
  end In_Subspace;

  function On_Component
               ( file : file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys; ind : natural;
                 projp : Multprec_Complex_Vectors.Vector;
                 tol : double_float ) return boolean is

  -- DESCRIPTION :
  --   Auxiliary routine for the on component test for solutions.
  --   The projp vector is the resulting vector after projection.

    res : boolean := false;
    eva : Complex_Number;
    fltacc : Floating_Number;
    tmp : double_float;

  begin
    for i in 1..(ind-1) loop
      if p(i) /= Null_Poly
       then eva := Eval(p(i),projp);
            fltacc := AbsVal(eva);
            tmp := Trunc(fltacc);
            Clear(fltacc);
            put(file,"Residual in membership test at component ");
            put(file,i,1); put(file," : ");
            put(file,tmp,3,2,3);
            if tmp < tol
             then res := true;
                  put_line(file,"  success");
             else put_line(file,"  failure");
            end if;
      end if;
      exit when res;
    end loop;
    return res;
  end On_Component;

  function On_Component
               ( file : file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys; ind : natural;
                 sols : Standard_Complex_Solutions.Solution_List;
                 hyp : Multprec_Complex_VecVecs.VecVec;
                 level,size : natural; tol : double_float )
               return boolean is

    res : boolean;
    use Standard_Complex_Solutions;

  begin
    if ind = 1
     then res := false;
     else declare
            point : constant Standard_Complex_Vectors.Vector
                  := Get(sols,ind).v;
            projp : Multprec_Complex_Vectors.Vector(1..level+1)
                  := Evaluate(hyp,point,level+1);
          begin
            res := On_Component(file,p,ind,projp,tol);
            Multprec_Complex_Vectors.Clear(projp);
          end;
    end if;
    return res;
  end On_Component;

  function On_Component
               ( file : file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys; ind : natural;
                 sols : Multprec_Complex_Solutions.Solution_List;
                 hyp : Multprec_Complex_VecVecs.VecVec;
                 level,size : natural; tol : double_float )
               return boolean is

    res : boolean;
    use Multprec_Complex_Solutions;

  begin
    if ind = 1
     then res := false;
     else declare
            point : constant Multprec_Complex_Vectors.Vector
                  := Get(sols,ind).v;
            extpt : Multprec_Complex_Vectors.Vector(point'range);
            projp : Multprec_Complex_Vectors.Vector(1..level+1);
          begin
            Multprec_Complex_Vectors.Copy(point,extpt);
            Set_Size(extpt,2*size);
            projp := Evaluate(hyp,extpt,level+1);
            res := On_Component(file,p,ind,projp,tol);
            Multprec_Complex_Vectors.Clear(projp);
            Multprec_Complex_Vectors.Clear(extpt);
          end;
    end if;
    return res;
  end On_Component;

  function Evaluate ( p : Multprec_Complex_Polynomials.Poly;
                      x : Standard_Complex_Vectors.Vector;
                      level : natural;
                      hyp : Multprec_Complex_VecVecs.VecVec;
                      pivots : Standard_Natural_Vectors.Link_to_Vector;
                      base : Multprec_Complex_VecVecs.Link_to_VecVec;
                      basecard : natural ) return Complex_Number is

  -- DESCRIPTION :
  --   Returns p(r(x)), with r the suitable projection.

    use Standard_Natural_Vectors;
    use Multprec_Complex_Vectors;
    use Multprec_Complex_VecVecs;

    res : Complex_Number;
    projx : Multprec_Complex_Vectors.Vector(1..level+1);

  begin
    if pivots = null
     then projx := Evaluate(hyp,x,level+1);
          res := Eval(p,projx);
     else declare
            dim : constant natural := pivots'last+level;
            restx : constant Standard_Complex_Vectors.Vector
                  := Remove_Variables(x,level,dim,pivots.all);
            mprestx : Multprec_Complex_Vectors.Vector(restx'range);
          begin
            if base = null
             then projx := Evaluate(hyp,restx,level+1);
                  res := Eval(p,projx);
             else mprestx := Create(restx);
                  projx := Intersect(hyp(1..basecard),base(1..basecard),
                                     mprestx,level+1);
                  res := Eval(p,projx);
                  Multprec_Complex_Vectors.Clear(mprestx);
            end if;
          end;
     end if;
     Multprec_Complex_Vectors.Clear(projx);
     return res;
  end Evaluate;

  function Evaluate ( p : Multprec_Complex_Polynomials.Poly;
                      x : Multprec_Complex_Vectors.Vector;
                      level : natural;
                      hyp : Multprec_Complex_VecVecs.VecVec;
                      pivots : Standard_Natural_Vectors.Link_to_Vector;
                      base : Multprec_Complex_VecVecs.Link_to_VecVec;
                      basecard : natural ) return Complex_Number is

  -- DESCRIPTION :
  --   Returns p(r(x)), with r the suitable projection.

    use Standard_Natural_Vectors;
    use Multprec_Complex_Vectors;
    use Multprec_Complex_VecVecs;

    res : Complex_Number;
    projx : Multprec_Complex_Vectors.Vector(1..level+1);

  begin
    if pivots = null
     then projx := Evaluate(hyp,x,level+1);
          res := Eval(p,projx);
     else declare
            dim : constant natural := pivots'last+level;
            restx : Multprec_Complex_Vectors.Vector(1..dim)
                  := Remove_Variables(x,level,dim,pivots.all);
          begin
            if base = null
             then projx := Evaluate(hyp,restx,level+1);
                  res := Eval(p,projx);
             else projx := Intersect(hyp(1..basecard),base(1..basecard),
                                     restx,level+1);
                  res := Eval(p,projx);
            end if;
            Multprec_Complex_Vectors.Clear(restx);
          end;
     end if;
     Multprec_Complex_Vectors.Clear(projx);
     return res;
  end Evaluate;

  function On_Component
               ( file : file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys; ind : natural;
                 subspaces : Multprec_Complex_Poly_Systems.Array_of_Poly_Sys;
                 pivots : Standard_Natural_VecVecs.VecVec;
                 basepts : Multprec_Complex_VecVecs.Array_of_VecVecs;
                 basecard : Standard_Natural_Vectors.Vector;
                 sols : Standard_Complex_Solutions.Solution_List;
                 hyp : Multprec_Complex_VecVecs.VecVec;
                 level,size : natural; tol : double_float )
               return boolean is

    res,sub : boolean;
    use Standard_Natural_Vectors;
    use Standard_Complex_Solutions;

  begin
    if ind = 1
     then res := false;
     else declare
            point : constant Standard_Complex_Vectors.Vector
                  := Get(sols,ind).v;
            eva : Complex_Number;
            fltacc : Floating_Number;
            tmp : double_float;
          begin
            res := false;
            for i in 1..(ind-1) loop
              if p(i) /= Null_Poly
               then
                 if pivots(i) = null
                  then sub := true;
                  else sub := In_Subspace(file,subspaces(i).all,i,point,tol);
                 end if;
                 if sub
                  then eva := Evaluate(p(i),point,level,hyp,pivots(i),
                                       basepts(i),basecard(i));
                       fltacc := AbsVal(eva);
                       tmp := Trunc(fltacc);
                       Clear(fltacc);
                       put(file,"Residual in membership test at component ");
                       put(file,i,1); put(file," : ");
                       put(file,tmp,3,2,3);
                       if tmp < tol
                        then res := true;
                             put_line(file,"  success");
                        else put_line(file,"  failure");
                       end if;
                 end if;
              end if;
              exit when res;
            end loop;
          end;
    end if;
    return res;
  end On_Component;

  function On_Component
               ( file : file_type;
                 p : Multprec_Complex_Poly_Systems.Poly_Sys; ind : natural;
                 subspaces : Multprec_Complex_Poly_Systems.Array_of_Poly_Sys;
                 pivots : Standard_Natural_VecVecs.VecVec;
                 basepts : Multprec_Complex_VecVecs.Array_of_VecVecs;
                 basecard : Standard_Natural_Vectors.Vector;
                 sols : Multprec_Complex_Solutions.Solution_List;
                 hyp : Multprec_Complex_VecVecs.VecVec;
                 level,size : natural; tol : double_float )
               return boolean is

    res,sub : boolean;
    use Standard_Natural_Vectors;
    use Multprec_Complex_Solutions;

  begin
    if ind = 1
     then res := false;
     else declare
            point : constant Multprec_Complex_Vectors.Vector
                  := Get(sols,ind).v;
            extpt : Multprec_Complex_Vectors.Vector(point'range);
            eva : Complex_Number;
            fltacc : Floating_Number;
            tmp : double_float;
          begin
            res := false;
            Multprec_Complex_Vectors.Copy(point,extpt);
            Set_Size(extpt,2*size);
            for i in 1..(ind-1) loop
              if p(i) /= Null_Poly
               then
                 if pivots(i) = null
                  then sub := true;
                  else sub := In_Subspace(file,subspaces(i).all,i,extpt,tol);
                 end if;
                 if sub
                  then eva := Evaluate(p(i),extpt,level,hyp,pivots(i),
                                       basepts(i),basecard(i));
                       fltacc := AbsVal(eva);
                       tmp := Trunc(fltacc);
                       Clear(fltacc);
                       put(file,"Residual in membership test at component ");
                       put(file,i,1); put(file," : ");
                       put(file,tmp,3,2,3);
                       if tmp < tol
                        then res := true;
                             put_line(file,"  success");
                        else put_line(file,"  failure");
                       end if;
                 end if;
              end if;
              exit when res;
            end loop;
          end;
    end if;
    return res;
  end On_Component;

end Multprec_Membership_Tests;
