with integer_io;                        use integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Complex_Numbers_io;       use Standard_Complex_Numbers_io;
with Standard_Natural_Vectors;
with Standard_Complex_Vectors;          use Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_VecVecs;          use Standard_Complex_VecVecs;
with Standard_Complex_Matrices;         use Standard_Complex_Matrices;
with Standard_Complex_Linear_Solvers;   use Standard_Complex_Linear_Solvers;
with Standard_Random_Numbers;           use Standard_Random_Numbers;
with Standard_Random_Vectors;           use Standard_Random_Vectors;
with Standard_Random_Matrices;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Poly_SysFun;      use Standard_Complex_Poly_SysFun;
with Standard_Complex_Jaco_Matrices;    use Standard_Complex_Jaco_Matrices;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Standard_Continuation_Data;        use Standard_Continuation_Data;
with Continuation_Parameters;
with Drivers_for_Poly_Continuation;     use Drivers_for_Poly_Continuation;
with Witness_Points,Witness_Points_io;  use Witness_Points,Witness_Points_io;
with Standard_Plane_Representations;    use Standard_Plane_Representations;
with Standard_Moving_Planes;            use Standard_Moving_Planes;
with Standard_Intrinsic_Solutions;      use Standard_Intrinsic_Solutions;
with Standard_Intrinsic_Newton;         use Standard_Intrinsic_Newton;
with Standard_Intrinsic_Continuation;   use Standard_Intrinsic_Continuation;
with Extrinsic_Diagonal_Homotopies;     use Extrinsic_Diagonal_Homotopies;

package body Standard_Cascading_Planes is

  function Project ( x : Matrix; h : natural ) return Matrix is

    res : Matrix(x'range(1),x'range(2));

  begin
    for i in x'first(1)..x'first(1)+h-1 loop
      for j in x'range(2) loop
        res(i,j) := x(i,j);
      end loop;
    end loop;
    for i in x'first(1)+h..x'last(1) loop
      for j in x'range(2) loop
        res(i,j) := Create(0.0);
      end loop;
    end loop;
    return res;
  end Project;

  function Project ( x : Vector; h : natural ) return Vector is

    res : Vector(x'range);

  begin
    for i in x'first(1)..x'first(1)+h-1 loop
      res(i) := x(i);
    end loop;
    for i in x'first(1)+h..x'last(1) loop
      res(i) := Create(0.0);
    end loop;
    return res;
  end Project;

  function Double_to_Diagonal ( A : Matrix ) return Matrix is

    n : constant natural := A'last(2);
    res : Matrix(A'range(1),1..2*n);

  begin
    for i in A'range(1) loop
      for j in A'range(2) loop
        res(i,j) := A(i,j);
        res(i,n+j) := -A(i,j);
      end loop;
    end loop;
    return res;
  end Double_to_Diagonal;

  function Compute_Offset ( C : Matrix; d : Vector ) return Vector is

    n : constant natural := d'last;
    n2 : constant natural := 2*n;
    res : Vector(C'range(2));
    mat : Matrix(C'range(2),C'range(2));
    piv : Standard_Natural_Vectors.Vector(C'range(2));
    inf : natural;

  begin
    for i in 1..n loop
      for j in 1..n loop
        if i = j
         then mat(i,j) := Create(1.0);
              mat(i,j+n) := Create(-1.0);
         else mat(i,j) := Create(0.0);
              mat(i,j+n) := Create(0.0);
        end if;
      end loop;
      for j in C'range(2) loop
        mat(i+n,j) := C(i,j);
      end loop;
      res(i) := Create(0.0);
      res(i+n) := -d(i);
    end loop;
    lufac(mat,n2,piv,inf);
    lusolve(mat,n2,piv,res);
    return res;
  end Compute_Offset;

  procedure Shift_Offset ( p : in out Matrix; b : in Vector ) is
  begin
    for i in p'range(1) loop
      p(i,0) := b(i);
    end loop;
  end Shift_Offset;

  procedure Evaluate ( file : in file_type;
                       equ : in Matrix; v : in Vector;
                       res : out double_float ) is

    y : Complex_Number;

  begin
    res := 0.0;
    for i in equ'range loop
      put(file,"at equation ");
      put(file,i,1); put(file," : ");
      y := equ(i,0);
      for j in v'range loop
        y := y + equ(i,j)*v(j);
      end loop;
      put(file,y); new_line(file);
      res := res + AbsVal(y);
    end loop;
  end Evaluate;

  procedure Evaluate ( equ : in Matrix; v : in Vector;
                       res : out double_float ) is

    y : Complex_Number;

  begin
    res := 0.0;
    for i in equ'range loop
      y := equ(i,0);
      for j in v'range loop
        y := y + equ(i,j)*v(j);
      end loop;
      res := res + AbsVal(y);
    end loop;
  end Evaluate;

  procedure Evaluate ( file : in file_type;
                       p : in Matrix; g : in Matrix;
                       res : out double_float ) is

    y : Complex_Number;
    rgk : double_float;

  begin
    res := 0.0;
    for k in g'range(2) loop
      rgk := 0.0;
     -- put(file,"Verifying generator ");
     -- put(file,k,1); put_line(file,"...");
      for i in p'range(1) loop
       -- put(file,"  at equation "); put(file,i,1); 
        if k = 0
         then y := p(i,0);
         else y := Create(0.0);
        end if;
        for j in 1..p'last(2) loop
          y := y + p(i,j)*g(j,k);
        end loop;
       -- put(file," : "); put(file,y); new_line(file);
        res := res + AbsVal(y);
        rgk := rgk + AbsVal(y);
      end loop;
      put(file,"Value at generator "); put(file,k,1);
      put(file," : "); put(file,rgk,3); new_line(file);
    end loop;
  end Evaluate;

  procedure Evaluate ( p : in Matrix; g : in Matrix;
                       res : out double_float ) is

    y : Complex_Number;

  begin
    res := 0.0;
    for k in g'range(2) loop
      for i in p'range(1) loop
        if k = 0
         then y := p(i,0);
         else y := Create(0.0);
        end if;
        for j in 1..p'last(2) loop
          y := y + p(i,j)*g(j,k);
        end loop;
        res := res + AbsVal(y);
      end loop;
    end loop;
  end Evaluate;

  procedure Intersect ( e1,e2 : in Matrix; p1,p2 : in out Matrix ) is

    eva0,eva1,eva2,t : Complex_Number;
    v : Vector(p2'range(1));
   -- res : double_float;

  begin
    eva0 := Create(0.0);
    eva1 := Create(0.0);
    for i in 1..e1'last(2) loop
      eva0 := eva0 + e1(1,i)*p2(i,0);
      eva1 := eva1 + e1(1,i)*p2(i,1);
    end loop;
    for k in 2..p2'last(2) loop
      eva2 := Create(0.0);
      for i in 1..e1'last(2) loop
        eva2 := eva2 + e1(1,i)*p2(i,k);
      end loop;
      t := (e1(1,0) + eva0 + eva1)/eva2;
      for i in p2'range(1) loop
        v(i) := p2(i,0) + p2(i,1) - t*p2(i,k);
      end loop;
     -- put_line("Verify if vector satisfies 1st equations : ");
     -- Evaluate(Standard_Output,e1,v,res);
     -- put("  residual : "); put(res,3); new_line;
     -- Evaluate(e1,v,res);
     -- put("Value of vector at 1st plane : "); put(res,3); new_line;
     -- put_line("Verify if vector satisfies 2nd equations : ");
     -- Evaluate(Standard_Output,e2,v,res);
     -- put("  residual : "); put(res,3); new_line;
     -- Evaluate(e2,v,res);
     -- put("Value of vector at 1st plane : "); put(res,3); new_line;
      for i in v'range loop
        p1(i,k-1) := v(i) - p1(i,0);
      end loop;
    end loop;
    for k in 2..p2'last(2) loop
      for i in p2'range(1) loop
        p2(i,k-1) := p1(i,k-1);
      end loop;
    end loop;
    p1 := Orthogonalize(p1);
    p2 := Orthogonalize(p2);
  end Intersect;

  procedure Intersect ( file : in file_type;
                        e1,e2 : in Matrix; p1,p2 : in out Matrix ) is

    eva0,eva1,eva2,t : Complex_Number;
    v : Vector(p2'range(1));
    res : double_float;

  begin
    eva0 := Create(0.0);
    eva1 := Create(0.0);
    for i in 1..e1'last(2) loop
      eva0 := eva0 + e1(1,i)*p2(i,0);
      eva1 := eva1 + e1(1,i)*p2(i,1);
    end loop;
    for k in 2..p2'last(2) loop
      eva2 := Create(0.0);
      for i in 1..e1'last(2) loop
        eva2 := eva2 + e1(1,i)*p2(i,k);
      end loop;
      t := (e1(1,0) + eva0 + eva1)/eva2;
      for i in p2'range(1) loop
        v(i) := p2(i,0) + p2(i,1) - t*p2(i,k);
      end loop;
      put_line(file,"Verify if vector satisfies 1st equations : ");
      Evaluate(file,e1,v,res);
      put(file,"  residual : "); put(file,res,3); new_line(file);
      Evaluate(e1,v,res);
      put(file,"Value of vector at 1st plane : ");
      put(file,res,3); new_line(file);
      put_line(file,"Verify if vector satisfies 2nd equations : ");
      Evaluate(file,e2,v,res);
      put(file,"  residual : "); put(file,res,3); new_line(file);
      Evaluate(e2,v,res);
      put(file,"Value of vector at 1st plane : ");
      put(file,res,3); new_line(file);
      for i in v'range loop
        p1(i,k-1) := v(i) - p1(i,0);
      end loop;
    end loop;
    for k in 2..p2'last(2) loop
      for i in p2'range(1) loop
        p2(i,k-1) := p1(i,k-1);
      end loop;
    end loop;
    p1 := Orthogonalize(p1);
    p2 := Orthogonalize(p2);
  end Intersect;

  function Target_Space ( n,n2,apb,b : natural; dA,BB,CC : Matrix;
                          d : Vector ) return Matrix is

    PC : Matrix(1..n,1..n2) := Project(CC,b);
    BPC : Matrix(1..apb,1..n2) := BB*PC;
    Pd : Vector(1..n) := Project(d,b);
    BPd : Vector(1..apb) := BB*Pd;
    eqs : Matrix(1..apb,0..n2);

  begin
    for i in eqs'range(1) loop
      eqs(i,0) := BPd(i);
    end loop;
    for i in eqs'range(1) loop
      for j in 1..n2 loop
        eqs(i,j) := dA(i,j) + BPC(i,j);
      end loop;
    end loop;
    return eqs;
  end Target_Space;

  function Start_Space ( g1,g2 : Matrix ) return Matrix is

    n : constant natural := g1'last(1);
    n2 : constant natural := 2*n;
    res : Matrix(1..n2,0..g1'last(2)+g2'last(2));

  begin
    for i in 1..n loop
      res(i,0) := g1(i,0);
      res(i+n,0) := g2(i,0);
      for j in 1..g1'last(2) loop
        res(i,j) := g1(i,j);
        res(i+n,j) := Create(0.0);
      end loop;
      for j in 1..g2'last(2) loop
        res(i,g1'last(2)+j) := Create(0.0);
        res(i+n,g1'last(2)+j) := g2(i,j);
      end loop;
    end loop;
    return res;
  end Start_Space;

end Standard_Cascading_Planes;
