package body Standard_Solution_Filters is

-- CRITERIA :

  function Vanishing ( sol : Solution; tol : double_float ) return boolean is
  begin
    return ((abs(sol.err) < tol) or (abs(sol.res) < tol));
  end Vanishing;

  function Zero_Component ( sol : Solution; k : natural;
                            tol : double_float ) return boolean is
  begin
    return (AbsVal(sol.v(k)) < tol);
  end Zero_Component;

  function Regular ( sol : Solution; tol : double_float ) return boolean is
  begin
    return (abs(sol.rco) > tol);
  end Regular;

  function On_Target ( sol : Solution; target : Complex_Number;
                       tol : double_float ) return boolean is
  begin
    return (AbsVal(sol.t-target) < tol);
  end On_Target;

-- FILTERS :

  function Vanishing_Filter ( sols : Solution_List; tol : double_float )
                            return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Vanishing(ls.all,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Vanishing_Filter;

  function Spurious_Filter ( sols : Solution_List; tol : double_float )
                           return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if not Vanishing(ls.all,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Spurious_Filter;

  function Zero_Component_Filter
               ( sols : Solution_List; k : natural; tol : double_float )
               return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Zero_Component(ls.all,k,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Zero_Component_Filter;

  function Free_Component_Filter
               ( sols : Solution_List; k : natural; tol : double_float )
               return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if not Zero_Component(ls.all,k,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Free_Component_Filter;

  function Regular_Filter ( sols : Solution_List; tol : double_float )
                          return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Regular(ls.all,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Regular_Filter;

  function Singular_Filter ( sols : Solution_List; tol : double_float )
                           return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if not Regular(ls.all,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Singular_Filter;

  function On_Target_Filter ( sols : Solution_List; target : Complex_Number;
                              tol : double_float ) return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if On_Target(ls.all,target,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end On_Target_Filter;

  function Off_Target_Filter ( sols : Solution_List; target : Complex_Number;
                               tol : double_float ) return Solution_List is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if not On_Target(ls.all,target,tol)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Off_Target_Filter;

end Standard_Solution_Filters;
