with Standard_Mathematical_Functions;   use Standard_Mathematical_Functions;
with Standard_Plane_Representations;

package body Standard_Moving_Planes is

  function Rotate ( A : Matrix; theta : double_float;
                    i1,i2 : natural ) return Matrix is

    res : Matrix(A'range(1),A'range(2)) := A;
    cot : constant double_float := cos(theta);
    sit : constant double_float := sin(theta);
    c : constant Complex_Number := Create(cot);
    s : constant Complex_Number := Create(sit);

  begin
    for j in 1..A'last(2) loop
      res(i1,j) := c*A(i1,j) - s*A(i2,j);
      res(i2,j) := s*A(i1,j) + c*A(i2,j);
    end loop;
    return res;
  end Rotate;

  function Rotating_Plane ( A : Matrix; i1,i2 : natural;
                            t : Complex_Number ) return Matrix is

    rt : constant double_float := REAL_PART(t);

  begin
    if rt = 0.0 or rt = 1.0
     then return A;
     else return Rotate(A,2.0*PI*rt,i1,i2);
    end if;
  end Rotating_Plane;

  function Moving_Plane ( A,B : Matrix; t : Complex_Number ) return Matrix is

    res : Matrix(A'range(1),A'range(2));

  begin
    if t = Create(0.0)
     then res := A;
     elsif t = Create(1.0)
         then res := B;
         else declare
                tt : Complex_Number := Create(1.0) - t;
              begin
                for i in A'range(1) loop
                  for j in A'range(2) loop
                    res(i,j) := tt*A(i,j) + t*B(i,j);
                  end loop;
                end loop;
              end;
             -- res := Standard_Plane_Representations.Orthogonalize(res);
    end if;
    return res;
  end Moving_Plane;

  function Moving_Plane ( A,B : Matrix; gamma,t : Complex_Number )
                        return Matrix is

    res : Matrix(A'range(1),A'range(2));

  begin
    if t = Create(0.0)
     then res := A;
     elsif t = Create(1.0)
         then res := B;
         else declare
                mt : Complex_Number := Create(1.0) - t;
                tt : Complex_Number := mt + gamma*t*mt;
              begin
                for i in A'range(1) loop
                  for j in A'range(2) loop
                    res(i,j) := tt*A(i,j) + t*B(i,j);
                  end loop;
                end loop;
              end;
             -- res := Standard_Plane_Representations.Orthogonalize(res);
    end if;
    return res;
  end Moving_Plane;

end Standard_Moving_Planes;
