with Standard_Complex_Numbers_Polar;    use Standard_Complex_Numbers_Polar;
with Standard_Natural_Vectors;
with Standard_Complex_Matrices;         use Standard_Complex_Matrices;
with Standard_Complex_Norms_Equals;     use Standard_Complex_Norms_Equals;
with Standard_Complex_Linear_Solvers;   use Standard_Complex_Linear_Solvers;
with Standard_Random_Numbers;           use Standard_Random_Numbers;
with Standard_Random_Vectors;           use Standard_Random_Vectors;
with Standard_Random_Matrices;          use Standard_Random_Matrices;
with Standard_Plane_Representations;    use Standard_Plane_Representations;
with Standard_Point_Coordinates;        use Standard_Point_Coordinates;

package body Standard_Plane_Operations is

  procedure Random_Affine_Plane ( n,k : in natural;
                                  b : out Vector; v : out VecVec ) is
  begin
    b := Random_Vector(1,n);
    for i in v'range loop
      if v(i) = null
       then v(i) := new Standard_Complex_Vectors.Vector(1..n);
      end if;
      v(i).all := Random_Vector(1,n);
    end loop;
  end Random_Affine_Plane;

  function Random_Point ( b : Vector; v : VecVec ) return Vector is

    c : Vector(v'range) := Random_Vector(v'first,v'last);
    res : Vector(b'range) := Affine_Expand(c,b,v);

  begin
    return res;
  end Random_Point;

  function Evaluate ( h,x : Vector ) return Complex_Number is

    res : Complex_Number := h(0) + h(x'range)*x;

  begin
    return res;
  end Evaluate;

  function Evaluate ( h : VecVec; x : Vector ) return Vector is

    res : Vector(h'range);
    
  begin
    for i in h'range loop
      res(i) := Evaluate(h(i).all,x);
    end loop;
    return res;
  end Evaluate;

  function Orthogonalize ( v : Array_of_VecVecs ) return Array_of_VecVecs is

    res : Array_of_VecVecs(v'range);

  begin
    for i in v'range loop
      res(i) := new VecVec'(Orthogonalize(v(i).all));
    end loop;
    return res;
  end Orthogonalize;

  function In_Span ( v : VecVec; x : Standard_Complex_Vectors.Vector;
                     tol : double_float ) return boolean is

    p : Vector(x'range) := x;
    ip : Complex_Number;

  begin
    for i in v'range loop
      ip := Inner_Product(p,v(i).all);
      p := p - ip*v(i).all;
    end loop;
    return (Max_Norm(p) <= tol);
  end In_Span;

  procedure Affine_Orthonormal_Basis
              ( n,k : in natural; slices : in VecVec;
                b : out Vector; v,w : out VecVec ) is

    eqs : VecVec(slices'range);

  begin
    for i in 1..k loop
      eqs(i) := new Standard_Complex_Vectors.Vector(0..n);
      for j in 0..n loop
        eqs(i)(j) := slices(i)(j);
      end loop;
    end loop;
    Generators(n,k,eqs,b,v);
    w := Orthogonalize(v);
  end Affine_Orthonormal_Basis;

  function Truncate ( v : in VecVec; n : in natural ) return VecVec is

    res : VecVec(v'range);

  begin
    for i in v'range loop
      if v(i) /= null
       then res(i) := new Standard_Complex_Vectors.Vector'(v(i)(1..n));
      end if;
    end loop;
    return res;
  end Truncate;

  function Truncate ( v : in Array_of_VecVecs; n : in natural )
                    return Array_of_VecVecs is

    res : Array_of_VecVecs(v'range);

  begin
    for i in v'range loop
      if v(i) /= null
       then res(i) := new VecVec'(Truncate(v(i).all,n));
      end if;
    end loop;
    return res;
  end Truncate;

end Standard_Plane_Operations;
