with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Complex_Vectors;          use Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_Norms_Equals;     use Standard_Complex_Norms_Equals;
with Standard_Complex_VecVecs;          use Standard_Complex_VecVecs;
with Standard_Complex_VecVecs_io;       use Standard_Complex_VecVecs_io;
with Standard_Complex_Matrices;         use Standard_Complex_Matrices;
with Standard_Complex_Matrices_io;      use Standard_Complex_Matrices_io;
with Standard_Random_Numbers;           use Standard_Random_Numbers;
with Standard_Random_Vectors;           use Standard_Random_Vectors;
with Standard_Complex_Singular_Values;  use Standard_Complex_Singular_Values;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Poly_SysFun;      use Standard_Complex_Poly_SysFun;
with Standard_Complex_Jaco_Matrices;    use Standard_Complex_Jaco_Matrices;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Standard_Continuation_Data;        use Standard_Continuation_Data;
with Continuation_Parameters;
with Increment_and_Fix_Continuation;    use Increment_and_Fix_Continuation;
with Drivers_for_Poly_Continuation;     use Drivers_for_Poly_Continuation;
with Standard_Plane_Representations;    use Standard_Plane_Representations;
with Standard_Moving_Planes;            use Standard_Moving_Planes;
with Standard_Intrinsic_Solutions;      use Standard_Intrinsic_Solutions;
with Witness_Points;                    use Witness_Points;
with Witness_Points_io;                 use Witness_Points_io;
with Standard_Point_Coordinates;        use Standard_Point_Coordinates;
with Standard_Intrinsic_Newton;
with Standard_Intrinsic_Trackers;       use Standard_Intrinsic_Trackers;
with Standard_Intrinsic_Continuation;   use Standard_Intrinsic_Continuation;

procedure ts_itrack is

-- DESCRIPTION :
--   Tracking paths with intrinsic coordinates, using the standard
--   path tracking routines in PHCpack.

  procedure Set_Continuation_Parameter
              ( sa : in out Solu_Info_Array; vt : in Complex_Number ) is
  begin
    for i in sa'range loop
      sa(i).sol.t := vt;
    end loop;
  end Set_Continuation_Parameter;

  function Random_Plane ( n,k : natural ) return Matrix is

  -- DESCRIPTION :
  --   Returns a matrix representing a k-plane in n-space.

    res : Matrix(1..n,0..k);

  begin
    for i in 1..n loop
      for j in 0..k loop
        res(i,j) := Random1;
      end loop;
    end loop;
    return Standard_Plane_Representations.Orthogonalize(res);
  end Random_Plane;

  procedure Call_Tracker ( f : in Poly_Sys; sols : in out Solution_List;
                           p : in Matrix ) is

    ef : Eval_Poly_Sys := Create(f);
    jm : Jaco_Mat(f'range,p'range(1)) := Create(f);
    jf : Eval_Jaco_Mat := Create(jm);
    tp : Matrix(p'range(1),p'range(2));
    oc : natural;
    output : boolean;
    wrk : Solution_List;
    ans : character;

    function H ( x : Vector; t : Complex_Number ) return Vector is

      mp : Matrix(p'range(1),p'range(2)) := Moving_Plane(p,tp,t);
      z : Vector(p'range(1)) := Affine_Expand(x,mp);

    begin
      return Eval(ef,z);
    end H;

    function dH1 ( x : Vector; t : Complex_Number ) return Matrix is

      mp : Matrix(p'range(1),p'range(2)) := Moving_Plane(p,tp,t);
      z : Vector(p'range(1)) := Affine_Expand(x,mp);
 
    begin
      return Standard_Intrinsic_Newton.Affine_Eval(jf,mp,z);
    end dH1;

    function dH2 ( x : Vector; t : Complex_Number ) return Vector is

      z1 : Vector(p'range(1)) := Affine_Expand(x,p);
      z2 : Vector(p'range(1)) := Affine_Expand(x,tp);

    begin
      return (Eval(ef,z2) - Eval(ef,z1));
    end dH2;

    procedure R_Cont is new Reporting_Continue(Max_Norm,H,dH2,dH1);
    procedure S_Cont is new Silent_Continue(Max_Norm,H,dH2,dH1);

  begin
    new_line;
    Driver_for_Process_io(Standard_Output,oc);
    output := (oc > 0);
    Continuation_Parameters.Tune(2);
    Set_Continuation_Parameter(sols,Create(0.0));
    loop
      tp := Random_Plane(p'last(1),p'last(2));
      Copy(sols,wrk);
      if output
       then R_Cont(Standard_Output,wrk,false);
       else S_Cont(wrk,false);
      end if;
      put("Do you want another run ? (y/n) "); 
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
    end loop;
  end Call_Tracker;

  procedure Test_Tracker ( n,d,k : in natural;
                           ep : in Poly_Sys; esols : in Solution_List ) is

  -- ON ENTRY :
  --   n        number of variables before embedding;
  --   d        dimension of the solution component;
  --   k        co-dimension of the solution component;
  --   ep       embedded polynomial system.

    p : constant Poly_Sys := Remove_Embedding(ep,d);
    s : VecVec(1..d) := Slices(ep,d);
    eqs : Matrix(1..d,0..n) := Equations_to_Matrix(s,n);
    gen : Matrix(1..n,0..k) := Generators(eqs);
    pla : Matrix(1..n,0..k) := Orthogonalize(gen);
    isols : Solution_List := Project(esols,pla);

  begin
    new_line;
    put_line("The original polynomial system :");
    put(p);
    Call_Tracker(p,isols,pla);
  end Test_Tracker;

  procedure Main is

    ep : Link_to_Poly_Sys;
    sols : Solution_List;
    n,d,k : natural;

  begin
    Standard_Read_Embedding(ep,sols,d);
    n := ep'last;
    new_line;
    put("Dimension of the solution component : "); put(d,1); new_line;
    put("Ambient dimension after embedding   : "); put(n,1); new_line;
    put("Ambient dimension before embedding  : "); put(n-d,1); new_line;
    n := n-d;
    k := n-d;
    put("Co-dimension of the component : "); put(k,1); new_line;
    Test_Tracker(n,d,k,ep.all,sols);
  end Main;

begin
  new_line;
  put_line("Path tracking in intrinsic coordinates...");
  Main;
end ts_itrack;
