with Standard_Random_Numbers;            use Standard_Random_Numbers;
with Standard_Natural_Vectors;
with Standard_Random_Vectors;            use Standard_Random_Vectors;
with Multprec_Complex_Vectors;
with Multprec_Random_Vectors;            use Multprec_Random_Vectors;
with Standard_Complex_Matrices;          use Standard_Complex_Matrices;
with Standard_Random_Matrices;           use Standard_Random_Matrices;
with Standard_Complex_Substitutors;      use Standard_Complex_Substitutors;
with Standard_Embed_Polynomials;         use Standard_Embed_Polynomials;
with Planes_and_Polynomials;             use Planes_and_Polynomials;

package body Witness_Points is

  function Random_Hyperplanes ( k,n : natural )
                              return Standard_Complex_VecVecs.VecVec is

    res : Standard_Complex_VecVecs.VecVec(1..k);

  begin
    for i in res'range loop
      res(i) := new Standard_Complex_Vectors.Vector'(Random_Vector(0,n));
    end loop;
    return res;
  end Random_Hyperplanes;

  function Random_Hyperplanes ( k,n,size : natural )
                              return Multprec_Complex_VecVecs.VecVec is

    res : Multprec_Complex_VecVecs.VecVec(1..k);

  begin
    for i in res'range loop
      res(i) := new Multprec_Complex_Vectors.Vector'(Random_Vector(0,n,size));
    end loop;
    return res;
  end Random_Hyperplanes;

-- MANIPULATION OF POLYNOMIALS :

  function Add_Embedding ( p : Poly; k : natural ) return Poly is

    res : Poly := Add_Variables(p,k);
    n : constant natural := Number_of_Unknowns(p);
    t : Term;

  begin
    t.dg := new Standard_Natural_Vectors.Vector'(1..n+k => 0);
    for i in 1..k loop
      t.cf := Random1;
      t.dg(n+i) := 1;
      Add(res,t);
      t.dg(n+i) := 0;
    end loop;
    Clear(t);
    return res;
  end Add_Embedding;

  function Add_Embedding ( p : Poly_Sys; k : natural ) return Poly_Sys is

    res : Poly_Sys(p'range);

  begin
    for i in p'range loop
      res(i) := Add_Embedding(p(i),k);
    end loop;
    return res;
  end Add_Embedding;

-- OPERATIONS ON POLYNOMIAL SYSTEMS :

  function Add_Slice ( p : Poly_Sys; hyp : Vector ) return Poly_Sys is

    res : Poly_Sys(p'first..p'last+1);

  begin
    res(p'range) := p;
    res(res'last) := Hyperplane(hyp);
    return res;
  end Add_Slice;

  function Remove_Slice ( p : Poly_Sys ) return Poly_Sys is

    res : Poly_Sys(p'range);
    t : Term;

  begin
    res(p'first..p'last-1) := p(p'first..p'last-1);
    t.dg := new Standard_Natural_Vectors.Vector'(p'range => 0);
    t.dg(t.dg'last) := 1;
    t.cf := Create(1.0);
    res(res'last) := Create(t);
    Clear(t.dg);
    return res;
  end Remove_Slice;

  function Eliminate_Slice ( p : Poly_Sys; k,i : natural ) return Poly_Sys is

    res : Poly_Sys(p'first..p'last-1);
    n : constant natural := Number_of_Unknowns(p(k));
    s,srv : Poly;
    t : Term;

  begin
    t.dg := new Standard_Natural_Vectors.Vector'(1..n => 0);
    t.dg(i) := 1;
    t.cf := Coeff(p(k),t.dg);
    s := p(k)-t;
    Min(s);
    srv := Remove_Variable(s,i);
    for j in res'range loop
      if j < k
       then res(j) := Substitute(i,srv,p(j)); 
       else res(j) := Substitute(i,srv,p(j+1));
      end if;
    end loop;
    Clear(t); Clear(s); Clear(srv);
    return res;
  end Eliminate_Slice;

  function Embed ( p : Poly_Sys ) return Poly_Sys is

    res : Poly_Sys(p'range);

  begin
    for i in res'range loop
      res(i) := Add_Variables(p(i),1);
    end loop;
    return res;
  end Embed;

  function Embed ( p : Poly_Sys; gamma : Vector ) return Poly_Sys is

    res : Poly_Sys(p'range);
    n : constant natural := Number_of_Unknowns(p(p'first));
    t : Term;

  begin
    t.dg := new Standard_Natural_Vectors.Vector'(1..n+1 => 0);
    t.dg(n+1) := 1;
    for i in res'range loop
      res(i) := Add_Variables(p(i),1);
      t.cf := gamma(i);
      Add(res(i),t);
    end loop;
    Clear(t);
    return res;
  end Embed;

  function Slice_and_Embed1 ( p : Poly_Sys; hyp : Vector ) return Poly_Sys is

  -- DESCRIPTION :
  --   Adds one random hyperplane to the system and does an embedding
  --   with one new variable.

    sli : Poly_Sys(p'first..p'last+1) := Add_Slice(p,hyp); 
    gam : Vector(p'first..p'last+1) := Random_Vector(1,p'last+1);
    res : Poly_Sys(p'first..p'last+1) := Embed(sli,gam);

  begin
    Standard_Complex_Polynomials.Clear(sli(sli'last));
    return res;
  end Slice_and_Embed1;

  function Slice_and_Embed ( p : Poly_Sys; k : natural ) return Poly_Sys is

    res : Poly_Sys(p'first..p'last+k);
    n : constant natural := p'length;
    t : Term;
    slices : Matrix(1..p'last+k+1,1..k);

  begin
    t.dg := new Standard_Natural_Vectors.Vector'(1..n+k => 0);
    for i in p'range loop
      res(i) := Add_Variables(p(i),k);
      for j in n+1..n+k loop
        t.cf := Random1;
        t.dg(j) := 1;
        Add(res(i),t);
        t.dg(j) := 0;
      end loop;
    end loop;
    Clear(t);
    if k = 1
     then slices := Random_Matrix(p'last+k+1,k);
     else slices := Random_Orthogonal_Matrix(p'last+k+1,k);
    end if;
    for i in 1..k loop
      declare
        hyp : Vector(0..p'last+k);
      begin
        for j in hyp'range loop
          hyp(j) := slices(j+1,i);
        end loop;
        res(p'last+i) := Hyperplane(hyp);
      end;
    end loop;
    return res;
  end Slice_and_Embed;

  function Embed_with_Dummies ( p : Poly_Sys; k : natural ) return Poly_Sys is

    nvars : constant natural := p'length+k;
    res : Poly_Sys(p'first..p'last+k) := Slice_and_Embed(p,k);
    t : Term;

  begin
    t.dg := new Standard_Natural_Vectors.Vector'(1..nvars => 0);
    t.cf := Create(1.0);
    for i in 0..k-1 loop
      t.dg(nvars-i) := 1;
      Clear(res(p'last-i));
      res(p'last-i) := Create(t);
      t.dg(nvars-i) := 0;
    end loop;
    return res;
  end Embed_with_Dummies;

  function Slice_and_Embed ( p : Poly_Sys; k : natural ) 
                           return Array_of_Poly_Sys is

    res : Array_of_Poly_Sys(0..k);
    slices : Matrix(1..p'last+k+1,1..k);

  begin
    if k = 1
     then slices := Random_Matrix(p'last+k+1,1);
     else slices := Random_Orthogonal_Matrix(p'last+k+1,k);
    end if;
    res(0) := new Poly_Sys'(p);
    for i in 1..k loop
      declare
        hyp : Vector(0..res(i-1)'last);
      begin
        for j in hyp'range loop
          hyp(j) := slices(j+1,i);
        end loop;
        res(i) := new Poly_Sys'(Slice_and_Embed1(res(i-1).all,hyp));
      end;
    end loop;
    return res;
  end Slice_and_Embed;

  function Copy_Last_and_Embed ( p : Poly_Sys ) return Poly_Sys is

  -- DESCRIPTION :
  --   Copies the last equation and embeds everything.

    res : Poly_Sys(p'first..p'last+1);
    gam : Vector(p'first..p'last+1) := Random_Vector(1,p'last+1);

  begin
    res(p'range) := p;
    res(res'last) := p(p'last);
    return Embed(res,gam);
  end Copy_Last_and_Embed;

  function Slice_and_Embed ( p : Poly_Sys; k,l : natural )
                           return Array_of_Poly_Sys is

    res : Array_of_Poly_Sys(0..k);

  begin
    res(0) := new Poly_Sys'(p);
    for i in 1..l loop
      declare
        sys : Poly_Sys(p'first..p'last+i)
            := Copy_Last_and_Embed(res(i-1).all);
      begin
        res(i) := new Poly_Sys'(sys);
      end;
    end loop;
    for i in l+1..k loop
      declare
        hyp : Vector(0..res(i-1)'last) := Random_Vector(0,res(i-1)'last);
      begin
        res(i) := new Poly_Sys'(Slice_and_Embed1(res(i-1).all,hyp));
      end;
    end loop;
    return res;
  end Slice_and_Embed;

  function Slices ( p : Poly_Sys; k : natural ) return VecVec is

    res : VecVec(1..k);

  begin
    for i in res'range loop
      res(i) := new Vector'(Polynomial(p(p'last-k+i)));
    end loop;
    return res;
  end Slices;

  function Square ( p : Poly_Sys ) return Poly_Sys is

    n : constant natural := p'length;
    m : constant natural := Number_of_Unknowns(p(p'first));

  begin
    if n < m
     then declare
            res : Poly_Sys(1..m);
            hyp : Vector(0..m);
          begin
            res(1..n) := p;
            for i in 1..m-n loop
              hyp := Random_Vector(0,m);
              res(n+i) := Hyperplane(hyp);
            end loop;
            return res;
          end;
     elsif n > m
         then declare
                res : Poly_Sys(1..n);
                k : natural := n-m;
                t : Term;
              begin
                t.dg := new Standard_Natural_Vectors.Vector'(1..n => 0);
                for i in 1..n loop
                  res(i) := Add_Variables(p(i),k);
                  for j in 1..k loop
                    t.dg(m+j) := 1; 
                    t.cf := Random;
                    Add(res(i),t);
                    t.dg(m+j) := 0;
                  end loop;
                end loop;
                Clear(t);
                return res;
              end;
         else return p;
    end if;
  end Square;

  function Is_Added_Term ( t : Term; dim : natural ) return boolean is

  -- DESCRIPTION :
  --   This routine tells whether the term was added in the embedding,
  --   i.e.: whether it has only nonzero degrees in the last dim positions.

  begin
    for i in t.dg'first..t.dg'last-dim loop
      if t.dg(i) /= 0
       then return false;        -- term has nonzero original variable
      end if;
    end loop;                    -- term has no nonzero original variable
    for i in t.dg'last-dim+1..t.dg'last loop
      if t.dg(i) /= 0
       then return true;         -- moreover, term has added variable
      end if;
    end loop;
    return false;                -- term is constant term
  end Is_Added_Term;

  function Remove_Embedding ( p : Poly; dim : natural ) return Poly is

    res : Poly := Null_Poly;

    procedure Remove_in_Term ( t : in Term; cont : out boolean ) is

      rt : Term;

    begin
      if not Is_Added_Term(t,dim)
       then rt.cf := t.cf;
            rt.dg := new Standard_Natural_Vectors.Vector'
                           (t.dg(t.dg'first..t.dg'last-dim));
            Add(res,rt);
      end if;
      cont := true;
    end Remove_in_Term;
    procedure Remove_in_Terms is new Visiting_Iterator(Remove_in_Term);

  begin
    Remove_in_Terms(p);
    return res;
  end Remove_Embedding;

  function Remove_Embedding ( p : Poly_Sys; dim : natural ) return Poly_Sys is

    res : Poly_Sys(p'first..p'last-dim);
    k : natural;

  begin
    for i in res'range loop
      res(i) := Remove_Embedding(p(i),dim);
    end loop;
    k := 0;
    while res(res'last-k) = Null_Poly loop
      k := k+1;
      exit when (k = res'length);
    end loop;
    return res(res'first..res'last-k);
  end Remove_Embedding;

  function Complete ( n,k : natural; p : Poly_Sys ) return Poly_Sys is
  begin
    if p'length = n-k
     then return p;
     else declare
            res : Poly_Sys(1..n-k);
            rnd : Complex_Number;
            extra : Poly;
          begin
            for i in 1..n-k loop
              Copy(p(i),res(i));
            end loop;
            for i in n-k+1..p'last loop
              for j in res'range loop
                rnd := Random1;
                extra := rnd*p(i);
                Add(res(j),extra);
                Clear(extra);
              end loop;
            end loop;
            return res;
          end;
    end if;
  end Complete;

-- OPERATIONS ON SOLUTION LISTS :

  function Add_Component ( s : Solution; c : Complex_Number )
                         return Solution is

    res : Solution(s.n+1);

  begin
    res.t := s.t;
    res.m := s.m;
    res.err := s.err;
    res.rco := s.rco;
    res.res := s.res;
    res.v(s.v'range) := s.v;
    res.v(res.n) := c;
    return res;
  end Add_Component;
 
  function Add_Component ( sols : Solution_List; c : Complex_Number )
                         return Solution_List is

    res,res_last,tmp : Solution_List;

  begin
    tmp := sols;
    while not Is_Null(tmp) loop
      Append(res,res_last,Add_Component(Head_Of(tmp).all,c));
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Add_Component;

  function Add_Embedding ( s : Solution; k : natural )
                         return Solution is

    res : Solution(s.n+k);

  begin
    res.t := s.t;
    res.m := s.m;
    res.err := s.err;
    res.rco := s.rco;
    res.res := s.res;
    res.v(s.v'range) := s.v;
    for i in s.n+1..res.n loop
      res.v(i) := Create(0.0);
    end loop;
    return res;
  end Add_Embedding;
 
  function Add_Embedding ( sols : Solution_List; k : natural )
                         return Solution_List is

    res,res_last,tmp : Solution_List;

  begin
    tmp := sols;
    while not Is_Null(tmp) loop
      Append(res,res_last,Add_Embedding(Head_Of(tmp).all,k));
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Add_Embedding;
 
  function Remove_Component ( s : Solution ) return Solution is

    res : Solution(s.n-1);

  begin
    res.t := s.t;
    res.m := s.m;
    res.err := s.err;
    res.rco := s.rco;
    res.res := s.res;
    res.v(s.v'first..s.v'last-1) := s.v(s.v'first..s.v'last-1);
    return res;
  end Remove_Component;

  procedure Remove_Component ( sols : in out Solution_List ) is

    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      declare
        remsol : Solution(ls.n-1) := Remove_Component(ls.all);
      begin
        Clear(ls);
        ls := new Solution'(remsol);
        Set_Head(tmp,ls);
      end;
      tmp := Tail_Of(tmp);
    end loop;
  end Remove_Component;
 
  function Remove_Component ( sols : Solution_List ) return Solution_List is

    res,res_last,tmp : Solution_List;

  begin
    tmp := sols;
    while not Is_Null(tmp) loop
      Append(res,res_last,Remove_Component(Head_Of(tmp).all));
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Remove_Component;

  function Remove_Embedding ( s : Solution; k : natural ) return Solution is

    res : Solution(s.n-k);

  begin
    res.m := s.m;
    res.t := s.t;
    res.err := s.err;
    res.rco := s.rco;
    res.res := s.res;
    res.v(1..s.n-k) := s.v(1..s.n-k);
    return res;
  end Remove_Embedding;

  function Remove_Embedding ( sols : Solution_List; k : natural )
                            return Solution_List is

    res,res_last,tmp : Solution_List;
    ls : Link_to_Solution;

  begin
    tmp := sols;
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      Append(res,res_last,Remove_Embedding(ls.all,k));
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Remove_Embedding;
 
end Witness_Points;
