with integer_io;                         use integer_io;
with Communications_with_User;           use Communications_with_User;
with File_Scanning;                      use File_Scanning;
with Characters_and_Numbers;             use Characters_and_Numbers;
with Symbol_Table,Symbol_Table_io;       use Symbol_Table;
with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Standard_Natural_Vectors;
with Standard_Natural_Vectors_io;        use Standard_Natural_Vectors_io;
with Standard_Integer_Vectors;
with Standard_Integer_Vectors_io;        use Standard_Integer_Vectors_io;
with Standard_Complex_Vectors;
with Standard_Random_Vectors;            use Standard_Random_Vectors;
with Standard_Natural_VecVecs;
with Standard_Complex_VecVecs;
with Multprec_Complex_Vectors;
with Multprec_Complex_VecVecs;
with Multprec_Complex_Vector_Tools;      use Multprec_Complex_Vector_Tools;
with Standard_to_Multprec_Convertors;    use Standard_to_Multprec_Convertors;
with Standard_Complex_Polynomials;
with Standard_Complex_Polynomials_io;    use Standard_Complex_Polynomials_io;
with Standard_Complex_Poly_Systems_io;   use Standard_Complex_Poly_Systems_io;
with Multprec_Complex_Polynomials;
with Multprec_Complex_Polynomials_io;    use Multprec_Complex_Polynomials_io;
with Multprec_Complex_Poly_Systems_io;   use Multprec_Complex_Poly_Systems_io;
with Standard_Complex_Solutions_io;      use Standard_Complex_Solutions_io;
with Permutations;                       use Permutations;
with Permute_Operations;                 use Permute_Operations;               
with Planes_and_Polynomials;             use Planes_and_Polynomials;
with Witness_Points;                     use Witness_Points;

package body Witness_Points_io is

  function Maximum ( a,b : natural ) return natural is

  -- DESCRIPTION :
  --   Returns the maximum of a and b.

  begin
    if a > b
     then return a;
     else return b;
    end if;
  end Maximum;

-- AUXILIARIES TO DETERMINE THE ELIMINATION ORDER :

  procedure Write_Symbol_Table ( file : in file_type ) is
 
  -- DESCRPITION :
  --   Writes the content of the symbol table in the current order.

  begin
    for i in 1..Symbol_Table.Number loop
      Symbol_Table_io.put(file,Symbol_Table.Get(i)); put(file," ");
    end loop;
    new_line;
  end Write_Symbol_Table;
 
  procedure Write_Symbol_Order ( perm : in Permutation ) is
 
  -- DESCRIPTION :
  --   Writes the symbols according to the given permutation.
 
  begin
    for i in 1..Symbol_Table.Number loop
      Symbol_Table_io.put(Symbol_Table.Get(perm(i))); put(" ");
    end loop;
  end Write_Symbol_Order;
 
  procedure Permute_Symbol_Table ( perm : in Permutation ) is
 
  -- DESCRIPTION :
  --   Permutes the symbols in the symbol table.
 
    nb : constant natural := Symbol_Table.Number;
    use Symbol_Table;
    syms : array(1..nb) of Symbol;
 
  begin
    for i in syms'range loop
      syms(i) := Symbol_Table.Get(i);
    end loop;
    for i in syms'range loop
      Symbol_Table.Replace(i,syms(perm(i)));
    end loop;
  end Permute_Symbol_Table;

  procedure Read_Permutation ( perm : out Permutation ) is

    nb : constant natural := Symbol_Table.Number;
    use Symbol_Table;
    sb : Symbol;
    error : boolean;

  begin
    for i in 1..nb loop
      loop
        put("Give symbol at place "); put(i,1); put(" : ");
        Symbol_Table_io.Get(sb);
        put("  Your symbol : "); Symbol_Table_io.put(sb);
        perm(i) := Symbol_Table.Get(sb);     
        error := false;
        if perm(i) = 0
         then put_line(" did not occur before!  Please try again...");
              error := true;
         else for j in 1..i-1 loop
                if perm(j) = perm(i)
                 then error := true;
                      put(" occurs already at new place ");
                      put(j,1); put_line("!  Please try again...");
                end if;
                exit when error;
              end loop;
              if not error
               then put(" was at position "); put(perm(i),1); new_line;
              end if;
        end if;
        exit when not error;
      end loop;
    end loop;
  end Read_Permutation;

  function Interactive_Read_Permutation
             ( nbunk : in natural ) return Standard_Integer_Vectors.Vector is
 
  -- DESCRIPTION :
  --   Returns a permutation, interactively determined by the user.
  --   The loop allows to adjust and to restart.
 
    perm : Standard_Integer_Vectors.Vector(1..nbunk);
    ans : character;
 
  begin
    loop
      new_line;
      put_line("Reading a permutation of the unknowns.");
      Read_Permutation(Permutation(perm));
      put("The permutation : "); put(perm); new_line;
      put("The new order of symbols : ");
      Write_Symbol_Order(Permutation(perm));
      new_line;
      put("Happy with this new order ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans = 'y');
    end loop;
    return perm;
  end Interactive_Read_Permutation;          

  procedure Permute_Solutions
               ( perm : in Permutation;
                 sols : in out Standard_Complex_Solutions.Solution_List ) is
 
  -- DESCRIPTION :
  --   Permutes the solutions in the list.
 
    use Standard_Complex_Solutions;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;
 
  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      ls.v := Permutation(perm)*ls.v;
      Set_Head(tmp,ls);
      tmp := Tail_Of(tmp);
    end loop;
  end Permute_Solutions;

  procedure Permute_Polynomial
              ( perm : in Permutation;
                f : in out Standard_Complex_Polynomials.Poly ) is

    use Standard_Complex_Polynomials;
    res : Poly := Null_Poly;

    procedure Permute_Term ( t : in Term; continue : out boolean ) is

      pt : Term := perm*t;

    begin
      Add(res,pt);
      continue := true;
    end Permute_Term;
    procedure Permute_Terms is new Visiting_Iterator(Permute_Term);

  begin
    Permute_Terms(f);
    Clear(f);
    f := res;
  end Permute_Polynomial;

  procedure Permute_Polynomial_System
              ( perm : in Permutation;
                f : in out Standard_Complex_Poly_Systems.Poly_Sys ) is
  begin
    for i in f'range loop
      Permute_Polynomial(perm,f(i));
    end loop;
  end Permute_Polynomial_System;

-- MANIPULATION OF SYMBOLS TO SHIFT ADDED VARIABLES TO END :

  function First_Added_Symbol ( n : natural ) return natural is

  -- DESCRIPTION :
  --   Returns the place in the symbol table of the first "zz" symbol.
  --   This is extremely important because the program assumes that
  --   these added variables come last and this is not necessarily so
  --   when the embedded system is read from file.
  --   Returns n+1 if there is not symbol starting with "zz".

  begin
    for i in 1..n loop
      declare 
        sb : Symbol := Symbol_Table.Get(i);
      begin
        if sb(1..2) = "zz"
         then return i;
        end if;
      end;
    end loop;
    return n+1;
  end First_Added_Symbol;

  function Count_Added_Symbols ( n : natural ) return natural is

  -- DESCRIPTION :
  --   Returns the number of symbols starting with "zz".

    cnt : natural := 0;

  begin
    for i in 1..n loop
      declare
        sb : Symbol := Symbol_Table.Get(i);
      begin
        if sb(1..2) = "zz"
         then cnt := cnt+1;
        end if;
      end;
    end loop;
    return cnt;
  end Count_Added_Symbols;

  function Swap ( t : Standard_Complex_Polynomials.Term; i,j : natural ) 
                return Standard_Complex_Polynomials.Term is

  -- DESCRIPTION :
  --   Swaps the variables i and j in the term.

    use Standard_Complex_Polynomials;
    res : Term;

  begin
    res.cf := t.cf;
    res.dg := new Standard_Natural_Vectors.Vector'(t.dg.all);
    res.dg(i) := t.dg(j);
    res.dg(j) := t.dg(i);
    return res;
  end Swap;

  procedure Swap ( p : in out Standard_Complex_Polynomials.Poly;
                   i,j : in natural ) is

  -- DESCRIPTION :
  --   Swaps the variables i and j in the polynomial p.

    use Standard_Complex_Polynomials;
    swp : Poly := Null_Poly;

    procedure Swap_in_Term ( t : in Term; cont : out boolean ) is

      nt : Term := Swap(t,i,j);

    begin
      Add(swp,nt);
      cont := true;
    end Swap_in_Term;
    procedure Swap_in_Terms is new Visiting_Iterator(Swap_in_Term);

  begin
    Swap_in_Terms(p);
    Clear(p);
    p := swp;
  end Swap;

  procedure Swap ( p : in out Standard_Complex_Poly_Systems.Poly_Sys;
                   i,j : in natural ) is

  -- DESCRIPTION :
  --   Swaps the variables i and j in every polynomial in the system p.

  begin
    for k in p'range loop
      Swap(p(k),i,j);
    end loop;
  end Swap;

  procedure Swap ( s : in out Standard_Complex_Solutions.Solution;
                   i,j : in natural ) is

  -- DESCRIPTION :
  --   Swaps the values of i and j in the solution vector.

    tmp : Complex_Number := s.v(i);

  begin
    s.v(i) := s.v(j); 
    s.v(j) := tmp;
  end Swap;

  procedure Swap ( sols : in out Standard_Complex_Solutions.Solution_List;
                   i,j : in natural ) is

  -- DESCRIPTION :
  --   Swaps the values of i and j in every solution vector of the list.

    use Standard_Complex_Solutions;
    tmp : Solution_List := sols;

  begin
    while not Is_Null(tmp) loop
      declare
        ls : Link_to_Solution := Head_Of(tmp);
      begin
        Swap(ls.all,i,j);
        Set_Head(tmp,ls);
      end;
      tmp := Tail_Of(tmp);
    end loop;
  end Swap;

  procedure Swap_Symbols ( i,j : in natural ) is

  -- DESCRIPTION :
  --   Swaps the symbols for i and j in the symbol table.

    sbi : Symbol := Symbol_Table.Get(i);
    sbj : Symbol := Symbol_Table.Get(j);

  begin
    Symbol_Table.Replace(i,sbj);
    Symbol_Table.Replace(j,sbi);
  end Swap_Symbols;

  function Is_Added_Term
                ( t : Standard_Complex_Polynomials.Term; k : natural )
                return boolean is

  -- DESCRIPTION :
  --   Returns true all exponents in t.dg are zero, except the k last ones.

    use Standard_Complex_Polynomials;

  begin
    for i in t.dg'first..t.dg'last-k loop
      if t.dg(i) /= 0
       then return false;                  -- we have an original term
      end if;
    end loop;
    for i in t.dg'last-k+1..t.dg'last loop
      if t.dg(i) /= 0
       then return true;                   -- the term was of form c*z
      end if;
    end loop;
    return false;                          -- we found the constant term
  end Is_Added_Term;

  function Remove_Last_Variables
              ( p : Standard_Complex_Polynomials.Poly; k : natural )
              return Standard_Complex_Polynomials.Poly is

  -- DESCRIPTION :
  --   Removes the last k variables from the polynomial p.
  --   Also removes the terms c*z, for every added variable z.

    use Standard_Complex_Polynomials;
    res : Poly := Null_Poly;

    procedure Remove_in_Term ( t : in Term; cont : out boolean ) is

      rt : Term;

    begin
      if not Is_Added_Term(t,k)
       then rt.cf := t.cf;
            rt.dg := new Standard_Natural_Vectors.Vector'
                           (t.dg(t.dg'first..t.dg'last-k));
            Add(res,rt);
            Clear(rt);
      end if;
      cont := true;
    end Remove_in_Term;
    procedure Remove_in_Terms is new Visiting_Iterator(Remove_in_Term);

  begin
    Remove_in_Terms(p);
    return res;
  end Remove_Last_Variables;

  function Remove_Slices
              ( p : Standard_Complex_Poly_Systems.Poly_Sys; k : natural )
              return Standard_Complex_Poly_Systems.Poly_Sys is

  -- DESCRIPTION :
  --   Returns a system with the k slices and k last variables removed.

    res : Standard_Complex_Poly_Systems.Poly_Sys(p'first..p'last-k);

  begin
    for i in res'range loop
      res(i) := Remove_Last_Variables(p(i),k);
    end loop;
    return res;
  end Remove_Slices;

  procedure Set_rco
              ( sols : in out Standard_Complex_Solutions.Solution_List ) is

  -- DESCRIPTION :
  --   Sets the rco of every solution to 1.  This is a patch...

    use Standard_Complex_Solutions;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      ls.rco := 1.0;
      Set_Head(tmp,ls);
      tmp := Tail_Of(tmp);
    end loop;
  end Set_rco;

-- TARGET ROUTINES :

  procedure Add_Slack_Symbols ( k : in natural ) is

  -- DESCRIPTION :
  --   Adds k new symbols of the form "ssk", with k as index.

    sb : Symbol;

  begin
    Symbol_Table.Enlarge(k);
    for i in 1..k loop
      sb := (sb'range => ' ');
      sb(1..2) := "ss";
      declare
        stri : constant string := Convert(i);
      begin
        for j in stri'range loop
          sb(j+2) := stri(j);
        end loop;
      end;
     -- sb(3) := Convert_Decimal(i);
      Symbol_Table.Add(sb);
    end loop;
  end Add_Slack_Symbols;

  procedure Add_Embed_Symbols ( k : in natural ) is

  -- DESCRIPTION :
  --   Adds k new symbols of the form "zzk", with k as index.

    sb : Symbol;

  begin
    Symbol_Table.Enlarge(k);
    for i in 1..k loop
      sb := (sb'range => ' ');
      sb(1..2) := "zz";
      declare
        stri : constant string := Convert(i);
      begin
        for j in stri'range loop
          sb(j+2) := stri(j);
        end loop;
      end;
     -- sb(3) := Convert_Decimal(i);
      Symbol_Table.Add(sb);
    end loop;
  end Add_Embed_Symbols;

  procedure Read_and_Add_Symbol ( i : in natural ) is

  -- DESCRIPTION :
  --   Reads a symbol to denote the i-th variable.

    sb : Symbol;

  begin
    put("Give symbol for variable "); put(i,1); put(" : ");
    sb := (sb'range => ' ');
    Symbol_Table_io.get(sb);
    Symbol_Table.add(sb);
  end Read_and_Add_Symbol;

  procedure Add_Extra_Symbols ( k : in natural ) is

    ns : constant natural := Symbol_Table.Number;

  begin
    put("The current symbols are : ");
    Write_Symbol_Table(Standard_Output);
    Symbol_Table.Enlarge(k);
    if k = 1
     then Read_and_Add_Symbol(ns+1);
     else put("Reading "); put(k,1);
          put_line(" extra symbols ...");
          for i in 1..k loop
            Read_and_Add_Symbol(ns+i);
          end loop;
    end if;
  end Add_Extra_Symbols;

  procedure Sort_Embedded_Symbols
              ( n,d : in natural;
                f : in out Standard_Complex_Poly_Systems.Poly_Sys;
                sols : in out Standard_Complex_Solutions.Solution_List ) is

  -- DESCRIPTION :
  --   Sorts the embedded symbols in ascending order.

    p : Permutation(1..n+d);

  begin
   -- put("Dimension before the embedding : "); put(n,1); new_line;
   -- put("Dimension of the solution set  : "); put(d,1); new_line;
   -- Write_Symbol_Table(Standard_Output);
    for i in 1..n loop
      p(i) := i;
    end loop;
    for i in n+1..n+d loop
      declare
        s : Symbol := Symbol_Table.Get(i);
        k : natural := Convert(s(3..s'last));
      begin
        p(i) := n+k;
      end;
    end loop;
    Permute_Symbol_Table(p);
    Permute_Solutions(p,sols);
    Permute_Polynomial_System(p,f);
   -- Write_Symbol_Table(Standard_Output);
  end Sort_Embedded_Symbols;

  procedure Standard_Read_Embedding
              ( file : in file_type;
                lp : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                sols : in out Standard_Complex_Solutions.Solution_List;
                dim : out natural ) is

    use Standard_Complex_Poly_Systems;

    n,addsym,last : natural;
    found : boolean;

  begin
    get(file,n);
    lp := new Poly_Sys(1..n);
    Symbol_Table.Init(n);
    get(file,lp.all);
    Scan_and_Skip(file,"SOLUTIONS",found);
    if found
     then get(file,sols);
     else new_line;
          Read(sols);
    end if;
    Set_rco(sols);
    dim := Count_Added_Symbols(n);
    addsym := First_Added_Symbol(n);
    last := n;
    while addsym /= n+1-dim loop
      Swap(lp.all,addsym,last);
      Swap(sols,addsym,last);
      Swap_Symbols(addsym,last);
      addsym := First_Added_Symbol(last);
      last := last-1;
    end loop;
    if dim > 1
     then Sort_Embedded_Symbols(n-dim,dim,lp.all,sols);
    end if;
  end Standard_Read_Embedding;

  procedure Standard_Read_Embedding
              ( lp : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                sols : in out Standard_Complex_Solutions.Solution_List;
                dim : out natural ) is

    infile : file_type;

  begin
    new_line;
    put_line("Reading the name of the file for the embedded system.");
    Read_Name_and_Open_File(infile);
    Standard_Read_Embedding(infile,lp,sols,dim);
    close(infile);
  end Standard_Read_Embedding;

  procedure Get_Multprec_System 
              ( stsys : in Standard_Complex_Poly_Systems.Poly_Sys;
                mpsys : in out Multprec_Complex_Poly_Systems.Link_to_Poly_Sys;
                size,dim : in natural ) is

    res : Multprec_Complex_Poly_Systems.Poly_Sys
            (stsys'first..stsys'last-dim);
    op : Standard_Complex_Poly_Systems.Poly_Sys(stsys'first..stsys'last-dim);
    ans : character;
    infile : file_type;
    m : natural;
    n : constant natural := stsys'last;
    newres : Multprec_Complex_Polynomials.Poly;

  begin
    put("Do you wish to read in the multi-precision coefficients? (y/n) "); 
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then put_line("Reading the polynomial system"
                    & " with multi-precision coefficients.");
          Read_Name_and_Open_File(infile);
          get(infile,m,res);
          for i in res'range loop     -- patch!!! #vars = size of symbol table
            for j in 1..dim loop
              newres := Remove_Variable(res(i),n+1-j);
              Multprec_Complex_Polynomials.Copy(newres,res(i));
              Multprec_Complex_Polynomials.Clear(newres);
            end loop;
          end loop;
     else op := Remove_Slices(stsys,dim);
          res := Convert(op);
    end if;
    Set_Size(res,size);
    mpsys := new Multprec_Complex_Poly_Systems.Poly_Sys'(res);
  end Get_Multprec_System;

  procedure Determine_Order
               ( p : in out Standard_Complex_Poly_Systems.Poly_Sys ) is

    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_Systems;
 
    nbunk : constant natural := Number_of_Unknowns(p(p'first));
    perm : Standard_Integer_Vectors.Vector(1..nbunk);
    pp : Poly_Sys(p'range);
    ans : character;
 
  begin
    put("There are "); put(nbunk,1); put(" unknowns, ordered as ");
    Write_Symbol_Table(Standard_Output);
    put("Do you wish to determine a new order ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then perm := Interactive_Read_Permutation(nbunk);
          pp := p*Permutation(perm);
          Permute_Symbol_Table(Permutation(perm));
          Clear(p); p := pp;
    end if;
  end Determine_Order;

  procedure Determine_Order
               ( p : in out Standard_Complex_Poly_Systems.Poly_Sys;
                 sols : in out Standard_Complex_Solutions.Solution_List ) is

    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_Systems;
 
    nbunk : constant natural := Number_of_Unknowns(p(p'first));
    perm : Standard_Integer_Vectors.Vector(1..nbunk);
    pp : Poly_Sys(p'range);
    ans : character;
 
  begin
    put("There are "); put(nbunk,1); put(" unknowns, ordered as ");
    Write_Symbol_Table(Standard_Output);
    put("Do you wish to determine a new order ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then perm := Interactive_Read_Permutation(nbunk);       
          pp := p*Permutation(perm);
          Permute_Symbol_Table(Permutation(perm));
          Clear(p); p := pp;
          Permute_Solutions(Permutation(perm),sols);
    end if;
  end Determine_Order;

  function Square_and_Embed
              ( p : Standard_Complex_Poly_Systems.Poly_Sys; k : natural )
              return Standard_Complex_Poly_Systems.Poly_Sys is

    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_Systems;

    nbequ : constant natural := p'length;
    nbunk : constant natural := Number_of_Unknowns(p(p'first));
    max : constant natural := Maximum(nbequ,nbunk);
    sp : Poly_Sys(1..max) := Square(p);
    ep : Poly_Sys(1..max+k);

  begin
    if nbequ /= nbunk
     then if nbequ > nbunk
           then Add_Slack_Symbols(nbequ-nbunk);
          end if;
    end if;
    if k = 0
     then return sp;
     else Add_Embed_Symbols(k);
          if nbunk > nbequ
           then for i in nbunk-k+1..nbunk loop
                  exit when (i = nbequ);  -- do not wipe out original eqs
                  Clear(sp(i));
                end loop;
          end if;
          ep := Slice_and_Embed(sp,k);
          return ep;
    end if;
  end Square_and_Embed;

end Witness_Points_io;
