with text_io;                            use text_io;
with Standard_Complex_Poly_Systems;
with Multprec_Complex_Poly_Systems;
with Standard_Complex_Solutions;

package Witness_Points_io is

-- DESCRIPTION :
--   This package offers utilities to deal with embeddings.

  procedure Write_Symbol_Table ( file : in file_type );
 
  -- DESCRPITION :
  --   Writes the content of the symbol table in the current order.

  procedure Add_Slack_Symbols ( k : in natural );

  -- DESCRIPTION :
  --   Adds k new symbols of the form "ssi" to the symbol table,
  --   where i is a number ranging from 1 to k.

  procedure Add_Embed_Symbols ( k : in natural );

  -- DESCRIPTION :
  --   Adds k new symbols of the form "zzi" to the symbol table,
  --   where i is a number ranging from 1 to k.

  procedure Add_Extra_Symbols ( k : in natural );

  -- DESCRIPTION :
  --   When variables are missing from the input, simply because
  --   they are free, the user must provide these symbols.
  --   This routine writes the symbol table and prompts the user
  --   for k extra symbols.

  procedure Standard_Read_Embedding
               ( file : in file_type;
                 lp : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                 sols : in out Standard_Complex_Solutions.Solution_List;
                 dim : out natural );
  procedure Standard_Read_Embedding
               ( lp : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                 sols : in out Standard_Complex_Solutions.Solution_List;
                 dim : out natural );

  -- DESCRIPTION :
  --   Reads the embedded system and its solutions from file.
  --   Ensures the added variables (marked with zz symbols) occur last.

  -- ON RETURN :
  --   file      if provided, then the writing starts from that file,
  --             otherwise the user is asked to provide a file;
  --   lp        pointer to the embedded polynomial system;
  --   sols      solutions of lp;
  --   dim       number of added variables in the embedding.

  procedure Get_Multprec_System 
               ( stsys : in Standard_Complex_Poly_Systems.Poly_Sys;
                 mpsys : in out Multprec_Complex_Poly_Systems.Link_to_Poly_Sys;
                 size,dim : in natural );

  -- DESCRIPTION :
  --   Depending on the answer of the user, the mpsys on return is read in
  --   from file or is the conversion of the given stsys.
  --   Note that stsys is an embedded system!

  procedure Determine_Order
               ( p : in out Standard_Complex_Poly_Systems.Poly_Sys );
  procedure Determine_Order
               ( p : in out Standard_Complex_Poly_Systems.Poly_Sys;
                 sols : in out Standard_Complex_Solutions.Solution_List );

  -- DESCRIPTION :
  --   The user can interactively determine the order of the variables.
  --   The effect of this routine is that the symbol table and the
  --   internal presentation of the polynomial system is changed.
  --   The external representation of the system does not change.
  --   The analogue operations are performed on the solution list.

  function Square_and_Embed
               ( p : Standard_Complex_Poly_Systems.Poly_Sys; k : natural )
               return Standard_Complex_Poly_Systems.Poly_Sys;

  -- DESCRIPTION :
  --   Performs squaring, slicing and embedding with k additional slices
  --   and slack variables on the system p.  In case the original system
  --   is underdetermined, then padding with zero equations happens.
  --   For overdetermined systems, the symbol table is extended with
  --   slack variables "ss", and with k embeddings variables "zz".

end Witness_Points_io;
