with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Complex_Numbers_io;       use Standard_Complex_Numbers_io;
with Standard_Complex_Vectors;          use Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Random_Vectors;           use Standard_Random_Vectors;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Polynomials_io;   use Standard_Complex_Polynomials_io;
with Standard_Complex_Poly_Functions;   use Standard_Complex_Poly_Functions;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Hypersurface_Witness_Sets;         use Hypersurface_Witness_Sets;

procedure ts_hypwit is

-- DESCRIPTION :
--   Standalone testing routine to compute a witness set for one polynomial.

  procedure Call_Root_Finder ( p : in Poly ) is

    n : constant natural := Number_of_Unknowns(p);
    ep : Eval_Poly := Create(p);
    b : Vector(1..n) := Random_Vector(1,n);
    v : Vector(1..n) := Random_Vector(1,n);
    d : constant natural := Degree(p);
    t,err,res : Vector(1..d);
    eps : constant double_float := 1.0E-12;
    fail : boolean;
    ans : character;
    nrm,f : double_float;

    function Eval ( x : Vector ) return Complex_Number is
    begin
      return Eval(ep,x);
    end Eval;

    procedure S_Root_Finder is new Silent_Root_Finder1(Eval);
    procedure R_Root_Finder is new Reporting_Root_Finder0(Eval);

  begin
    new_line;
    put("The polynomial : "); put(p); new_line;
    new_line;
    put("Do you want intermediate output ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then R_Root_Finder(Standard_Output,d,eps,d*10,fail,b,v,t,nrm);
     else S_Root_Finder(d,eps,d*10,fail,b,v,t,err,res,nrm);
          put_line("A witness set for the polynomial : ");
          for i in t'range loop
            put(t(i)); put(" : "); f := AbsVal(err(i)); put(f,3);
                       put(" : "); f := AbsVal(res(i)); put(f,3);
            new_line;
          end loop;
          put("The residual norm : "); put(nrm,3); new_line;
    end if;
    if fail
     then put_line("The required accuracy is not reached.");
     else put_line("Witness set computation succeeded.");
    end if;
  end Call_Root_Finder;

  procedure Main is

    lp : Link_to_Poly_Sys;

  begin
    new_line;
    put_line("Computing a witness set for one polynomial...");
    new_line;
    put_line("Reading one polynomial in several variables...");
    get(lp);
    Call_Root_Finder(lp(lp'first));
  end Main;

begin
  Main;
end ts_hypwit;
