with text_io;                            use text_io;
with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Standard_Complex_Poly_Systems;      use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;         use Standard_Complex_Solutions;

package Black_Box_Poly_Continuations is

-- DESCRIPTION :
--   This package provides two procedure for performing polynomial
--   continuation in blackbox mode.  They mainly differ by the fact
--   that the homotopy might be already provided in the input parameter.

  procedure Black_Box_Polynomial_Continuation
                  ( targetfile,startfile,outfile : in file_type;
                    pocotime : out duration );

 -- procedure Black_Box_Polynomial_Continuation
 --                 ( infile,outfile : in file_type; pocotime : out duration );
 -- no longer supported or used...

  procedure Black_Box_Polynomial_Continuation
                  ( p,q : in Poly_Sys; a : in Complex_Number;
                    sols : in out Solution_List;
                    pocotime : out duration );
  procedure Black_Box_Polynomial_Continuation
                  ( p,q : in Poly_Sys; sols : in out Solution_List;
                    pocotime : out duration );

  procedure Black_Box_Polynomial_Continuation
                  ( outfile : in file_type; 
                    p,q : in Poly_Sys; sols : in out Solution_List;
                    pocotime : out duration );
--  procedure Black_Box_Polynomial_Continuation
--                  ( outfile : in file_type; 
--                    p,q : in Poly_Sys; a : in Complex_Number;
--                    sols : in out Solution_List;
--                    pocotime : out duration );

  -- DESCRIPTION :
  --   Performs polynomial continuation with default settings.

  -- REQUIRED :
  --   Files must be opened for input or output.

  -- ON ENTRY :
  --   outfile      file to write the results on;
  --   p            target polynomial system;
  --   q            a start system for solving p;
  --   a            constant to multiply q with in the homotopy;
  --   sols         the solutions of q.

  -- ON RETURN :
  --   sols         the solutions of p;
  --   pocotime     elapsed user cpu time for polyhedral continuation.

end Black_Box_Poly_Continuations;
