with text_io,integer_io;                 use text_io,integer_io;
with Communications_with_User;           use Communications_with_User;
with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Numbers_io;                         use Numbers_io;
with Standard_Complex_Polynomials;       use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems;      use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;   use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Poly_SysFun;       use Standard_Complex_Poly_SysFun;
with Standard_Homotopy;
with Standard_Complex_Solutions;         use Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;      use Standard_Complex_Solutions_io;
with Multprec_Complex_Solutions;
with Projective_Transformations;         use Projective_Transformations;
with Standard_Root_Refiners;             use Standard_Root_Refiners;
with Drivers_for_Poly_Continuation;      use Drivers_for_Poly_Continuation;
with Systems_with_Parameters;            use Systems_with_Parameters;
with Parameter_Homotopy_Continuation;    use Parameter_Homotopy_Continuation;
--with Bye_Bye_Message;

procedure mainpoco ( infilename,outfilename : in string ) is

  procedure Read_System ( filename : in string;
                          infile : out file_type;
                          lp : out Link_to_Poly_Sys ) is
   
  begin
    if filename /= ""
     then Open(infile,in_file,filename);
          get(infile,lp);
    end if;
  exception
    when others => 
      new_line;
      put("Could not open file with name "); put_line(filename);
      lp := null; return;
  end Read_System;

  procedure Refine_Solutions
              ( outft : in file_type; p : in Poly_Sys;
                target : in Complex_Number;
                sols,refsols : in out Solution_List;
                solsfile : in boolean ) is

    epsxa,epsfa,tolsing : constant double_float := 10.0**(-8);
    nb : natural := 0;

  begin
    if Head_Of(sols).n > p'last
     then Affine_Transformation(sols);
    end if;
    if target = Create(1.0)
     then if solsfile
           then Reporting_Root_Refiner
                  (outft,p,sols,refsols,epsxa,epsfa,tolsing,
                   nb,5,false,false);
           else Reporting_Root_Refiner
                  (outft,p,sols,epsxa,epsfa,tolsing,nb,5,
                   false,false);
          end if;
     else declare
            pt : Poly_Sys(p'range);
          begin
            pt := Standard_Homotopy.Eval(target);
            if solsfile
             then Reporting_Root_Refiner
                    (outft,pt,sols,refsols,epsxa,epsfa,tolsing,
                     nb,5,false,false);
             else Reporting_Root_Refiner
                    (outft,pt,sols,epsxa,epsfa,tolsing,
                     nb,5,false,false);
            end if;
            Clear(pt);
         end;
    end if;
  end Refine_Solutions;

  procedure Secant_Homotopy ( p : in Poly_Sys ) is

    solsft,outft : file_type;
    sols,refsols : Solution_List;
    mpsols : Multprec_Complex_Solutions.Solution_List;
    solsfile : boolean;
    len : natural;
    ans : character;
    target : Complex_Number;

  begin
    Create_Output_File(outft,outfilename);
    put(outft,p); new_line(outft);
    new_line;
    put("Do you want the solutions on separate file ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then 
       put_line("Reading the name of the file to write the solutions on.");
       Read_Name_and_Create_File(solsft);
       solsfile := true;
     else
       solsfile := false;
    end if;
    Driver_for_Polynomial_Continuation(outft,p,sols,mpsols,target);
    if Length_Of(sols) > 0
     then Refine_Solutions(outft,p,target,sols,refsols,solsfile);
    end if;
   -- put(outft,Bye_Bye_Message);
    Close(outft);
    if solsfile
     then len := Length_Of(refsols);
          if len > 0
           then put(solsft,len,Head_Of(refsols).n,refsols);
          end if;
          Close(solsft);
    end if;
  end Secant_Homotopy;

  procedure Parameter_Homotopy ( infile : in file_type; p : in Poly_Sys ) is

    outfile : file_type;
    sols : Solution_List;
    nb_equ,nb_unk,nb_par : natural;

  begin
    Read_Solution_Parameters(infile,outfile,p,sols,nb_equ,nb_unk,nb_par);
    Coefficient_Parameter_Homotopy_Continuation
      (outfile,p,sols,nb_equ,nb_unk,nb_par);
  end Parameter_Homotopy;

  procedure Main is

    inft : file_type;
    lp : Link_to_Poly_Sys;

  begin
    Read_System(infilename,inft,lp);
    if lp = null
     then new_line;
          put_line("Reading the target polynomial system...");
          Read_Name_and_Open_File(inft);
          get(inft,lp);
    end if;
    if (Number_of_Unknowns(lp(lp'first)) = lp'last)
     then close(inft);
          Secant_Homotopy(lp.all);
     else Parameter_Homotopy(inft,lp.all);
    end if;
  end Main;

begin
  Main;
end mainpoco;
