with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Multprec_Floating_Vectors;
with Continuation_Parameters;
with Multprec_Continuation_Data;         use Multprec_Continuation_Data;
with Multprec_Path_Trackers;             use Multprec_Path_Trackers;

package body Multprec_IncFix_Continuation is

  tol : constant double_float := 10.0E-14;

  procedure Silent_Continue
               ( sols : in out Solution_List; proj : in boolean;
                 target : in Complex_Number := Create(1) ) is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    patpp : Pred_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endpp : Pred_Pars := Convert(Continuation_Parameters.Create_End_Game);
    patcp : Corr_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endcp : Corr_Pars := Convert(Continuation_Parameters.Create_End_Game);
    pathdir : Multprec_Floating_Vectors.Link_to_Vector;
    errv : Floating_Number;
    order : constant natural := Continuation_Parameters.endext_order;

    procedure Continue_along_Path is
      new Linear_Single_Normal_Silent_Continue(Norm,H,dH,dH);

    procedure Continue_in_End_Game is
      new Linear_Single_Conditioned_Silent_Continue(Norm,H,dH,dH);

  begin
    while not Is_Null(tmp) loop
      declare
        ls : Link_to_Solution := Head_Of(tmp);
        s : Solu_Info := Deep_Create(ls.all);
      begin
        Continue_along_Path(s,target,Create(tol),false,patpp,patcp);
        Continue_in_End_Game(s,target,Create(tol),false,
                             order,pathdir,errv,endpp,endcp);
        Append(res,res_last,Deep_Create(s));
      end;
      tmp := Tail_Of(tmp);
    end loop;
    Copy(res,sols); Clear(res);
  end Silent_Continue;

  procedure Reporting_Continue
               ( file : in file_type; sols : in out Solution_List;
                 proj : in boolean;
                 target : in Complex_Number := Create(1) ) is

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    patpp : Pred_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endpp : Pred_Pars := Convert(Continuation_Parameters.Create_End_Game);
    patcp : Corr_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endcp : Corr_Pars := Convert(Continuation_Parameters.Create_End_Game);
    pathdir : Multprec_Floating_Vectors.Link_to_Vector;
    errv : Floating_Number;
    order : constant natural := Continuation_Parameters.endext_order;

    procedure Continue_along_Path is
      new Linear_Single_Normal_Reporting_Continue(Norm,H,dH,dH);

    procedure Continue_in_End_Game is
      new Linear_Single_Conditioned_Reporting_Continue(Norm,H,dH,dH);

  begin
    while not Is_Null(tmp) loop
      declare
        ls : Link_to_Solution := Head_Of(tmp);
        s : Solu_Info := Deep_Create(ls.all);
      begin
        Continue_along_Path(file,s,target,Create(tol),false,patpp,patcp);
        Continue_in_End_Game(file,s,target,Create(tol),false,
                             order,pathdir,errv,endpp,endcp);
        Append(res,res_last,Deep_Create(s));
      end;
      tmp := Tail_Of(tmp);
    end loop;
    Copy(res,sols); Clear(res);
  end Reporting_Continue;

end Multprec_IncFix_Continuation;
