with integer_io;                        use integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Random_Numbers;           use Standard_Random_Numbers;
with Standard_Complex_Numbers_io;       use Standard_Complex_Numbers_io;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_Matrices;         use Standard_Complex_Matrices;
with Standard_Complex_Matrices_io;      use Standard_Complex_Matrices_io;
with Standard_Complex_Norms_Equals;     use Standard_Complex_Norms_Equals;
with Symbol_Table,Symbol_Table_io;      use Symbol_Table;
with Standard_Complex_Poly_Functions;   use Standard_Complex_Poly_Functions;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Poly_SysFun;      use Standard_Complex_Poly_SysFun;
with Standard_Complex_Jaco_Matrices;    use Standard_Complex_Jaco_Matrices;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Increment_and_Fix_Continuation;    use Increment_and_Fix_Continuation;
with Drivers_for_Poly_Continuation;     use Drivers_for_Poly_Continuation;
with Standard_Root_Refiners;            use Standard_Root_Refiners;
with Systems_with_Parameters;           use Systems_with_Parameters;
with Solutions_with_Parameters;         use Solutions_with_Parameters;

package body Parameter_Homotopy_Continuation is


-- TARGET ROUTINES :

  function Define_Start ( v : Standard_Complex_Vectors.Vector;
                          ip : Standard_Natural_Vectors.Vector )
                        return Standard_Complex_Vectors.Vector is

    res : Standard_Complex_Vectors.Vector(ip'range);

  begin
    for i in ip'range loop
      res(i) := v(ip(i));
    end loop;
    return res;
  end Define_Start;

  function Define_Target ( ip : Standard_Natural_Vectors.Vector )
                         return Standard_Complex_Vectors.Vector is

    res : Standard_Complex_Vectors.Vector(ip'range);

  begin
    put_line("Reading target values for the parameters...");
    for i in res'range loop
      put(" ");
      Symbol_Table_io.put(Symbol_Table.Get(ip(i)));
      put(" : ");
      get(res(i));
    end loop;
    return res;
  end Define_Target;

  procedure Write_Parameter_Values
             ( file : in file_type;
               labels : in Standard_Natural_Vectors.Vector;
               values : in Standard_Complex_Vectors.Vector ) is

  begin
    for i in labels'range loop
      put(file," ");
      Symbol_Table_io.put(file,Symbol_Table.Get(labels(i)));
      put(file," : ");
      put(file,values(i));
      new_line(file);
    end loop;
  end Write_Parameter_Values;

  function Interpolate ( a,b : Standard_Complex_Vectors.Vector;
                         t : Complex_Number )
                       return Standard_Complex_Vectors.Vector is

    res : Standard_Complex_Vectors.Vector(a'range);
    zero : constant Complex_Number := Create(0.0);
    one : Complex_Number := Create(1.0);

  begin
    if t = zero
     then res := a;
     elsif t = one
         then res := b;
         else one := one - t;
              for i in a'range loop
                res(i) := one*a(i) + t*b(i);  
              end loop;
    end if;
    return res;
  end Interpolate;

  function Circulate ( a,b : Standard_Complex_Vectors.Vector;
                       gamma,t : Complex_Number )
                     return Standard_Complex_Vectors.Vector is

    s : Complex_Number := t + gamma*t*(Create(1.0)-t);

  begin
    return Interpolate(a,b,s);
  end Circulate;

  function Differentiate ( a,b : Standard_Complex_Vectors.Vector )
                         return Standard_Complex_Vectors.Vector is

    use Standard_Complex_Vectors;
    res : Vector(a'range) := b-a;

  begin
    return res;
  end Differentiate;

  procedure Parameter_Continuation
              ( file : in file_type;
                n : in natural; p : in Poly_Sys;
                pars : in Standard_Natural_Vectors.Vector;
                vars : in Standard_Natural_Vectors.Vector;
		sols : in out Solution_List; output : in boolean ) is

    ep : Eval_Poly_Sys(p'range) := Create(p);
    jm : Jaco_Mat(p'range,1..n) := Create(p);
    ejm : Eval_Jaco_Mat(p'range,1..n) := Create(jm);
  
    function xvt ( x : Standard_Complex_Vectors.Vector;
                   t : Complex_Number )
                 return Standard_Complex_Vectors.Vector is

    -- DESCRIPTION :
    --   Returns the value of all variables and parameters at
    --   the current value of the continuation parameter t.

      z : Standard_Complex_Vectors.Vector(1..n);
      v : Standard_Complex_Vectors.Vector(pars'range)
        := Evaluate_Parameters(t);

    begin
      for i in pars'range loop
        z(pars(i)) := v(i);
      end loop;
      for i in vars'range loop
        z(vars(i)) := x(i);
      end loop;
      return z;
    end xvt;

    function Eval ( x : Standard_Complex_Vectors.Vector;
                    t : Complex_Number )
                  return Standard_Complex_Vectors.Vector is

      z : Standard_Complex_Vectors.Vector(1..n) := xvt(x,t);

    begin
      return Eval(ep,z);
    end Eval;

    function Diff ( x : Standard_Complex_Vectors.Vector;
                    t : Complex_Number ) return Matrix is

      res : Matrix(p'range,x'range);
      z : Standard_Complex_Vectors.Vector(1..n) := xvt(x,t);

    begin
      for i in p'range loop
        for j in x'range loop
          res(i,j) := Eval(ejm(i,vars(j)),z);
        end loop;
      end loop;
      return res;
    end Diff;

    function Diff_t ( x : Standard_Complex_Vectors.Vector;
                      t : Complex_Number )
                    return Standard_Complex_Vectors.Vector is

      res : Standard_Complex_Vectors.Vector(p'range)
          := (p'range => Create(0.0));
      z : Standard_Complex_Vectors.Vector(1..n) := xvt(x,t);
      d : constant Standard_Complex_Vectors.Vector
        := Differentiate_Parameters(t);

    begin
      for i in p'range loop
        for j in pars'range loop
          res(i) := res(i) + Eval(ejm(i,pars(j)),z)*d(j);
        end loop;
      end loop;
      return res;
    end Diff_t;

    procedure Simulate_Continuation is

    -- DESCRIPTION :
    --   Simulates ten steps in the continuation.

      t : Complex_Number := Create(0.0);
      p : Standard_Complex_Vectors.Vector(pars'range);
      s : Standard_Complex_Vectors.Vector(vars'range)
        := Head_Of(sols).v;
    begin
      for i in 0..10 loop
        put("Parameters at t = ");
        put(t); put_line(" :");
	p := Evaluate_Parameters(t);
        put_line(p);
        put_line("value at current solution : ");
        put_line(Eval(s,t));
        put_line("derivatives at current solution : ");
        put(Diff(s,t));
        t := t + 0.1;
      end loop;
    end Simulate_Continuation;

    procedure Sil_Cont is
      new Silent_Continue(Max_Norm,Eval,Diff_t,Diff);
    procedure Rep_Cont is
      new Reporting_Continue(Max_Norm,Eval,Diff_t,Diff);

  begin
   -- Simulate_Continuation;
    if output
     then Rep_Cont(file,sols,false,Create(1.0));
     else Sil_Cont(sols,false,Create(1.0));
    end if;
    Clear(ep); Clear(jm); Clear(ejm);
  end Parameter_Continuation;

  procedure Call_Standard_Root_Refiner
               ( file : in file_type; p : in Poly_Sys;
                 sols : in out Solution_List ) is

    epsxa,epsfa,tolsing : double_float;
    numit : natural;
    max : constant natural := 5;

  begin
    epsxa := 1.0E-14;
    epsfa := 1.0E-14;
    tolsing := 1.0E-08;
    Reporting_Root_Refiner
      (file,p,sols,epsxa,epsfa,tolsing,numit,max,false,false);
  end Call_Standard_Root_Refiner;

  function Select_Symbols ( s : Array_of_Symbols;
                            v : Standard_Natural_Vectors.Vector ) 
                          return Array_of_Symbols is

    res : Array_of_Symbols(v'range);

  begin
    for i in v'range loop
      res(i) := s(v(i));
    end loop;
    return res;
  end Select_Symbols;

  procedure Coefficient_Parameter_Homotopy_Continuation
              ( file : in file_type;
                p : in Poly_Sys; sols : in Solution_List;
                nb_equ,nb_unk,nb_par : in natural ) is

    ind_par : constant Standard_Natural_Vectors.Vector
            := Define_Parameters(nb_equ,nb_unk,nb_par);
    ind_var : constant Standard_Natural_Vectors.Vector
            := Complement(nb_unk,ind_par);
    start_pars : Standard_Complex_Vectors.Vector(ind_par'range);
    target_pars : Standard_Complex_Vectors.Vector(ind_par'range);
    var_sols,new_sols : Solution_List;
    gamma : Complex_Number := Random1;
    oc : natural;
    output : boolean;
    s : constant Array_of_Symbols := Symbol_Table.Content;
    sv : Array_of_Symbols := Select_Symbols(s,ind_var);
    sp : Poly_Sys(p'range);

    function Eval_Pars ( t : Complex_Number )
                       return Standard_Complex_Vectors.Vector is
    begin
     -- return Interpolate(start_pars,target_pars,t);
      return Circulate(start_pars,target_pars,gamma,t);
    end Eval_Pars;

    function Diff_Pars ( t : Complex_Number )
                       return Standard_Complex_Vectors.Vector is
    begin
      return Differentiate(start_pars,target_pars);
    end Diff_Pars;

    procedure Par_Con is new Parameter_Continuation(Eval_Pars,Diff_Pars);

  begin
   -- put("The indices to the variables :");  put(ind_var); new_line;
   -- put("The indices to the parameters :"); put(ind_par); new_line;
    start_pars := Define_Start(Head_Of(sols).v,ind_par);
    put_line("The start values for the parameters :");
    Write_Parameter_Values(Standard_Output,ind_par,start_pars);
    target_pars := Define_Target(ind_par);
    put_line("The target values for the parameters :");
    Write_Parameter_Values(Standard_Output,ind_par,target_pars);
    new_line(file);
    put_line(file,"THE START VALUES FOR THE PARAMETERS :");
    Write_Parameter_Values(file,ind_par,start_pars);
    put_line(file,"THE TARGET VALUES FOR THE PARAMETERS :");
    Write_Parameter_Values(file,ind_par,target_pars);
    var_sols := Select_Variables(sols,nb_equ,ind_var);
    Symbol_Table.Clear;
    Symbol_Table.Init(sv);
    new_line(file);
    put_line(file,"THE SOLUTIONS : (after selecting variables)");
    put(file,Head_Of(var_sols).n,Length_Of(var_sols),var_sols);
    new_line;
    skip_line;
    Driver_for_Continuation_Parameters(file);
    new_line;
    Driver_for_Process_io(file,oc);
    output := (oc > 0);
    new_line;
    put_line("Starting parameter continuation...");
    put_line("See the output file for results.");
    new_line;
    Par_Con(file,nb_unk,p,ind_par,ind_var,var_sols,output);
    sp := Substitute(p,ind_par,target_pars);
    new_line(file);
    put_line(file,"THE TARGET SYSTEM :");
    put(file,sp'last,sp);
    Call_Standard_Root_Refiner(file,sp,var_sols);
    new_sols := Join_Variables(var_sols,nb_unk,ind_var,ind_par,target_pars);
    Symbol_Table.Clear;
    Symbol_Table.Init(s);
    new_line(file);
    put_line(file,"THE SOLUTIONS : (with target values of parameters)");
    put(file,Head_Of(new_sols).n,Length_Of(new_sols),new_sols);
  end Coefficient_Parameter_Homotopy_Continuation;

end Parameter_Homotopy_Continuation; 
