with Standard_Natural_Vectors;
with Standard_Complex_Vectors;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;

package Solutions_with_Parameters is

-- DESCRIPTION :
--   The two major operations specific to solutions of systems with
--   natural parameters are the separation of the variables from the
--   paramters and the joining of variables with parameters.

  function Select_Variables
              ( s : Solution; nv : natural;
	        v : Standard_Natural_Vectors.Vector )
              return Solution;

  -- DESCRIPTION :
  --   Returns the solution obtained by selecting those nv variables
  --   from s which are indexed by the vector v.

  function Select_Variables
              ( s : Solution_List; nv : natural;
                v : Standard_Natural_Vectors.Vector )
              return Solution_List;

  -- DESCRIPTION :
  --   Returns the solutions obtained by selecting those nv variables
  --   from s which are indexed by the vector v.

  function Join_Variables
              ( s : Solution; n : natural;
                vars,pars : Standard_Natural_Vectors.Vector;
                val_pars : Standard_Complex_Vectors.Vector )
              return Solution;

  -- DESCRIPTION :
  --   Joins the solution and values for the parameters into one
  --   solution of vectors of n entries long.

  -- ON ENTRY :
  --   s        a solution;
  --   n        dimension of the solution on return;
  --   vars     index to the variables of s in the solution on return;
  --   pars     index to the parameters in the solution on return;
  --   val_pars contains the values for the parameters.

  function Join_Variables
              ( s : Solution_List; n : natural;
                vars,pars : Standard_Natural_Vectors.Vector;
                val_pars : Standard_Complex_Vectors.Vector )
              return Solution_List;

  -- DESCRIPTION :
  --   Joins the solutions and values for the parameters into a
  --   list of solutions.

end Solutions_with_Parameters;
