with text_io,integer_io;                 use text_io,integer_io;
with Communications_with_User;           use Communications_with_User;
with File_Scanning;                      use File_Scanning;
with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Numbers;
with Standard_Random_Numbers;            use Standard_Random_Numbers;
with Multprec_Floating_Numbers;          use Multprec_Floating_Numbers;
with Multprec_Complex_Numbers;
with Multprec_Random_Numbers;            use Multprec_Random_Numbers;
with Standard_Floating_Vectors;
with Standard_Complex_Norms_Equals;      use Standard_Complex_Norms_Equals;
with Multprec_Floating_Vectors;
with Multprec_Complex_Norms_Equals;      use Multprec_Complex_Norms_Equals;
with Symbol_Table;
with Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;   use Standard_Complex_Poly_Systems_io;
with Standard_to_Multprec_Convertors;    use Standard_to_Multprec_Convertors;
with Multprec_Complex_Poly_Systems;
with Multprec_Complex_Poly_Systems_io;   use Multprec_Complex_Poly_Systems_io;
with Standard_Homotopy;
with Multprec_Homotopy;
with Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;      use Standard_Complex_Solutions_io;
with Multprec_Complex_Solutions;
with Multprec_Complex_Solutions_io;      use Multprec_Complex_Solutions_io;
with Continuation_Parameters;
with Standard_Continuation_Data;
with Standard_Path_Trackers;
with Multprec_Continuation_Data;
with Multprec_Path_Trackers;
with Drivers_for_Poly_Continuation;      use Drivers_for_Poly_Continuation;

procedure ts_path is

-- DESCRIPTION :
--   Test facility for the path trackers.

  tol : constant double_float := 10.0E-14;

  procedure Standard_Read_Homotopy
              ( lp,lq : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                qsols : in out Standard_Complex_Solutions.Solution_List ) is

  -- DESCRIPTION :
  --   Reads the target system lp, start system lq and start solutions qsols,
  --   for standard complex numbers.

    use Standard_Complex_Poly_Systems;

    n : natural;
    targetfile,startfile : file_type;
    found : boolean;

  begin
    new_line;
    put_line("Reading the name of the file for the target system.");
    Read_Name_and_Open_File(targetfile);
    get(targetfile,n);
    lp := new Poly_Sys(1..n);
    Symbol_Table.Init(n);
    get(targetfile,lp.all);
    Close(targetfile);
    lq := new Poly_Sys(lp'range);
    new_line;
    put_line("Reading the name of the file for the start system.");
    Read_Name_and_Open_File(startfile);
    get(startfile,n,lq.all);
    Scan_and_Skip(startfile,"SOLUTIONS",found);
    if found
     then get(startfile,qsols);
     else new_line;
          Read(qsols);
    end if;
    Close(startfile);
  end Standard_Read_Homotopy;

  procedure Multprec_Read_Homotopy
              ( lp,lq : in out Multprec_Complex_Poly_Systems.Link_to_Poly_Sys;
                qsols : in out Multprec_Complex_Solutions.Solution_List ) is

  -- DESCRIPTION :
  --   Reads the target system lp, start system lq and start solutions qsols,
  --   for multi-precision numbers.

    n : natural;
    targetfile,startfile : file_type;
    found : boolean;
    stlp,stlq : Standard_Complex_Poly_Systems.Link_to_Poly_Sys;

  begin
    new_line;
    put_line("Reading the name of the file for the target system.");
    Read_Name_and_Open_File(targetfile);
    get(targetfile,n);
    lp := new Multprec_Complex_Poly_Systems.Poly_Sys(1..n);
    stlp := new Standard_Complex_Poly_Systems.Poly_Sys(1..n);
    Symbol_Table.Init(n);
   -- get(targetfile,lp.all);
    get(targetfile,stlp.all);
    lp.all := Convert(stlp.all);
    Close(targetfile);
    new_line;
    put_line("Reading the name of the file for the start system.");
    Read_Name_and_Open_File(startfile);
    lq := new Multprec_Complex_Poly_Systems.Poly_Sys(1..n);
    stlq := new Standard_Complex_Poly_Systems.Poly_Sys(1..n);
   -- get(startfile,n,lq.all);
    get(startfile,n,stlq.all);
    lq.all := Convert(stlq.all);
    Scan_and_Skip(startfile,"SOLUTIONS",found);
    if found
     then get(startfile,qsols);
     else new_line;
          Read(qsols);
    end if;
    Standard_Complex_Poly_Systems.Clear(stlp);
    Standard_Complex_Poly_Systems.Clear(stlq);
    Close(startfile);
  end Multprec_Read_Homotopy;

  procedure Call_Standard_Path_Trackers
              ( file : in file_type;
                sols : in out Standard_Complex_Solutions.Solution_List ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Solutions;
    use Standard_Continuation_Data;
    use Standard_Path_Trackers;

    tmp : Solution_List := sols;
    patpp : Pred_Pars := Continuation_Parameters.Create_for_Path;
    endpp : Pred_Pars := Continuation_Parameters.Create_End_Game;
    patcp : Corr_Pars := Continuation_Parameters.Create_for_Path;
    endcp : Corr_Pars := Continuation_Parameters.Create_End_Game;
    pathdir : Standard_Floating_Vectors.Link_to_Vector;
    errv : double_float;
    order : constant natural := Continuation_Parameters.endext_order;

    procedure Continue_along_Path is
      new Linear_Single_Normal_Reporting_Continue
            (Max_Norm,Standard_Homotopy.Eval,
                      Standard_Homotopy.Diff,Standard_Homotopy.Diff);

    procedure Continue_in_End_Game is
      new Linear_Single_Conditioned_Reporting_Continue
            (Max_Norm,Standard_Homotopy.Eval,
                      Standard_Homotopy.Diff,Standard_Homotopy.Diff);

  begin
    while not Is_Null(tmp) loop
      declare
        s : Solu_Info := Shallow_Create(Head_Of(tmp));
      begin
        Continue_along_Path(file,s,Create(1.0),tol,false,patpp,patcp);
        Continue_in_End_Game(file,s,Create(1.0),tol,false,
                             order,pathdir,errv,endpp,endcp);
      end;
      tmp := Tail_Of(tmp);
    end loop;
  end Call_Standard_Path_Trackers;

  procedure Call_Multprec_Path_Trackers
              ( file : in file_type;
                sols : in out Multprec_Complex_Solutions.Solution_List ) is

    use Multprec_Complex_Numbers;
    use Multprec_Complex_Solutions;
    use Multprec_Continuation_Data;
    use Multprec_Path_Trackers;

    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    patpp : Pred_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endpp : Pred_Pars := Convert(Continuation_Parameters.Create_End_Game);
    patcp : Corr_Pars := Convert(Continuation_Parameters.Create_for_Path);
    endcp : Corr_Pars := Convert(Continuation_Parameters.Create_End_Game);
    pathdir : Multprec_Floating_Vectors.Link_to_Vector;
    errv : Floating_Number;
    order : constant natural := Continuation_Parameters.endext_order;

    procedure Continue_along_Path is
      new Linear_Single_Normal_Reporting_Continue
            (Max_Norm,Multprec_Homotopy.Eval,
                      Multprec_Homotopy.Diff,Multprec_Homotopy.Diff);

    procedure Continue_in_End_Game is
      new Linear_Single_Conditioned_Reporting_Continue
            (Max_Norm,Multprec_Homotopy.Eval,
                      Multprec_Homotopy.Diff,Multprec_Homotopy.Diff);

  begin
    while not Is_Null(tmp) loop
      declare
        ls : Link_to_Solution := Head_Of(tmp);
        s : Solu_Info := Deep_Create(ls.all);
      begin
        Continue_along_Path(file,s,Create(1),Create(tol),false,patpp,patcp);
        Continue_in_End_Game(file,s,Create(1),Create(tol),false,
                             order,pathdir,errv,endpp,endcp);
        Append(res,res_last,Deep_Create(s));
      end;
      tmp := Tail_Of(tmp);
    end loop;
    Copy(res,sols); Clear(res);
  end Call_Multprec_Path_Trackers;

  procedure Test_Standard_Path_Trackers is

    use Standard_Complex_Poly_Systems;
    use Standard_Complex_Solutions;

    file : file_type;
    oc : natural;
    lp,lq : Link_to_Poly_Sys;
    qsols : Solution_List;

  begin
    new_line;
    put_line("Reading the name of the output file.");
    Read_Name_and_Create_File(file);
    Standard_Read_Homotopy(lp,lq,qsols);
    put_line(file,"TARGET SYSTEM : ");
    put(file,lp'last,lp.all);
    new_line(file);
    put_line(file,"START SYSTEM : ");
    put(file,lq'last,lq.all);
    new_line(file);
    put_line(file,"START SOLUTIONS : ");
    if not Is_Null(qsols)
     then put(file,Length_Of(qsols),Head_Of(qsols).n,qsols);
          new_line(file);
          Standard_Homotopy.Create(lp.all,lq.all,1,Random1);
          new_line;
          Driver_for_Continuation_Parameters(file);
          new_line;
          Driver_for_Process_io(file,oc);
          Call_Standard_Path_Trackers(file,qsols);
          new_line(file);
          put_line(file,"THE COMPUTED SOLUTIONS : ");
          put(file,Length_Of(qsols),Head_Of(qsols).n,qsols);
          Standard_Homotopy.Clear;
    end if;
  end Test_Standard_Path_Trackers;

  procedure Test_Multprec_Path_Trackers is

    use Multprec_Complex_Numbers;
    use Multprec_Complex_Poly_Systems;
    use Multprec_Complex_Solutions;

    file : file_type;
    oc,deci,size : natural;
    lp,lq : Link_to_Poly_Sys;
    qsols : Solution_List;
    ran : Complex_Number;

  begin
    new_line;
    put_line("Reading the name of the output file.");
    Read_Name_and_Create_File(file);
    Multprec_Read_Homotopy(lp,lq,qsols);
    put_line(file,"TARGET SYSTEM : ");
    put(file,lp.all);
    new_line(file);
    put_line(file,"START SYSTEM : ");
    put(file,lq.all);
    new_line(file);
    put_line(file,"START SOLUTIONS : ");
    new_line;
    put("Give the number of decimal places : "); get(deci); skip_line;
    size := Decimal_to_Size(deci); 
    if not Is_Null(qsols)
     then put(file,Length_Of(qsols),Head_Of(qsols).n,qsols);
          Set_Size(lp.all,size);
          Set_Size(lq.all,size);
          Set_Size(qsols,size);
          new_line(file);
          ran := Random(size);
          Multprec_Homotopy.Create(lp.all,lq.all,1,ran);
          new_line;
          Continuation_Parameters.Tune(0,deci);
          Driver_for_Continuation_Parameters(file);
          new_line;
          Driver_for_Process_io(file,oc);
          Call_Multprec_Path_Trackers(file,qsols);
          new_line(file);
          put_line(file,"THE COMPUTED SOLUTIONS : ");
          put(file,Length_Of(qsols),Head_Of(qsols).n,qsols);
          Multprec_Homotopy.Clear;
    end if;
  end Test_Multprec_Path_Trackers;

  procedure Main is
  
    ans : character;

  begin
    new_line;
    put_line("Testing the path trackers");
    loop
      new_line;
      put_line("Choose one of the following : ");
      put_line("  0. Exit this program.");
      put_line("  1. Test standard predictor-corrector method.");
      put_line("  2. Test multi-precision predictor-corrector method.");
      put("Type 0,1, or 2 to select : ");
      Ask_Alternative(ans,"012");
      exit when ans = '0';
      case ans is
        when '1' => Test_Standard_Path_Trackers;
        when '2' => Test_Multprec_Path_Trackers;
        when others => null;
      end case;
    end loop;
  end Main;

begin
  Main;
end ts_path;
