with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Standard_Natural_Vectors;
with Standard_Integer_Vectors;
with Standard_Floating_Vectors;
with Standard_Complex_Vectors;
with C_Integer_Arrays;                   use C_Integer_Arrays;
with C_Double_Arrays;                    use C_Double_Arrays;

package Assignments_in_Ada_and_C is

-- DESCRIPTION :
--   This package provides assignment operators between Ada and C data.

  procedure Assign ( this : in integer; to_that : in C_intarrs.Pointer );
  procedure Assign ( this : in C_intarrs.Pointer; to_that : out integer );
  procedure Assign ( this : in double_float; to_that : in C_dblarrs.Pointer );
  procedure Assign ( this : in C_dblarrs.Pointer; to_that : out double_float );

  -- DESCRIPTION :
  --   Assigns what is in this to the location pointed by to_that.

  procedure Assign ( ada_cf : in Complex_Number;
                     c_cf : in C_dblarrs.Pointer );

  -- DESCRIPTION :
  --   Assign the complex number in ada_cf to the array in c_cf,
  --   copying real and imaginary part of ada_cf to c_cf.

  procedure Assign ( c_cf : in C_dblarrs.Pointer;
                     ada_cf : out Complex_Number );

  -- DESCRIPTION :
  --   Assigns to ada_cf the values of real and imaginary parts in c_cf.

  procedure Assign ( ada_d : in Standard_Natural_Vectors.Vector;
                     c_d : in C_intarrs.Pointer );
  procedure Assign ( ada_d : in Standard_Integer_Vectors.Vector;
                     c_d : in C_intarrs.Pointer );
  procedure Assign ( ada_d : in Standard_Floating_Vectors.Vector;
                     c_d : in C_dblarrs.Pointer );
  procedure Assign ( ada_d : in Standard_Complex_Vectors.Vector;
                     c_d : in C_dblarrs.Pointer );

  -- DESCRIPTION :
  --   Assigns the content of the Ada vector to the C vector.
  --   The complex numbers in ada_d are written to c_d as a sequence
  --   of real and imaginary double floats.

  -- REQUIRED : ada_d'range = 1..n, for some n.

  procedure Assign ( v_n : in natural; c_d : in C_intarrs.Pointer; 
                     ada_d : out Standard_Natural_Vectors.Vector );
  procedure Assign ( v_n : in natural; c_d : in C_intarrs.Pointer; 
                     ada_d : out Standard_Integer_Vectors.Vector );
  procedure Assign ( v_n : in natural; c_d : in C_dblarrs.Pointer;
                     ada_d : out Standard_Floating_Vectors.Vector );
  procedure Assign ( v_n : in natural; c_d : in C_dblarrs.Pointer;
                     ada_d : out Standard_Complex_Vectors.Vector );

  -- DESCRIPTION :
  --   Assigns the content of the C vector to the Ada vector.
  --   For complex vectors, it copies the consecutive real and
  --   and imaginary parts in c_d to ada_d.

  -- REQUIRED : ada_d'range = 1..n, for some n;
  --    for complex vectors: ada_d'range = 1..v_n/2, for some v_n.

end Assignments_in_Ada_and_C;
