with text_io,integer_io;                 use text_io,integer_io;
with Lists_of_Floating_Vectors;          use Lists_of_Floating_Vectors;

package body Cells_Container is

-- DATA STRUCTURES :

  mix : Standard_Integer_Vectors.Link_to_Vector;
  lifsup,lifsup_last : Link_to_Array_of_Lists;
  cells,cells_last : Mixed_Subdivision;

-- AUXILIARIES :

  function Retrieve ( l : List; k : natural )
                    return Standard_Floating_Vectors.Link_to_Vector is

  -- DESCRIPTION :
  --   Returns the k-th point of the list l.
  --   If k > Length_Of(l), then null is returned.

    tmp : List := l;

  begin
    for i in 1..(k-1) loop
      exit when Is_Null(tmp);
      tmp := Tail_Of(tmp);
    end loop;
    if not Is_Null(tmp)
     then return Head_Of(tmp);
     else return null;
    end if;
  end Retrieve;

  function Position ( l : List; x : Standard_Floating_Vectors.Vector )
                    return natural is

  -- DESCRIPTION :
  --   Returns 0 if x does not occur in the list l,
  --   otherwise returns the position of x in the list l.

    res : natural := 0;
    tmp : List := l;
    p : Standard_Floating_Vectors.Link_to_Vector;

  begin
    while not Is_Null(tmp) loop
      res := res + 1;
      p := Head_Of(tmp);
      if Standard_Floating_Vectors.Equal(x,p.all)
       then return res;
       else tmp := Tail_Of(tmp);
      end if;
    end loop;
    return 0;
  end Position;

-- CREATORS :

  procedure Initialize 
              ( mixture : in Standard_Integer_Vectors.Link_to_Vector;
                lifting : in Link_to_Array_of_Lists;
                mcc : in Mixed_Subdivision ) is
  begin
    mix := mixture;
    lifsup := lifting;
    cells := mcc;
  end Initialize;

  procedure Initialize 
              ( mixture : in Standard_Integer_Vectors.Link_to_Vector ) is
  begin
    mix := mixture;
  end Initialize;

  procedure Initialize ( lifting : in Link_to_Array_of_Lists ) is
  begin
    lifsup := lifting;
  end Initialize;

  procedure Initialize ( mcc : in Mixed_Subdivision ) is
  begin
    cells := mcc;
  end Initialize;

-- SELECTORS :

  function Length return natural is
  begin
    return Length_Of(cells);
  end Length;

  function Dimension return natural is

    use Standard_Floating_Vectors;

    mic : Mixed_Cell;

  begin
    if Is_Null(cells)
     then return 0;
     else mic := Head_Of(cells);
          if mic.nor = null
           then return 0;
           else return mic.nor'last;
          end if;
    end if;
  end Dimension;

  function Type_of_Mixture return Standard_Integer_Vectors.Link_to_Vector is
  begin
    return mix;
  end Type_of_Mixture;

  function Lifted_Supports return Link_to_Array_of_Lists is
  begin
    return lifsup;
  end Lifted_Supports;

  procedure Retrieve ( k : in natural; mic : out Mixed_Cell;
                       fail : out boolean ) is

    tmp : Mixed_Subdivision := cells;
    cnt : natural := 0;

  begin
    while not Is_Null(tmp) loop
      cnt := cnt + 1;
      if cnt = k
       then mic := Head_Of(tmp);
            fail := false;
            return;
       else tmp := Tail_Of(tmp);
      end if;
    end loop;
    fail := true;
  end Retrieve;

  procedure Retrieve_Mixed_Cell
             ( k : in natural; fail : out boolean;
               cnt,lab : out Standard_Integer_Vectors.Link_to_Vector;
               normal : out Standard_Floating_Vectors.Link_to_Vector ) is

    mic : Mixed_Cell;
    sum,ind : natural := 0;
    tmp : List;

  begin
    Retrieve(k,mic,fail);
    if not fail then
      normal := mic.nor;
      cnt := new Standard_Integer_Vectors.Vector(mic.pts'range);
      for i in mic.pts'range loop
        cnt(i) := Length_Of(mic.pts(i));
        sum := sum + cnt(i);
      end loop;
      lab := new Standard_Integer_Vectors.Vector(1..sum);
      for i in mic.pts'range loop
        tmp := mic.pts(i);
        while not Is_Null(tmp) loop
          ind := ind + 1;
          lab(ind) := Position(lifsup(i),Head_Of(tmp).all);
          tmp := Tail_Of(tmp);
        end loop;
      end loop;
    end if;
  end Retrieve_Mixed_Cell;

  function Retrieve return Mixed_Subdivision is
  begin
    return cells;
  end Retrieve;

-- CONSTRUCTOR :

  function Append_to_Support
             ( k : in natural; 
               x : in Standard_Floating_Vectors.Vector ) 
             return boolean is

    r : natural;
    use Standard_Integer_Vectors;

  begin
    if mix = null then
      return true;
    else 
      r := mix'last;
      if k > r then
        return true;
      else
        if lifsup = null then
          lifsup := new Array_of_Lists(1..r);
          lifsup_last := new Array_of_Lists(1..r);
        end if;
        Append(lifsup(k),lifsup_last(k),x);
        return false;
      end if;
    end if;
  end Append_to_Support;

  procedure Append ( mic : in Mixed_Cell ) is
  begin
    Append(cells,cells_last,mic);
  end Append;

  procedure Append_Mixed_Cell
             ( cnt,lab : in Standard_Integer_Vectors.Vector;
               normal : in Standard_Floating_Vectors.Vector ) is

    mic : Mixed_Cell;
    last : Array_of_Lists(cnt'range);
    ind : integer;
    lpt : Standard_Floating_Vectors.Link_to_Vector;

  begin
    mic.nor := new Standard_Floating_Vectors.Vector'(normal);
    mic.pts := new Array_of_Lists(cnt'range);
    ind := lab'first;
    for i in cnt'range loop
      for j in 1..cnt(i) loop
        lpt := Retrieve(lifsup(i),lab(ind));
        ind := ind + 1;
        Append(mic.pts(i),last(i),lpt);
      end loop;
    end loop;
    Cells_Container.Append(mic);
  end Append_Mixed_Cell;

-- DESTRUCTOR :

  procedure Clear is
  begin
    Deep_Clear(lifsup);
    Deep_Clear(cells);
  end Clear;

end Cells_Container;
