with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Natural_VecVecs;          use Standard_Natural_VecVecs;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;

package Monodromy_Permutations is

-- DESCRIPTION :
--   This package manages solution lists and slices
--   to determine permutations computed by monodromy.

  procedure Initialize ( n,d,k : in natural );

  -- DESCRIPTION :
  --   Initializes the internal data structures with space
  --   for n monodromy loops or n+1 solution lists of length d,
  --   to factor a k-dimensional solution component.

  procedure Store ( sols : in Solution_List );

  -- DESCRIPTION :
  --   Stores the list of solutions.

  function Permutation return Vector;

  -- DESCRIPTION :
  --   Returns the permutation generated by the last stored
  --   list of solutions, compared with the first one stored.

  function Decomposition return Link_to_VecVec;

  -- DESCRIPTION :
  --   Returns the current classification of witness points along
  --   the irreducible decomposition.

  procedure Update_Decomposition ( p : in Vector; nf0,nf1 : out natural );

  -- DESCRIPTION :
  --   Updates the decomposition with the permutation p.
  --   On return are in nf0 the initial number of groupings,
  --   and in nf1 the new number of groupings.

  function Certify_with_Linear_Trace return boolean;

  -- DESCRIPTION :
  --   Applies the linear trace test to each factor
  --   in the current monodromy breakup.
  --   Returns true if all factors are certified, returns false otherwise.

  procedure Clear;

  -- DESCRIPTION :
  --   Destruction of all internal data structures.

end Monodromy_Permutations;
