with Communications_with_User;           use Communications_with_User;
with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Norms_Equals;      use Standard_Complex_Norms_Equals;
with Standard_Complex_Polynomials;       use Standard_Complex_Polynomials;
with Standard_Homotopy;
with Continuation_Parameters;
with Increment_and_Fix_Continuation;     use Increment_and_Fix_Continuation;
with Standard_Root_Refiners;             use Standard_Root_Refiners;

package body PHCpack_Operations is

-- INTERNAL DATA :

  file_okay,empty_homotopy,zero_constant : boolean;

  start_sys,target_sys : Link_to_Poly_Sys;
  start_sols,target_sols : Solution_List;

  gamma_constant : Complex_Number;

  output_file : file_type;

-- OPERATIONS :

  procedure Define_Output_File is
  begin
    new_line;
    put_line("Reading the name of the output file...");
    Read_Name_and_Create_File(output_file);
    file_okay := true;
  end Define_Output_File;

  function Is_File_Defined return boolean is
  begin
    return file_okay;
  end Is_File_Defined;

  function Retrieve_Output_File return file_type is
  begin
    if file_okay
     then return output_file;
     else return standard_output;
    end if;
  end Retrieve_Output_File;

  procedure Store_Start_System ( p : in Poly_Sys ) is
  begin
    Clear(start_sys);
    start_sys := new Poly_Sys(p'range);
    for i in p'range loop
      Copy(p(i),start_sys(i));
    end loop;
  end Store_Start_System;

  procedure Store_Start_Solutions ( sols : in Solution_List ) is
  begin
    Copy(sols,start_sols);
  end Store_Start_Solutions;

  procedure Store_Target_System ( p : in Poly_Sys ) is
  begin
    Clear(target_sys);
    target_sys := new Poly_Sys(p'range);
    for i in p'range loop
      Copy(p(i),target_sys(i));
    end loop;
  end Store_Target_System;

  procedure Store_Target_Solutions ( sols : in Solution_List ) is
  begin
    Copy(sols,target_sols);
  end Store_Target_Solutions;

  procedure Retrieve_Start_System ( p : out Link_to_Poly_Sys ) is
  begin
    p := start_sys;
  end Retrieve_Start_System;

  procedure Retrieve_Start_Solutions ( sols : out Solution_List ) is
  begin
    sols := start_sols;
  end Retrieve_Start_Solutions;

  procedure Retrieve_Target_System ( p : out Link_to_Poly_Sys ) is
  begin
    p := target_sys;
  end Retrieve_Target_System;

  procedure Retrieve_Target_Solutions ( sols : out Solution_List ) is
  begin
    sols := target_sols;
  end Retrieve_Target_Solutions;

  procedure Store_Gamma_Constant ( gamma : in Complex_Number ) is
  begin
    zero_constant := false;
    gamma_constant := gamma;
  end Store_Gamma_Constant;

  procedure Copy_Labels is

  -- DESCRIPTION :
  --   Copies the multiplicity labels from start to target solutions,
  --   only in case these labels are > 1 and if m field of target is 1.

    tmp_start : Solution_List := start_sols;
    tmp_target : Solution_List := target_sols;
    ls_start,ls_target : Link_to_Solution;

  begin
    while not Is_Null(tmp_start) loop
      ls_start := Head_Of(tmp_start);
      if ls_start.m > 1
       then ls_target := Head_Of(tmp_target);
            if ls_target.m = 1
             then ls_target.m := ls_start.m;
                  Set_Head(tmp_target,ls_target);
            end if;
      end if;
      tmp_start := Tail_Of(tmp_start);
      tmp_target := Tail_Of(tmp_target);
    end loop;
  end Copy_Labels;

  procedure Solve_by_Homotopy_Continuation is

    k : constant natural := 2;
    r : Complex_Number; 
    epsxa : constant double_float := 1.0E-13;
    epsfa : constant double_float := 1.0E-13;
    tolsing : constant double_float := 1.0E-8;
    numit : natural := 0;
    max : constant natural := 4;

    procedure Sil_Cont is
      new Silent_Continue(Max_Norm,Standard_Homotopy.Eval,
                          Standard_Homotopy.Diff,Standard_Homotopy.Diff);

  begin
    if zero_constant
     then r := Create(0.793450603947633,-0.608634651572795);
     else r := gamma_constant;
    end if;
    if empty_homotopy
     then Standard_Homotopy.Create(target_sys.all,start_sys.all,k,r);
          empty_homotopy := false; -- for dynamic load balancing
    end if;
    Continuation_Parameters.Tune(2);
    Copy(start_sols,target_sols);
    Set_Continuation_Parameter(target_sols,Create(0.0));
    Sil_Cont(target_sols,false,Create(1.0));
    Silent_Root_Refiner
      (target_sys.all,target_sols,epsxa,epsfa,tolsing,numit,max,false);
    Copy_Labels;
   -- Standard_Homotopy.Clear;  -- for dynamic load balancing
  end Solve_by_Homotopy_Continuation;

  procedure Clear is
  begin
    Clear(start_sys); Clear(start_sols);
    Clear(target_sys); Clear(target_sols);
    if not empty_homotopy
     then Standard_Homotopy.Clear;
          empty_homotopy := true;
    end if;
  end Clear;

begin
  file_okay := false;
  empty_homotopy := true;
  zero_constant := true;
end PHCpack_Operations;
