with text_io;                             use text_io;
with Standard_Complex_Numbers;            use Standard_Complex_Numbers;
with Standard_Complex_Poly_Systems;       use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;          use Standard_Complex_Solutions;

package PHCpack_Operations is

-- DESCRIPTION :
--   Provides the most common operations of PHCpack,
--   with data management for polynomial systems and solutions.

  procedure Define_Output_File;
  function Is_File_Defined return boolean;
  function Retrieve_Output_File return file_type;

  -- DESCRIPTION :
  --   The "Determine_Output_File" lets the user define an output file,
  --   which can be retrieved by "Retrieve_Output_File".
  --   If "Is_File_Defined" returns true, then it is okay to write to
  --   "Retrieve_Output_File", otherwise "standard_output" is returned.

  procedure Store_Start_System ( p : in Poly_Sys );
  procedure Store_Start_Solutions ( sols : in Solution_List );
  procedure Store_Target_System ( p : in Poly_Sys );
  procedure Store_Target_Solutions ( sols : in Solution_List );

  procedure Retrieve_Start_System ( p : out Link_to_Poly_Sys );
  procedure Retrieve_Start_Solutions ( sols : out Solution_List );
  procedure Retrieve_Target_System ( p : out Link_to_Poly_Sys );
  procedure Retrieve_Target_Solutions ( sols : out Solution_List );

  procedure Store_Gamma_Constant ( gamma : in Complex_Number );

  procedure Solve_by_Homotopy_Continuation;

  procedure Clear;

end PHCpack_Operations;
