with text_io;                            use text_io;
with Communications_with_User;           use Communications_with_User;
with File_Scanning;                      use File_Scanning;
with Standard_Complex_Poly_Systems;      use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;   use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Solutions;         use Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;      use Standard_Complex_Solutions_io;
with PHCpack_Operations;

package body PHCpack_Operations_io is

  procedure Read_System ( file : in file_type;
                          p : out Link_to_Poly_Sys; 
                          sols : out Solution_List ) is

    found : boolean;

  begin
    get(file,p);
    Scan_and_Skip(file,"SOLUTIONS",found);
    if found
     then get(file,sols);
    end if;
  end Read_System;

  procedure Read_System ( p : out Link_to_Poly_Sys; 
                          sols : out Solution_List ) is

    file : file_type;

  begin
    Read_Name_and_Open_File(file);
    Read_System(file,p,sols);
    Close(file);
  end Read_System;

  procedure Read_Start_System is

    p : Link_to_Poly_Sys;
    sols : Solution_List;

  begin
    new_line;
    put_line("Reading the start system...");
    Read_System(p,sols);
    PHCpack_Operations.Store_Start_System(p.all);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Start_Solutions(sols);
    end if;
  end Read_Start_System;

  procedure Read_Start_System ( filename : in string ) is

    file : file_type;
    p : Link_to_Poly_Sys;
    sols : Solution_List;

  begin
    Open(file,in_file,filename);
    Read_System(file,p,sols);
    Close(file);
    PHCpack_Operations.Store_Start_System(p.all);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Start_Solutions(sols);
    end if;
  end Read_Start_System;

  procedure Read_Start_Solutions is

    file : file_type;
    sols : Solution_List;

  begin
    new_line;
    put_line("Reading the start solutions...");
    Read_Name_and_Open_File(file);
    get(file,sols);
    Close(file);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Start_Solutions(sols);
    end if;
  end Read_Start_Solutions;

  procedure Read_Start_Solutions ( filename : in string ) is

    file : file_type;
    sols : Solution_List;

  begin
    Open(file,in_file,filename);
    get(file,sols);
    Close(file);
    PHCpack_Operations.Store_Start_Solutions(sols);
  end Read_Start_Solutions;

  procedure Read_Target_System is

    p : Link_to_Poly_Sys;
    sols : Solution_List;

  begin
    new_line;
    put_line("Reading the target system...");
    Read_System(p,sols);
    PHCpack_Operations.Store_Target_System(p.all);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Target_Solutions(sols);
    end if;
  end Read_Target_System;

  procedure Read_Target_System ( filename : in string ) is

    file : file_type;
    p : Link_to_Poly_Sys;
    sols : Solution_List;

  begin
    Open(file,in_file,filename);
    Read_System(file,p,sols);
    Close(file);
    PHCpack_Operations.Store_Target_System(p.all);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Target_Solutions(sols);
    end if;
  end Read_Target_System;

  procedure Read_Target_Solutions is

    file : file_type;
    sols : Solution_List;

  begin
    new_line;
    put_line("Reading the target solutions...");
    Read_Name_and_Open_File(file);
    get(file,sols);
    Close(file);
    if not Is_Null(sols)
     then PHCpack_Operations.Store_Target_Solutions(sols);
    end if;
  end Read_Target_Solutions;

  procedure Read_Target_Solutions ( filename : in string ) is

    file : file_type;
    sols : Solution_List;

  begin
    Open(file,in_file,filename);
    get(file,sols);
    Close(file);
    PHCpack_Operations.Store_Target_Solutions(sols);
  end Read_Target_Solutions;

  procedure Write_Start_System is

    p : Link_to_Poly_Sys;

  begin
    PHCpack_Operations.Retrieve_Start_System(p);
    put_line(PHCpack_Operations.Retrieve_Output_File,"The start system : ");
    put(PHCpack_Operations.Retrieve_Output_File,p.all);
  end Write_Start_System;

  procedure Write_Start_System ( filename : in string ) is

    file : file_type;
    p : Link_to_Poly_Sys;

  begin
    Create(file,out_file,filename);
    PHCpack_Operations.Retrieve_Start_System(p);
    put(file,p.all);
    Close(file);
  end Write_Start_System;

  procedure Write_Start_Solutions is

    sols : Solution_List;

  begin
    PHCpack_Operations.Retrieve_Start_Solutions(sols);
    if not Is_Null(sols)
     then put(PHCpack_Operations.Retrieve_Output_File,
              Length_Of(sols),Head_Of(sols).n,sols);
    end if;
  end Write_Start_Solutions;

  procedure Write_Start_Solutions ( filename : in string ) is

    file : file_type;
    sols : Solution_List;

  begin
    PHCpack_Operations.Retrieve_Start_Solutions(sols);
    if not Is_Null(sols)
     then Create(file,out_file,filename);
          put(file,Length_Of(sols),Head_Of(sols).n,sols);
          Close(file);
    end if;
  end Write_Start_Solutions;

  procedure Write_Target_System is

    p : Link_to_Poly_Sys;

  begin
    PHCpack_Operations.Retrieve_Target_System(p);
    put_line(PHCpack_Operations.Retrieve_Output_File,"The target system : ");
    put(PHCpack_Operations.Retrieve_Output_File,p.all);
  end Write_Target_System;

  procedure Write_Target_System ( filename : in string ) is

    file : file_type;
    p : Link_to_Poly_Sys;

  begin
    Create(file,out_file,filename);
    PHCpack_Operations.Retrieve_Target_System(p);
    put(file,p.all);
    Close(file);
  end Write_Target_System;

  procedure Write_Target_Solutions is

    sols : Solution_List;

  begin
    PHCpack_Operations.Retrieve_Target_Solutions(sols);
    if not Is_Null(sols)
     then put(PHCpack_Operations.Retrieve_Output_File,
              Length_Of(sols),Head_Of(sols).n,sols);
    end if;
  end Write_Target_Solutions;

  procedure Write_Target_Solutions ( filename : in string ) is

    file : file_type;
    sols : Solution_List;

  begin
    PHCpack_Operations.Retrieve_Target_Solutions(sols);
    if not Is_Null(sols)
     then Create(file,out_file,filename);
          put(file,Length_Of(sols),Head_Of(sols).n,sols);
          Close(file);
    end if;
  end Write_Target_Solutions;

end PHCpack_Operations_io;
