with text_io,integer_io;                use text_io,integer_io;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Complex_Numbers_io;       use Standard_Complex_Numbers_io;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_VecVecs_io;       use Standard_Complex_VecVecs_io;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;

with Standard_Complex_Vectors;          use Standard_Complex_Vectors;
with Witness_Points;                    use Witness_Points;
with Sampling_Machine;
with Solutions_Container;

package body Sampling_Operations is

-- INTERNAL DATA :

  start_samples,new_samples : Solution_List;
  original_solutions : Solution_List;
  dimension : natural;
  start_slices,new_slices : Link_to_VecVec;
  gamma : Link_to_Vector;

-- OPERATIONS :

  procedure Initialize ( p : in Poly_Sys; sols : in Solution_List;
                         k : in natural ) is

    n : constant natural := p'last;

  begin
    Sampling_Machine.Initialize(p);
    Sampling_Machine.Default_Tune_Sampler(0);
    Sampling_Machine.Default_Tune_Refiner;
   -- put_line("initialized sampling machine...");
    start_samples := sols;
    dimension := k;
    start_slices := new VecVec'(Slices(p,k));
    new_slices := new VecVec(1..k);
    for i in 1..k loop
      new_slices(i) := new Vector'(start_slices(i).all);
    end loop;
    gamma := new Vector(1..n);
    for i in 1..n loop
      gamma(i) := Create(1.0);
    end loop;
   -- put("  k = "); put(k,1);
   -- put("  d = "); put(Length_Of(sols),1); new_line;
    Copy(sols,original_solutions);
  end Initialize;

  function Retrieve_First_Solutions return Solution_List is
  begin
    return original_solutions;
  end Retrieve_First_Solutions;

  function Retrieve_Start_Slices return Link_to_VecVec is
  begin
    return start_slices;
  end Retrieve_Start_Slices;

  function Retrieve_Target_Slices return Link_to_VecVec is
  begin
    return new_slices;
  end Retrieve_Target_Slices;

  procedure Assign_Slice ( c : in Complex_Number; i,j : in natural ) is
  begin
    new_slices(i+1)(j) := c;
   -- put("received "); put(c);
   -- put(" to assign at ("); put(i+1,1); put(","); put(j,1);
   -- put_line(")");
  end Assign_Slice;

  procedure Store_Gamma ( c : in Complex_Number; i : in natural ) is
  begin
    gamma(i+1) := c;
   -- put("received "); put(c);
   -- put(" as gamma constant "); put(i+1,1); new_line;
  end Store_Gamma;

  procedure Swap_Slices is

    tmp_slices : Link_to_VecVec := new_slices;
    tmp_samples : Solution_List := new_samples;

  begin
    new_slices := start_slices;
    start_slices := tmp_slices;
    new_samples := start_samples;
    start_samples := tmp_samples;
    Set_Continuation_Parameter(start_samples,Create(0.0));
    Sampling_Machine.Change_Slices(start_slices.all);
  end Swap_Slices;

  procedure Sample is
  begin
   -- put_line("Sampling with gamma =");
   -- put_line(gamma.all);
   -- put_line("Sampling with slices :"); put(new_slices.all);
    Sampling_Machine.Sample
     (start_samples,new_slices.all,gamma.all,new_samples);
   -- put_line("Computed solutions : ");
   -- put(standard_output,
   --     Length_Of(new_samples),Head_Of(new_samples).n,new_samples);
    Solutions_Container.Clear;
    Solutions_Container.Initialize(new_samples);
  end Sample;

  procedure Clear is
  begin
    Sampling_Machine.Clear;
    dimension := 0;
  end Clear;

end Sampling_Operations;
