with Standard_Complex_Numbers;            use Standard_Complex_Numbers;
with Standard_Complex_VecVecs;            use Standard_Complex_VecVecs;
with Standard_Complex_Poly_Systems;       use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;          use Standard_Complex_Solutions;

package Sampling_Operations is

-- DESCRIPTION :
--   This is the analogue to PHCpack_Operations to guide the path
--   tracking needed in the monodromy breakup algorithm.

  procedure Initialize ( p : in Poly_Sys; sols : in Solution_List;
                         k : in natural );

  -- DESCRIPTION :
  --   Initializes the sampling machine with an embedded system p,
  --   and points in the list sols on a k-dimensional solution set.

  function Retrieve_First_Solutions return Solution_List;

  -- DESCRIPTION :
  --   Returns a pointer to the solutions, used with Initialize.

  function Retrieve_Start_Slices return Link_to_VecVec;

  -- DESCRIPTION :
  --   Returns a pointer to the current set of start slices.

  function Retrieve_Target_Slices return Link_to_VecVec;

  -- DESCRIPTION :
  --   Returns a pointer to the current set of target slices.
 
  procedure Assign_Slice ( c : in Complex_Number; i,j : in natural );

  -- DESCRIPTION :
  --   Assigns c to the j-th coefficient of the i-th slice.
  --   Notice that i starts to run from 0.

  procedure Store_Gamma ( c : in Complex_Number; i : in natural );

  -- DESCRIPTION :
  --   Stores c as the i-th gamma constant.
  --   Notice that i starts to run from 0.

  procedure Swap_Slices;

  -- DESCRIPTION :
  --   Swaps the start with the target set of slices,
  --   and the start solutions with the target solutions.

  procedure Sample;

  -- DESCRIPTION :
  --   Computes a new witness set on the new slices.
  --   The solutions are put in the solutions container.

  procedure Clear;

end Sampling_Operations;
