with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;

package body Systems_Container is

-- INTERNAL DATA :

  lp : Link_to_Poly_Sys;

-- CREATORS :

  procedure Initialize ( p : in Poly_Sys ) is
  begin
    lp := new Poly_Sys(p'range);
    for i in p'range loop
      Copy(p(i),lp(i));
    end loop;
  end Initialize;

  procedure Initialize ( n : in natural ) is
  begin
    lp := new Poly_Sys(1..n);
  end Initialize;

-- CONSTRUCTORS :

  procedure Add_Term ( k : in natural; t : in Term ) is
  begin
    Add(lp(k),t);
  end Add_Term;

  procedure Add_Poly ( k : in natural; p : in Poly ) is
  begin
    Add(lp(k),p);
  end Add_Poly;

-- SELECTORS :

  function Dimension return natural is
  begin
    if lp = null 
     then return 0;
     else return lp'last;
    end if;
  end Dimension;

  function Number_of_Terms ( k : natural ) return natural is
  begin
    if lp = null
     then return 0;
     else return Number_of_Terms(lp(k));
    end if;
  end Number_of_Terms;

  function Retrieve_Term ( k,i : natural ) return Term is

    res : Term;
    cnt : natural := 0;

    procedure Find_Term ( t : in Term; continue : out boolean ) is
    begin
      cnt := cnt + 1;
      if cnt = i
       then res.cf := t.cf;
            res.dg := new Standard_Natural_Vectors.Vector'(t.dg.all);
            continue := false;
       else continue := true;
      end if;
    end Find_Term;
    procedure Scan_Terms is new Visiting_Iterator(Find_Term);

  begin
    res.cf := Create(0.0);
    if lp = null
     then return res;
     elsif i = 0 or i > Number_of_Terms(k)
         then return res;
         else Scan_Terms(lp(k));
    end if;
    return res;
  end Retrieve_Term;

  function Retrieve_Poly ( k : natural ) return Poly is
  begin
    if lp = null 
     then return Null_Poly;
     elsif k = 0 or k > lp'last
         then return Null_Poly;
         else return lp(k);
    end if;
  end Retrieve_Poly;

  function Retrieve return Link_to_Poly_Sys is
  begin
    return lp;
  end Retrieve;

-- DESTRUCTOR :

  procedure Clear is
  begin
    Clear(lp);
  end Clear;

end Systems_Container;
