with text_io,integer_io;                use text_io,integer_io;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Natural_Vectors;
with Standard_Natural_VecVecs;
with Standard_Complex_Vectors;
with Standard_Random_Vectors;           use Standard_Random_Vectors;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;
with Sampling_Machine;
with Witness_Points_io;                 use Witness_Points_io;
with Planes_and_Polynomials;            use Planes_and_Polynomials;
with Assignments_in_Ada_and_C;          use Assignments_in_Ada_and_C;
with Systems_Container;
with Solutions_Container;
with PHCpack_Operations;
with Sampling_Operations;
with Monodromy_Partitions;
with Monodromy_Permutations;

function use_c2fac ( job : integer;
                     a : C_intarrs.Pointer;
		     b : C_intarrs.Pointer;
                     c : C_dblarrs.Pointer ) return integer is

  procedure Write_Menu is
  begin
    new_line;
    put_line("General MENU to use factorization in PHCpack from C :");
    put_line("  0. display this menu;");
    put_line("  1. read an embedding of a polynomial system;");
    put_line("  2. initialize the sampling machine;");
    put_line("  3. assign coefficient of a slice;");
    put_line("  4. store a random gamma constant in PHCpack;");
    put_line("  5. compute a new witness set on the new slices;");
    put_line("  6. swaps slices and solution sets to turn back;");
    put_line("  7. copy embedded system from sampler to systems container;");
    put_line(" 10. initialize maximum number of monodromy loops;");
    put_line(" 11. copy solutions in container to Monodromy_Permutations;");
    put_line(" 12. compute permutation by last stored solution list;");
    put_line(" 13. update the monodromy breakup with a new permutation;");
    put_line(" 14. write the current monodromy breakup;");
    put_line(" 15. apply the linear trace to certify the monodromy breakup.");
  end Write_Menu;

begin
  case job is
    when  0 => Write_Menu;
    when  1 => declare
                 p : Link_to_Poly_Sys;
                 sols : Solution_List;
                 dim : natural;
                 data : Standard_Natural_Vectors.Vector(1..2);
               begin
                 Standard_Read_Embedding(p,sols,dim);
                 Systems_Container.Initialize(p.all);
                 Solutions_Container.Initialize(sols);
                -- Sampling_Operations.Initialize(p.all,sols,dim);
                 data(1) := dim;
                 data(2) := Length_Of(sols);
                -- put("The dimension is "); put(data(1),1); new_line;
                -- put("The degree is "); put(data(2),1); new_line;
                 Assign(p'last,a);
                 Assign(data,b);
               end;
    when  2 => declare
                 lp : Link_to_Poly_Sys := Systems_Container.Retrieve;
                 sols : Solution_List := Solutions_Container.Retrieve;
                 va : C_Integer_Array(0..0) := C_intarrs.Value(a);
                 dim : constant natural := natural(va(0));
               begin
                 Sampling_Operations.Initialize(lp.all,sols,dim);
               end;
    when  3 => declare
                 va : C_Integer_Array(0..0) := C_intarrs.Value(a);
                 vb : C_Integer_Array(0..0) := C_intarrs.Value(b);
                 vc : C_Double_Array(0..1) := C_DblArrs.Value(c);
                 re : double_float := double_float(vc(0));
                 im : double_float := double_float(vc(1));
                 cf : Complex_Number := Create(re,im);
                 i : constant natural := natural(va(0));
                 j : constant natural := natural(vb(0));
               begin
                 Sampling_Operations.Assign_Slice(cf,i,j);
               end;
    when  4 => declare
                 va : C_Integer_Array(0..0) := C_intarrs.Value(a);
                 vc : C_Double_Array(0..1) := C_DblArrs.Value(c);
                 re : double_float := double_float(vc(0));
                 im : double_float := double_float(vc(1));
                 gamma : Complex_Number := Create(re,im);
                 i : constant natural := natural(va(0));
               begin
                 Sampling_Operations.Store_Gamma(gamma,i);
               end;
    when  5 => Sampling_Operations.Sample;
    when  6 => Sampling_Operations.Swap_Slices;
    when  7 => declare
                 p : constant Poly_Sys := Sampling_Machine.Embedded_System;
               begin
                 Systems_Container.Initialize(p);
               end;
    when  8 => declare
                 s : Solution_List
                   := Sampling_Operations.Retrieve_First_Solutions;
               begin
                 Solutions_Container.Clear;
                 Solutions_Container.Initialize(s);
               end;
    when 10 => declare
                 va : C_Integer_Array(0..0) := C_intarrs.Value(a);
                 vb : C_Integer_Array(0..1) := C_intarrs.Value(b);
                 n : constant natural := natural(va(0));
                 d : constant natural := natural(vb(0));
                 k : constant natural := natural(vb(1));
               begin
                -- put("initializing monodromy_permutations with ");
                -- put("  n = "); put(n,1);
                -- put("  d = "); put(d,1);
                -- put("  k = "); put(k,1); new_line;
                 Monodromy_Permutations.Initialize(n,d,k);
               end;
    when 11 => declare
                 sols : Solution_List := Solutions_Container.Retrieve;
               begin
                 Monodromy_Permutations.Store(sols);
               end;
    when 12 => declare
                 perm : constant Standard_Natural_Vectors.Vector
                      := Monodromy_Permutations.Permutation;
               begin
                -- put("Ada permutation : ");
                -- for i in perm'range loop
                --   put(" "); put(perm(i),1);
                -- end loop;
                -- new_line;
                 Assign(perm,b);
               end;
    when 13 => declare
                 va : C_Integer_Array(0..0) := C_intarrs.Value(a);
                 n : constant natural := natural(va(0));
                 p : Standard_Natural_Vectors.Vector(1..n);
                 nf : Standard_Natural_Vectors.Vector(1..2);
               begin
                 Assign(n,b,p);
                -- put("Processing");
                -- for i in p'range loop
                --   put(" "); put(p(i),1);
                -- end loop;
                 Monodromy_Permutations.Update_Decomposition(p,nf(1),nf(2));
                -- put(" : "); put(nf(1),1); put(" -> "); put(nf(2),1);
                -- new_line;
                 Assign(nf,a);
               end;
    when 14 => declare
                 deco : Standard_Natural_VecVecs.Link_to_VecVec
                      := Monodromy_Permutations.Decomposition;
                 use Standard_Natural_VecVecs;
               begin
                 if deco /= null
                  then Monodromy_Partitions.Write_Factors
                         (standard_output,deco.all);
                 end if;
               end;
    when 15 => declare
                 done : boolean
                      := Monodromy_Permutations.Certify_with_Linear_Trace;
               begin
                 if done
                  then Assign(1,a);
                  else Assign(0,a);
                 end if;
               end;
    when others => put_line("  Sorry.  Invalid operation.");
                   return 1;
  end case;
  return 0;
end use_c2fac;
