with integer_io;                        use integer_io;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;

with text_io;                           use text_io;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;        use Standard_Complex_Solutions;
with Standard_Root_Refiners;            use Standard_Root_Refiners;
with Systems_Container;
with Solutions_Container;
with PHCpack_Operations;
with C_to_PHCpack;
with use_syscon;
with use_solcon;
with use_c2fac;

function use_c2phc ( job : integer;
                     a : C_intarrs.Pointer;
		     b : C_intarrs.Pointer;
                     c : C_dblarrs.Pointer ) return integer is

  procedure Write_Menu is
  begin
    new_line;
    put_line("General MENU to use PHCpack from C :");
    put_line("  0. display this menu;");
    put_line("MENU to move data from PHCpack from and to containers :");
    put_line("  1. copy target system to systems container;");
    put_line("  2. copy system in container to target system;");
    put_line("  3. copy start system to systems container;");
    put_line("  4. copy system in container to start system;");
    put_line("  5. copy target solutions to solutions container;");
    put_line("  6. copy solutions in container to target solutions;");
    put_line("  7. copy start solutions to solutions container;");
    put_line("  8. copy solutions in container to start solutions;");
    put_line("  9. validate solutions container using systems container.");
    put_line("MENU to the core operations in PHCpack :");
    put_line(" 10. displays the menu of available operations;");
    put_line(" 11. read target polynomial system;");
    put_line(" 12. write target polynomial system;");
    put_line(" 13. read start polynomial system;");
    put_line(" 14. write start polynomial system;");
    put_line(" 15. write start solutions;");
    put_line(" 16. solve by homotopy continuation;");
    put_line(" 17. write the target solutions;");
    put_line(" 18. clear the data in PHCpack_Operations;");
    put_line(" 19. define the output file.");
    put_line("MENU to the operations in the systems container :");
    put_line(" 20. read polynomial system and put in container;");
    put_line(" 21. write the polynomial system in the container;");
    put_line(" 22. return the dimension of the polynomial system;");
    put_line(" 23. initializes the container with a dimension;"); 
    put_line(" 24. return the number of terms in a polynomial;");
    put_line(" 25. return a term in a polynomial;");
    put_line(" 26. add a term to a polynomial;");
    put_line(" 27. the systems container is cleared;");
    put_line("MENU to the operations in the solutions container :");
    put_line(" 30. read solutions from file and put in container;");
    put_line(" 31. write solutions in the container;");
    put_line(" 32. return the length of the container;");
    put_line(" 33. return the dimension of the solution vectors;");
    put_line(" 34. return a solution from the container;");
    put_line(" 35. changes a solution in the container;");
    put_line(" 36. append a solution to the container;");
    put_line(" 37. clears all solutions from the container;");
    put_line("MENU to the monodromy factorization :");
    put_line(" 40. displays menu of the factorization operations;");
    put_line(" 41. reads a witness set respresented as embedded system;");
    put_line(" 42. initialize the sampling machine;");
    put_line(" 43. assign coefficient of a slice;");
    put_line(" 44. store a random gamma constant in PHCpack;");
    put_line(" 45. compute a new witness set on the new slices;");
    put_line(" 46. swaps slices and solution sets to turn back;");
    put_line(" 47. copy embedded system from sampler to systems container;");
    put_line(" 50. initialize maximum number of monodromy loops;");
    put_line(" 51. copy solutions in container to Monodromy_Permutations;");
    put_line(" 52. compute permutation by last stored solution list;");
    put_line(" 53. update the breakup with a new permutation;");
    put_line(" 54. write the current monodromy breakup;");
    put_line(" 55. applies the linear trace to certify the decomposition.");
  end Write_Menu;

begin
  case job is
    when 0 => Write_Menu;
    when 1 => declare
                lp : Link_to_Poly_Sys;
              begin
                PHCpack_Operations.Retrieve_Target_System(lp);
                if lp = null
                 then return 1;
                 else Systems_Container.Initialize(lp.all);
                end if;
              end;
    when 2 => declare
                lp : Link_to_Poly_Sys := Systems_Container.Retrieve;
              begin
                if lp = null
                 then return 1;
                 else PHCpack_Operations.Store_Target_System(lp.all);
                end if;
              end;
    when 3 => declare
                lp : Link_to_Poly_Sys;
              begin
                PHCpack_Operations.Retrieve_Start_System(lp);
                if lp = null
                 then return 1;
                 else Systems_Container.Initialize(lp.all);
                end if;
              end;
    when 4 => declare
                lp : Link_to_Poly_Sys := Systems_Container.Retrieve;
              begin
                if lp = null
                 then return 1;
                 else PHCpack_Operations.Store_Start_System(lp.all);
                end if;
              end;
    when 5 => declare
                sols : Solution_List;
              begin
                PHCpack_Operations.Retrieve_Target_Solutions(sols);
                if Is_Null(sols)
                 then return 1;
                 else Solutions_Container.Initialize(sols);
                end if;
              end;
    when 6 => declare
                sols : Solution_List := Solutions_Container.Retrieve;
              begin
                if Is_Null(sols)
                 then return 1;
                 else PHCpack_Operations.Store_Target_Solutions(sols);
                end if;
              end;
    when 7 => declare
                sols : Solution_List;
              begin
                PHCpack_Operations.Retrieve_Start_Solutions(sols);
                if Is_Null(sols)
                 then return 1;
                 else Solutions_Container.Initialize(sols);
                end if;
              end;
    when 8 => declare
                sols : Solution_List := Solutions_Container.Retrieve;
              begin
                if Is_Null(sols)
                 then return 1;
                 else PHCpack_Operations.Store_Start_Solutions(sols);
                end if;
              end;
    when 9 => declare
                lp : Link_to_Poly_Sys := Systems_Container.Retrieve;
                sols : Solution_List := Solutions_Container.Retrieve;
                work,tmp : Solution_List;
              begin
               -- put_line("refining the roots ...");
               -- put(lp.all);
                if lp = null or Is_Null(sols) then
                  return 1;             
                else
                 -- put_line("Solutions before the refine :");
                 -- put(standard_output,Length_Of(sols),Head_Of(sols).n,sols);
                  Copy(sols,work);
                  declare
                    epsxa : constant double_float := 1.0E-12;
                    epsfa : constant double_float := 1.0E-12;
                    tolsi : constant double_float := 1.0E-8;
                    nit : natural := 0;
                  begin
                   -- Reporting_Root_Refiner
                   --   (standard_output,lp.all,work,
                   --    epsxa,epsfa,tolsi,nit,5,false,false);
                    Silent_Root_Refiner
                      (lp.all,work,epsxa,epsfa,tolsi,nit,5,false);
                    Solutions_Container.Clear;
                    Solutions_Container.Initialize(work);
                    put("Refiner Results :");
                    tmp := work;
                    while not Is_Null(tmp) loop
                      put(" ");
                      put(Head_Of(tmp).m,1);
                      tmp := Tail_Of(tmp);
                    end loop;
                    new_line;
                  end;
                 -- sols := Solutions_Container.Retrieve;
                 -- put_line("Solutions after the refine :");
                 -- put(standard_output,Length_Of(sols),Head_Of(sols).n,sols);
                end if;
              end;
    when 10..19 => return C_to_PHCpack(job-10);
    when 20..27 => return use_syscon(job-20,a,b,c);
    when 30..37 => return use_solcon(job-30,a,b,c);
    when 40..55 => return use_c2fac(job-40,a,b,c);
    when others => put_line("  Sorry.  Invalid operation.");
                   return 1;
  end case;
  return 0;
end use_c2phc;
