with C_Integer_Arrays;                  use C_Integer_Arrays;
with C_Double_Arrays;                   use C_Double_Arrays;

function use_c2phc ( job : integer;
                     a : C_intarrs.Pointer;
		     b : C_intarrs.Pointer;
                     c : C_dblarrs.Pointer ) return integer;

-- DESCRIPTION :
--   Provides a gateway from C to the operations in PHCpack.

-- ON ENTRY :
--   job     = 0 : display the menu of all available options;
--
-- moving data from PHCpack_operations from and to containers :
--
--   job     = 1 : copy target system to systems container;
--           = 2 : copy system in container to target system;
--           = 3 : copy start system to systems container;
--           = 4 : copy system in container to start system;
--           = 5 : copy target solutions to solutions container;
--           = 6 : copy solutions in container to target solutions;
--           = 7 : copy start solutions to solutions container;
--           = 8 : copy solutions in container to start solutions;
--           = 9 : validate the solutions in the container, using
--                 the contents of the system container as target.
--
-- the core operations in PHCpack :
--
--   job     = 10 : displays the menu of available operations;
--           = 11 : read target polynomial system;
--           = 12 : write target polynomial system;
--           = 13 : read start polynomial system;
--           = 14 : write start polynomial system;
--           = 15 : write start solutions;
--           = 16 : solve by homotopy continuation;
--           = 17 : write the target solutions;
--           = 18 : clear the data in PHCpack_Operations;
--           = 19 : define the output file.
--
-- the operations of the systems container :
--
--   job     = 20 : read polynomial system and put in container;
--           = 21 : write the polynomial system in the container;
--           = 22 : return in a[0] the dimension of the polynomial system;
--           = 23 : initializes the container with the dimension in a[0];
--           = 24 : return in a[0] the number of terms in the i-th polynomial;
--           = 25 : return in c the coefficient (real and imaginary part),
--                     and in b the exponent vector (i = a[1], j = a[2])
--                  of the j-th term in the i-th polynomial;
--           = 26 : add to the i-th polynomial the term with coefficient
--                     in c and exponent vector in b;
--           = 27 : the systems container is cleared;
--
-- with the following meaning of the parameters (a,b,c) :
--
--   a       memory allocated for array of integers, a = (n,i,j),
--           where n is the dimension,
--                 i is index to the polynomial in the system to work on;
--                 j is index to the mononomial in the i-th polynomial;
--   b       memory allocated for array of integers, used for exponents;
--   c       memory allocated for array of double floating-point numbers,
--           used for real and imaginary part of complex coefficient.
--
-- the operations in the solutions container :
--
--   job     = 30 : read solutions from file and put in container;
--           = 31 : write solutions in the container;
--           = 32 : return in b the length of the container;
--           = 33 : return in b the dimension of the solution vectors;
--           = 34 : return in b the multiplicity m of the solution (# in a),
--                         in c the solution as an array of 2*n+5 doubles,
--            in the following order:
--                  two doubles for the complex continuation parameter t,
--                  2*n doubles for the coefficients of the solution vector,
--                  one double for the norm of last Newton update,
--                  one double for the inverse of condition# estimate,
--                  one double for the norm of the residual;
--           = 35 : changes the solution with number in to the values 
--                 in b and c, where b is the array (n,m),
--                 where n is the dimension and m the multiplicity;
--           = 36 : appends the solution with data in b  = (n,m) and
--                  (also see job = 4 for info) to the container;
--           = 37 : clears all solutions from the container;
--
-- with the following meaning of the parameters (a,b,c) :
--
--   a        indicates the number of solution to work on;
--   b        array with allocated memory for integers,
--            usually only len, n, m, or the array (n,m),
--            where len is the length of the container;
--                  n is the dimension of the solution vectors; and
--                  m is the multiplicity of the solution;
--   c        array with allocated memory for double floats to store
--            the complex value for the continuation parameter t,
--            the complex coefficients (real and imaginary parts)
--            of the solution vector, err, rco, and res.
--
-- the operations of the monodromy factorization :
--
--   job     = 40 : display the menu of all available options;
--   job     = 41 : prompts the user for a witness set,
--                  stores the system in the systems container,
--                  and its solutions in the solutions container,
--                  and returns in a the dimension of the ambient space
--                  and in b the following two numbers:
--                    b[0] : dimension of the solution set;
--                    b[1] : degree of the solution set.
--   job     = 42 : takes the system and the solutions from the containers
--                  and initializes the sampling machine, on input,
--                  the dimension of the witness set must be in a;
--   job     = 43 : assigns the coefficient c[0] + c[1]*I to the
--                  b-th coefficient of the a-th slice;
--   job     = 44 : stores the c[0] + c[1]*I as random gamma constant
--                  in the PHCpack_Operations;
--   job     = 45 : compute a new witness set on the new slices;
--   job     = 46 : swaps slices and solution sets to turn back,
--                  if a[0] = 1, then we move forward,
--                  if a[0] = 0, then we go back;
--   job     = 47 : initializes Monodromy_Permutations with two numbers:
--                    a[0] : number of monodromy loops,
--                    b[0] : degree of the solution component to factor;
--   job     = 48 : store solutions in container to Monodromy_Permutations;
--   job     = 49 : compute permutation by last stored solution list,
--                  and return this new permutation in b;
--   job     = 50 : updates decomposition with a new permutation,
--                  a[0] must contain the dimension and b the permutation;
--   job     = 51 : writes the current decomposition.
--

-- ON RETURN :
--   0 if the operation was successful, otherwise something went wrong,
--   e.g.: job not in the right range.
