with text_io,integer_io;                use text_io,integer_io;
with Interfaces.C;                      use Interfaces.C;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Natural_Vectors;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_Matrices;         use Standard_Complex_Matrices;
with Standard_Complex_Matrices_io;      use Standard_Complex_Matrices_io;
with Standard_Complex_VecMats;          use Standard_Complex_VecMats;
with Assignments_in_Ada_and_C;          use Assignments_in_Ada_and_C;
with Pieri_Homotopy;

function use_c2pieri ( job : integer;
                       a : C_intarrs.Pointer;
		       b : C_intarrs.Pointer;
                       c : C_dblarrs.Pointer ) return integer is

  procedure Write_Menu is
  begin
    new_line;
    put_line("General MENU to use Pieri homotopies from C :");
    put_line("  0. display this menu;");
    put_line("  1. initialize dimensions (m,p,q);");
    put_line("  2. initialize m*p + q*(m+p) input m-planes;");
    put_line("  3. initialize m*p + q*(m+p) interpolation points;");
    put_line("  4. store pivots of pattern at start solution curve;");
    put_line("  5. store pivots of pattern at target solution curve;");
    put_line("  6. store coefficients of start solution curve;");
    put_line("  7. retrieve coefficients of target solution curve;");
    put_line("  8. track solution path without intermediate output;");
    put_line("  9. track solution path with output diagnostics;");
    put_line(" 10. verify intersection conditions without output;");
    put_line(" 11. verify intersection conditions with extra output;");
    put_line(" 12. destroy the state machine.");
  end Write_Menu;

  procedure Get_Dimensions
              ( a : C_intarrs.Pointer; m,p,q : out natural ) is

  -- DESCRIPTION :
  --   Extracts the dimension (m,p,q) from the array a.

    v : C_Integer_Array(0..2)
      := C_intarrs.Value(a,Interfaces.C.ptrdiff_t(3));

  begin
    m := natural(v(0));
    p := natural(v(1));
    q := natural(v(2));
   -- put("Ada: m = "); put(m,1);
   -- put("  p = "); put(p,1);
   -- put("  q = "); put(q,1); new_line;
  end Get_Dimensions;

  procedure Get_All_Dimensions
              ( a,b : C_intarrs.Pointer; m,p,q,n : out natural ) is

  -- DESCRIPTION : 
  --   Calls Get_Dimensions on the array a and extracts n from b.

    v : C_Integer_Array(0..0) := C_intarrs.Value(b);

  begin
    Get_Dimensions(a,m,p,q);
    n := natural(v(0));
   -- put("Ada: n = "); put(n,1); new_line;
  end Get_All_Dimensions;

  procedure Get_and_Initialize_Input_Planes
              ( c : C_dblarrs.Pointer; m,p,q,n : in natural ) is

  -- DESCRIPTION :
  --   Extracts the coefficients of the input planes from c
  --   and initializes the Pieri homotopy state machine with it.

    nbcff : constant natural := 2*(m+p)*m*n;
    d : Interfaces.C.size_t := Interfaces.C.size_t(nbcff);
    v : C_Double_Array(0..d-1)
      := C_dblarrs.Value(c,Interfaces.C.ptrdiff_t(d));
    ind : Interfaces.C.size_t := 0;
    planes : VecMat(1..n);

  begin
   -- put_line("The coefficients of the input planes : ");
   -- for i in v'range loop
   --   put(double_float(v(i))); new_line;
   -- end loop;
    for k in planes'range loop
      declare
        plane : Matrix(1..m+p,1..m);
      begin
        for i in 1..m+p loop
          for j in 1..m loop
            plane(i,j) := Create(double_float(v(ind)),
                                 double_float(v(ind+1)));
            ind := ind + 2;
          end loop;
        end loop;
        planes(k) := new Matrix'(plane);
       -- put("input plane "); put(k,1); put_line(" :"); put(plane);
      end;
    end loop;
    Pieri_Homotopy.Initialize_Input_Planes(planes);
  end Get_and_Initialize_Input_Planes;

  procedure Get_and_Initialize_Interpolation_Points
              ( c : C_dblarrs.Pointer; m,p,q,n : in natural ) is

  -- DESCRIPTION :
  --   Extracts the coefficients of the interpolation points from c
  --   and initializes the Pieri homotopy state machine with it.

    nbcff : constant natural := 2*n;
    d : Interfaces.C.size_t := Interfaces.C.size_t(nbcff);
    v : C_Double_Array(0..d-1)
      := C_dblarrs.Value(c,Interfaces.C.ptrdiff_t(d));
    ind : Interfaces.C.size_t := 0;
    points : Standard_Complex_Vectors.Vector(1..n);
    
  begin
   -- put_line("The coefficients of the interpolation points : ");
   -- for i in v'range loop
   --   put(double_float(v(i))); new_line;
   -- end loop;
    for k in points'range loop
      points(k) := Create(double_float(v(ind)),double_float(v(ind+1)));
      ind := ind + 2;
    end loop;
   -- put_line("The interpolation points : "); put_line(points);
    Pieri_Homotopy.Initialize_Interpolation_Points(points);
  end Get_and_Initialize_Interpolation_Points;

  function Get_Complex_Vector
              ( c : C_dblarrs.Pointer; n : in natural )
              return Standard_Complex_Vectors.Vector is

  -- DESCRIPTION :
  --   Extracts from c the coefficients of a complex n-vector.

    res : Standard_Complex_Vectors.Vector(1..n);
    nbcff : constant natural := 2*n;
    d : Interfaces.C.size_t := Interfaces.C.size_t(nbcff);
    v : C_Double_Array(0..d-1)
      := C_dblarrs.Value(c,Interfaces.C.ptrdiff_t(d));
    ind : Interfaces.C.size_t := 0;
    
  begin
   -- put_line("The coefficients of the start plane : ");
   -- for i in v'range loop
   --   put(double_float(v(i))); new_line;
   -- end loop;
    for k in res'range loop
      res(k) := Create(double_float(v(ind)),double_float(v(ind+1)));
      ind := ind + 2;
    end loop;
    return res;
  end Get_Complex_Vector;

  function Get_Number ( a : C_intarrs.Pointer ) return natural is

  -- DESCRIPTION :
  --   Extracts a number from the array a and returns its value.

    v : C_Integer_Array(0..0) := C_intarrs.Value(a);
    p : constant natural := natural(v(0));

  begin
    return p;
  end Get_Number;

  procedure Get_Pivots
              ( a : in C_intarrs.Pointer; p : in natural;
                top,bottom : out Standard_Natural_Vectors.Vector ) is

 
    d : Interfaces.C.size_t := Interfaces.C.size_t(2*p);
    vp : C_Integer_Array(0..d-1)
       := C_intarrs.Value(a,Interfaces.C.ptrdiff_t(d));
    ind : Interfaces.C.size_t := 0;

  begin
    for i in 1..p loop
      top(i) := natural(vp(ind));
      ind := ind + 1;
    end loop;
    for i in 1..p loop
      bottom(i) := natural(vp(ind));
      ind := ind + 1;
    end loop;
  end Get_Pivots;

  procedure Write_Pivots
              ( top,bottom : in Standard_Natural_Vectors.Vector ) is
  begin
    put("Ada: [");
    put(top);
    put(" ],[");
    put(bottom);
    put_line(" ]");
  end Write_Pivots;

begin
  case job is
    when 0 => Write_Menu;
    when 1 => declare
                m,p,q : natural;
              begin
                Get_Dimensions(a,m,p,q);
                Pieri_Homotopy.Initialize_Dimensions(m,p,q);
              end;
    when 2 => declare
                m,p,q,n : natural;
              begin
                Get_All_Dimensions(a,b,m,p,q,n);
                Get_and_Initialize_Input_Planes(c,m,p,q,n);
              end;
    when 3 => declare
                m,p,q,n : natural;
              begin
                Get_All_Dimensions(a,b,m,p,q,n);
                Get_and_Initialize_Interpolation_Points(c,m,p,q,n);
              end;
    when 4 => declare
                p : constant natural := Get_Number(a);
                top,bottom : Standard_Natural_Vectors.Vector(1..p);
              begin
                Get_Pivots(b,p,top,bottom);
               -- Write_Pivots(top,bottom);
                Pieri_Homotopy.Store_Start_Pivots(top,bottom);
              end;
    when 5 => declare
                p : constant natural := Get_Number(a);
                top,bottom : Standard_Natural_Vectors.Vector(1..p);
              begin
                Get_Pivots(b,p,top,bottom);
               -- Write_Pivots(top,bottom);
                Pieri_Homotopy.Store_Target_Pivots(top,bottom);
              end;
    when 6 => declare
                n : constant natural := Get_Number(a);            
                x : Standard_Complex_Vectors.Vector(1..n);
              begin
               -- put("The number of variables : "); put(n,1); new_line;
                if n > 0
                 then x := Get_Complex_Vector(c,n);
                end if;
                Pieri_Homotopy.Store_Start(x);
              end;
    when 7 => declare
                n : constant natural := Get_Number(a);
                x : Standard_Complex_Vectors.Vector(1..n);
              begin
                Pieri_Homotopy.Retrieve_Target(x);
                Assign(x,c);
              end;
    when 8 => Pieri_Homotopy.Track_Path;
    when 9 => Pieri_Homotopy.Track_Path(Standard_Output);
    when 10 => declare
                 res : double_float := Pieri_Homotopy.Verify_Determinants;
               begin
                 Assign(res,c);
               end;
    when 11 => declare
                 res : double_float
                     := Pieri_Homotopy.Verify_Determinants(Standard_Output);
               begin
                 Assign(res,c);
               end;
    when 12 => Pieri_Homotopy.Clear;
    when others => put_line("  Sorry.  Invalid operation.");
                   return 1;
  end case;
  return 0;
end use_c2pieri;
