with text_io,integer_io;                use text_io,integer_io;
with Interfaces.C;                      use Interfaces.C;
with Communications_with_User;          use Communications_with_User;
with Standard_Integer_Vectors;
with Standard_Integer_Vectors_io;       use Standard_Integer_Vectors_io;
with Standard_Floating_Vectors;
with Lists_of_Floating_Vectors;         use Lists_of_Floating_Vectors;
with Arrays_of_Floating_Vector_Lists;   use Arrays_of_Floating_Vector_Lists;
with Floating_Mixed_Subdivisions;       use Floating_Mixed_Subdivisions;
with Floating_Mixed_Subdivisions_io;    use Floating_Mixed_Subdivisions_io;
with Floating_Lifting_Utilities;        use Floating_Lifting_Utilities;
with Cells_Container;                   use Cells_Container;
with Assignments_in_Ada_and_C;          use Assignments_in_Ada_and_C;

function use_celcon ( job : integer;
                      a : C_intarrs.Pointer;
                      b : C_intarrs.Pointer;
                      c : C_dblarrs.Pointer ) return integer is

  function Select_Point ( l : List; k : natural )
                        return Standard_Floating_Vectors.Link_to_Vector is

  -- DESCRIPTION :
  --   Returns the k-th point from the list l.

    res : Standard_Floating_Vectors.Link_to_Vector;
    tmp : List := l;

  begin
    for i in 1..(k-1) loop
      exit when Is_Null(tmp);
      tmp := Tail_Of(tmp);
    end loop;
    if not Is_Null(tmp)
     then res := Head_Of(tmp);
    end if;
    return res;
  end Select_Point;

begin
  case job is
    when 0 =>
      declare
        file : file_type;
        n,m,r : natural;
        mix : Standard_Integer_Vectors.Link_to_Vector;
        lif : Link_to_Array_of_Lists;
        sub : Mixed_Subdivision;
      begin
        new_line;
        put_line("Reading the file name for a mixed-cell configuration.");
        Read_Name_and_Open_File(file);
        get(file,n,m,mix,sub);
        close(file);
        r := mix'last;
        lif := new Array_of_Lists'(Lifted_Supports(r,sub));
        Cells_Container.Initialize(mix,lif,sub);
      end;
      return 0;
    when 1 =>
      declare
        mcc : Mixed_Subdivision := Cells_Container.Retrieve;
        mix : Standard_Integer_Vectors.Link_to_Vector;
        n,mv : natural;
      begin
        if not Is_Null(mcc)
         then n := Cells_Container.Dimension;
              mix := Cells_Container.Type_of_Mixture;
              put(standard_output,n-1,mix.all,mcc,mv);
              put("The mixed volume is "); put(mv,1); put_line(".");
        end if;
      end;
      return 0;
    when 2 => Assign(Cells_Container.Length,a); return 0;
    when 3 => Assign(Cells_Container.Dimension,a); return 0;
    when 4 =>
      declare
        use Standard_Integer_Vectors;
        mix : Link_to_Vector;
        r : natural;
      begin
        mix := Cells_Container.Type_of_Mixture;
        if mix /= null
         then r := mix'last;
              Assign(mix.all,b);
         else r := 0;
        end if;
        Assign(r,a);
      end;
      return 0;
    when 5 =>
      declare
        lif : Link_to_Array_of_Lists;
        r : natural;
      begin
        lif := Cells_Container.Lifted_Supports;
        if lif /= null then
          r := lif'last;
          declare
            nl : Standard_Integer_Vectors.Vector(1..r);
          begin
            for i in nl'range loop
              nl(i) := Length_Of(lif(i));
            end loop;
            Assign(nl,b);
          end;
        else
          r := 0;
        end if;
        Assign(r,a);
      end;
      return 0;
    when 6 =>
      declare
        lif : Link_to_Array_of_Lists;
        v_a : C_Integer_Array(0..0) := C_intarrs.Value(a);
        v_b : C_Integer_Array(0..0) := C_intarrs.Value(b);
        k_a : constant natural := natural(v_a(0));
        k_b : constant natural := natural(v_b(0));
        use Standard_Floating_Vectors;
        lpt : Link_to_Vector;
      begin
        lif := Cells_Container.Lifted_Supports;
       -- put("retrieving point "); put(k_b,1);
       -- put(" from list "); put(k_a,1); put_line(" ...");
        if lif /= null then
          lpt := Select_Point(lif(k_a),k_b);
          if lpt /= null then
            Assign(lpt.all,c);     
            return 0;
          end if;
        end if;
        return 1;
      end;
    when 7 =>
      declare
        v : C_Integer_Array(0..0) := C_intarrs.Value(a);
        k : constant natural := natural(v(0));
        mic : Mixed_Cell;
        fail : boolean;
      begin
        Cells_Container.Retrieve(k,mic,fail);
        if fail
         then return 1;
         else Assign(mic.nor.all,c);
              return 0;
        end if;
      end;
    when 8 =>
      declare
        v : C_Integer_Array(0..0) := C_intarrs.Value(a);
        k : constant natural := natural(v(0));
        mic : Mixed_Cell;
        fail : boolean;
      begin
        Cells_Container.Retrieve(k,mic,fail);
        if not fail then
          declare
            nl : Standard_Integer_Vectors.Vector(mic.pts'range);
          begin
            for i in mic.pts'range loop
              nl(i) := Length_Of(mic.pts(i));
            end loop;
            Assign(nl,b);
          end;
          return 0;
        end if;
        return 1;
      end;
    when 9 =>
      declare
        v_a : C_Integer_Array(0..0) := C_intarrs.Value(a);
        v_b : C_Integer_Array(0..1) := C_intarrs.Value(b);
        k : constant natural := natural(v_a(0));
        i : constant natural := natural(v_b(0));
        j : constant natural := natural(v_b(1));
        mic : Mixed_Cell;
        fail : boolean;
        use Standard_Floating_Vectors;
        lpt : Link_to_Vector;
      begin
        Cells_Container.Retrieve(k,mic,fail);
        if not fail then
          lpt := Select_Point(mic.pts(i),j);
          if lpt /= null
           then Assign(lpt.all,c);
                return 0;
          end if;
        end if;
        return 1;
      end;
    when 10 =>
      declare
        v : C_Integer_Array(0..0) := C_intarrs.Value(a);
        k : constant natural := natural(v(0));
        mix : Standard_Integer_Vectors.Link_to_Vector
            := Cells_Container.Type_of_Mixture;
        mic : Mixed_Cell;
        fail : boolean;
        mv : natural;
        n : constant natural := Cells_Container.Dimension-1;
        use Standard_Integer_Vectors;
      begin
        Cells_Container.Retrieve(k,mic,fail);
        if fail or mix = null
         then return 1;
         else Mixed_Volume(n,mix.all,mic,mv);
              Assign(mv,b);
              return 0;
        end if;
      end;
    when 11 =>
      declare
        v : C_Integer_Array(0..0) := C_intarrs.Value(a);
        r : constant natural := natural(v(0));
        mix : Standard_Integer_Vectors.Link_to_Vector
            := new Standard_Integer_Vectors.Vector(1..r);
        r1 : Interfaces.C.size_t := Interfaces.C.size_t(r-1);
        mbv : C_Integer_Array(0..r1) := C_Intarrs.Value(b);
      begin
        for i in 0..r-1 loop
          mix(i+1) := integer(mbv(Interfaces.C.size_t(i)));
        end loop;
        Cells_Container.Initialize(mix);
        return 0;
      end;
    when 12 =>
      declare
        va : C_Integer_Array(0..0) := C_intarrs.Value(a);
        vb : C_Integer_Array(0..0) := C_intarrs.Value(b);
        k : constant natural := natural(va(0));
        n : constant natural := natural(vb(0));
        x : Standard_Floating_Vectors.Vector(1..n);
        fail : boolean;
      begin
        Assign(n,c,x);
        fail := Cells_Container.Append_to_Support(k+1,x);
        if fail
         then return 1;
         else return 0;
        end if;
      end;
    when 13 =>
      declare
        va : constant C_Integer_Array(0..2) := C_intarrs.Value(a);
        r : constant natural := natural(va(0));
        n : constant natural := natural(va(1));
        d : Interfaces.C.size_t := Interfaces.C.size_t(va(2));
        vb : constant C_Integer_Array(0..d-1) := C_intarrs.Value(b);
        m : constant natural := natural(vb(0));
        x : Standard_Floating_Vectors.Vector(1..n);
        cnt : Standard_Integer_Vectors.Vector(1..r);
        lab : Standard_Integer_Vectors.Vector(1..m);
      begin
       -- put("  r = "); put(r,1);
       -- put("  n = "); put(n,1);
       -- put("  d = "); put(natural(d),1); new_line;
        for i in cnt'range loop
          cnt(i) := integer(vb(Interfaces.C.size_t(i)));
        end loop;
        for i in lab'range loop
          lab(i) := integer(vb(Interfaces.C.size_t(i+r)));
        end loop;
       -- put("the number of points in each support :"); put(cnt); new_line;
       -- put("the labels of points in each support :"); put(lab); new_line;
        Assign(n,c,x);
        Cells_Container.Append_Mixed_Cell(cnt,lab,x);
        return 0;
      end;
    when 14 => Cells_Container.Clear; return 0;
    when 15 => 
      declare
        va : constant C_Integer_Array(0..0) := C_intarrs.Value(a);
        k : constant natural := natural(va(0));
        cnt,lab : Standard_Integer_Vectors.Link_to_Vector;
        normal : Standard_Floating_Vectors.Link_to_Vector;
        fail : boolean;
      begin
        Cells_Container.Retrieve_Mixed_Cell(k,fail,cnt,lab,normal);
        if fail then
          return 1;
        else
          declare
            sum : constant integer := Standard_Integer_Vectors.Sum(cnt.all);
            cntlab : Standard_Integer_Vectors.Vector(1..cnt'last+lab'last+1);
            ind : natural := 0;
          begin
            ind := ind + 1;
            cntlab(ind) := sum;
            for i in cnt'range loop
              ind := ind + 1;
              cntlab(ind) := cnt(i);
            end loop;
            for i in lab'range loop
              ind := ind + 1;
              cntlab(ind) := lab(i);
            end loop;
           -- put("cntlab = "); put(cntlab); new_line;
            Assign(cntlab,b);
          end;
          Assign(normal.all,c);
          return 0;
        end if;
      end;
    when others => put_line("invalid operation"); return 1;
  end case;
end use_celcon;
