with C_Integer_Arrays;                  use C_Integer_Arrays;
with C_Double_Arrays;                   use C_Double_Arrays;

function use_solcon ( job : integer;
                      a : C_intarrs.Pointer;
                      b : C_intarrs.Pointer;
                      c : C_dblarrs.Pointer ) return integer;

-- DESCRIPTION :
--   Provides a gateway to the solutions container.

-- ON ENTRY :
--   job      = 0 : read solutions from file and put in container;
--            = 1 : write solutions in the container;
--            = 2 : return in b the length of the container;
--            = 3 : return in b the dimension of the solution vectors;
--            = 4 : return in b the multiplicity m of the solution (# in a),
--                         in c the solution as an array of 2*n+5 doubles,
--            in the following order:
--                  two doubles for the complex continuation parameter t,
--                  2*n doubles for the coefficients of the solution vector,
--                  one double for the norm of last Newton update,
--                  one double for the inverse of condition# estimate,
--                  one double for the norm of the residual;
--            = 5 : changes the solution with number in to the values 
--                  in b and c, where b is the array (n,m),
--                  where n is the dimension and m the multiplicity;
--            = 6 : appends the solution with data in b  = (n,m) and
--                  (also see job = 4 for info) to the container;
--            = 7 : clears all solutions from the container;
--   a        indicates the number of solution to work on;
--   b        array with allocated memory for integers,
--            usually only len, n, m, or the array (n,m),
--            where len is the length of the container;
--                  n is the dimension of the solution vectors; and
--                  m is the multiplicity of the solution;
--   c        array with allocated memory for double floats to store
--            the complex value for the continuation paramter t,
--            the complex coefficients (real and imaginary parts)
--            of the solution vector, err, rco, and res.

-- ON RETURN :
--   0 if operation was successful, otherwise something went wrong,
--   like accessing a solution which is not there or having a job
--   number outside the range 0..7.
