with text_io,integer_io;               use text_io,integer_io;
with Characters_and_Numbers;           use Characters_and_Numbers;
with Standard_Floating_Numbers;        use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;     use Standard_Floating_Numbers_io;
with Standard_Complex_Vectors;         use Standard_Complex_Vectors;
with Standard_Random_Vectors;          use Standard_Random_Vectors;
with Standard_Complex_Matrices;        use Standard_Complex_Matrices;
with Symbol_Table;
with Standard_Complex_Polynomials;     use Standard_Complex_Polynomials;
with Standard_Complex_Polynomials_io;  use Standard_Complex_Polynomials_io;
with Standard_Complex_Poly_Functions;  use Standard_Complex_Poly_Functions;
with Standard_Complex_Poly_Systems;    use Standard_Complex_Poly_Systems;
with Standard_Complex_Solutions;       use Standard_Complex_Solutions;
with Hypersurfaces_and_Filters;        use Hypersurfaces_and_Filters;
with Intrinsic_Witness_Sets_io;        use Intrinsic_Witness_Sets_io;

procedure maindiff ( filename : in string ) is

  procedure Write_Witness_Set
              ( name : in string; n : natural; p : in Poly;
                b,v : in Vector; s : in Solution_List ) is

    dimension : constant string := Convert(n-1);
    output_name : constant string := name & "_w" & dimension;
    output_file : file_type;
    f : Poly_Sys(1..1);
    plane : Matrix(1..n,0..1);

  begin
    put_line("writing to " & output_name);
    f(1) := p;
    for i in b'range loop
      plane(i,0) := b(i);
      plane(i,1) := v(i);
    end loop;
    create(output_file,out_file,output_name);
    Write_Witness_Set(output_file,n,n-1,f,s,plane);
  end Write_Witness_Set;

  procedure Hypersurface_Witness_Set
               ( name : in string; n : in natural; p : in Poly ) is

  -- DESCRIPTION :
  --   Computes a witness set for the hypersurface defined by p,
  --   and writes this to the file "filename" & wk, where k is n-1.

    d : constant natural := Degree(p);
    ep : Eval_Poly := Create(p);
    b,v : Vector(1..n);
    s : Solution_List;
    res : double_float;

  begin
    b := Random_Vector(1,n); v := Random_Vector(1,n);
    SP_Hypersurface_Witness_Set(n,d,ep,b,v,s,res);
    put("The residual : "); put(res,3); new_line;
    Write_Witness_Set(name,n,p,b,v,s);
  end Hypersurface_Witness_Set;

  procedure Witness_Set ( file : in file_type; name : in string) is

  -- DESCRIPTION :
  --   Reads a system from file and computes a witness set.

    nb_equ,nb_var : natural;
    p : Poly;
    ep : Eval_Poly;

  begin
    get(file,nb_equ);
    put("the number of equations : "); put(nb_equ,1); new_line;
    if nb_equ = 1
     then get(file,nb_var);
          put("the number of variables : "); put(nb_var,1); new_line;
          Symbol_Table.Init(nb_var);
          get(file,p);
          put("p = "); put(p); new_line;
          Hypersurface_Witness_Set(name,nb_var,p);
     else put_line("The general case is not yet implemented.");
    end if;
  end Witness_Set;

  procedure Main is

    file,input_file : file_type;
    action : integer;
    input_name : string(1..80);
    cnt : natural;

  begin
    put_line("maindiff called with " & filename); 
    open(file,in_file,filename);
    get(file,action);
    case action is
      when 1 => skip_line(file);
                get_line(file,input_name,cnt);
                put_line("the system is at " & input_name(1..cnt));
                open(input_file,in_file,input_name);
                put_line("computing the witness set ...");
                Witness_Set(input_file,input_name(1..cnt));
      when 2 => put_line("intersect witness sets");
      when 3 => put_line("apply membership test");
      when others => put_line("action not understood");
    end case;
  end Main;

begin
  Main;
end maindiff;                                                                  
