with text_io,integer_io;                  use text_io,integer_io;
with Communications_with_User;            use Communications_with_User;
with Standard_Complex_Numbers;            use Standard_Complex_Numbers;
with Standard_Natural_Vectors;
with Standard_Integer_Vectors;
with Standard_Integer_Vectors_io;         use Standard_Integer_Vectors_io;
with Symbol_Table,Symbol_Table_io;
with Standard_Complex_Polynomials;        use Standard_Complex_Polynomials;
with Standard_Complex_Polynomials_io;     use Standard_Complex_Polynomials_io;
with Permutations;                        use Permutations;
with Permute_Operations;                  use Permute_Operations;

procedure ts_prolong is

-- DESCRIPTION :
--   Test on reordering the variables in a polynomial happens
--   in two stages :
--     1) permuting the variables in the polynomial;
--     2) permuting the symbols in the table.

  procedure Write_Symbol_Table is

  -- DESCRPITION :
  --   Writes the content of the symbol table in the current order.
  begin
    for i in 1..Symbol_Table.Number loop
      Symbol_Table_io.put(Symbol_Table.Get(i)); put(" ");
    end loop;
    new_line;
  end Write_Symbol_Table;

  procedure Write_Symbol_Order ( perm : in Permutation ) is

  -- DESCRIPTION :
  --   Writes the symbols according to the given permutation.

  begin
    for i in 1..Symbol_Table.Number loop
      Symbol_Table_io.put(Symbol_Table.Get(perm(i))); put(" ");
    end loop;
  end Write_Symbol_Order;

  procedure Add_Diff_Variables is

  -- DESCRIPTION :
  --   For every symbol "x" in the symbol table, the symbol "xt"
  --   will be added to the symbol table.

    nb : constant natural := Symbol_Table.Number;
    use Symbol_Table;
    sb : Symbol;

  begin
    Symbol_Table.Enlarge(nb);
    for i in 1..nb loop
      sb := Symbol_Table.Get(i);
      for j in sb'range loop
        if sb(j) = ' '
         then sb(j) := 't';
              Symbol_Table.Add(sb);
              exit;
        end if;
      end loop;
    end loop;
  end Add_Diff_Variables;

  procedure Permute_Symbol_Table ( perm : in Permutation ) is

  -- DESCRIPTION :
  --   Permutes the symbols in the symbol table.

    nb : constant natural := Symbol_Table.Number;
    use Symbol_Table;
    syms : array(1..nb) of Symbol;

  begin
    for i in syms'range loop
      syms(i) := Symbol_Table.Get(i);
    end loop;
    for i in syms'range loop
      Symbol_Table.Replace(i,syms(perm(i)));
    end loop;
  end Permute_Symbol_Table;

  procedure Read_Permutation ( perm : out Permutation ) is

    nb : constant natural := Symbol_Table.Number;
    use Symbol_Table;
    sb : Symbol;
    error : boolean;

  begin
    for i in 1..nb loop
      loop
        put("Give symbol at place "); put(i,1); put(" : ");
        Symbol_Table_io.Get(sb);
        put("  Your symbol : "); Symbol_Table_io.put(sb);
        perm(i) := Symbol_Table.Get(sb);     
        error := false;
        if perm(i) = 0
         then put_line(" did not occur before!  Please try again...");
              error := true;
         else for j in 1..i-1 loop
                if perm(j) = perm(i)
                 then error := true;
                      put(" occurs already at new place ");
                      put(j,1); put_line("!  Please try again...");
                end if;
                exit when error;
              end loop;
              if not error
               then put(" was at position "); put(perm(i),1); new_line;
              end if;
        end if;
        exit when not error;
      end loop;
    end loop;
  end Read_Permutation;

  procedure Change_Order ( p : in out Poly ) is

  -- DESCRIPTION :
  --   Reorders the symbols in the polynomial.

    nbunk : constant natural := Number_of_Unknowns(p);
    perm : Standard_Integer_Vectors.Vector(1..nbunk);
    pp : Poly;
    ans : character;

  begin
    put("Number of unknowns  : "); put(nbunk,1); new_line; 
    put("The order of the unknowns : "); Write_Symbol_Table;
    put("Do you want to change this order ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then loop
            Read_Permutation(Permutation(perm));
            put("Your permutation : "); put(perm); new_line;
            put("The new order of symbols : ");
            Write_Symbol_Order(Permutation(perm));
            new_line;
            put("Happy with this new order ? (y/n) ");
            Ask_Yes_or_No(ans);
            exit when (ans = 'y');
          end loop;
          pp := Permutation(perm)*p;
          put("The permuted polynomial : "); put(pp); new_line;
          Permute_Symbol_Table(Permutation(perm));
          put("The permuted order : "); Write_Symbol_Table;
          put("The permuted polynomial : "); put(pp); new_line;
          Clear(p); p := pp;
    end if;
  end Change_Order;

  function Prolong ( p : Poly ) return Poly is

  -- DESCRIPTION :
  --   Given a polynomial whose independent variables depend on t,
  --   the derivative of p with the chain rule is returned.

    res : Poly := Null_Poly;
    n : constant natural := Number_of_Unknowns(p);

    procedure Prolong_Term ( t : in Term; cont : out boolean ) is

      dt : Term;

    begin
      dt.dg := new Standard_Natural_Vectors.Vector'(1..2*n => 0);
      dt.dg(t.dg'range) := t.dg.all;
      for i in 1..n loop
        if t.dg(i) > 0
         then dt.cf := t.cf*Create(t.dg(i));
              dt.dg(i) := t.dg(i)-1;
              dt.dg(i+n) := 1;
              Add(res,dt);
              dt.dg(i) := t.dg(i);
              dt.dg(i+n) := 0;
        end if;
      end loop;
      Clear(dt.dg);
      cont := true;
    end Prolong_Term;
    procedure Prolong_Terms is new Visiting_Iterator(Prolong_Term);

  begin
    Prolong_Terms(p);
    return res;
  end Prolong;

  procedure Prolong ( p : in Poly ) is

  -- DESCRIPTION :
  --   Enlarges the symbol table with the derivative variables,
  --   and computes the prolonged polynomial.

    pp : Poly; 

  begin
    Add_Diff_Variables;
    put("All symbols : "); Write_Symbol_Table;
    pp := Prolong(p);
    put("The prolonged polynomial : "); put_line(pp); new_line;
  end Prolong;

  procedure Main is

    n : natural;
    p : Poly;

  begin
    new_line;
    put_line("Prolong a polynomial into the first order jet space.");
    new_line;
    put("Give the number of variables : "); get(n);
    Symbol_Table.Init(n);
    put("Give a polynomial : "); get(p);
    Change_Order(p);
    Prolong(p);
  end Main;

begin
  Main;
end ts_prolong;
