with Standard_Complex_Numbers;            use Standard_Complex_Numbers;
with Standard_Floating_Vectors;

package body Hash_Table is

-- INTERNAL DATA consists of hash function and vector of keys :

  keycnt : natural;
  hash : Standard_Complex_Vectors.Link_to_Vector;
  keys : Standard_Floating_Vectors.Link_to_Vector;
  hashre,hashim : double_float;

-- AUXILIARIES :

  function Member_Key ( key,tol : double_float ) return natural is

  -- DESCRIPTION :
  --   Returns the index of the matching key with the given tolerance.
  --   If there is no such a key, then zero is returned.

  begin
    for i in 1..keycnt loop
      if abs(key - keys(i)) < tol
       then return i;
      end if;
    end loop;
    return 0;
  end Member_Key;

  procedure Add_Key ( key : in double_float ) is
  begin
    keycnt := keycnt + 1;
    keys(keycnt) := key;
  end Add_Key;

-- INITIALIZER :

  procedure Initialize 
              ( m : in natural; f : Vector; re,im : double_float ) is
  begin
    hash := new Standard_Complex_Vectors.Vector'(f);
    hashre := re;
    hashim := im;
    keycnt := 0;
    keys := new Standard_Floating_Vectors.Vector(1..m);
  end Initialize;

-- CONSTRUCTOR :

  procedure Add ( v : in Vector; tol : in double_float ) is

    keyv : constant double_float := Key(v);

  begin
    if Member_Key(keyv,tol) = 0
     then Add_Key(keyv);
    end if;
  end Add;

-- SELECTORS :

  function Number return natural is
  begin
    return keycnt;
  end Number;

  function Key ( v : Vector ) return double_float is

    res : double_float;
    hsv : Complex_Number := hash.all*v;

  begin
    res := REAL_PART(hsv)*hashre + IMAG_PART(hsv)*hashim;
    return res;
  end Key;

  function Member ( v : Vector; tol : double_float ) return natural is

    keyv : constant double_float := Key(v);

  begin
    return 0;
  end Member;

-- DESTRUCTOR : 

  procedure Clear is
  begin
    Standard_Complex_Vectors.Clear(hash);
    Standard_Floating_Vectors.Clear(keys);
    keycnt := 0;
  end Clear;

end Hash_Table;
