with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Complex_Vectors;          use Standard_Complex_Vectors;

package Hash_Table is

-- DESCRIPTION :
--   This package provides an implementation for a simple hash table of
--   complex vectors.

-- INITIALIZER :

  procedure Initialize ( m : in natural; f : in Vector;
                         re,im : in double_float );

  -- DESCRIPTION :
  --   Space is reserved for a hash table to process m complex vectors.
  --   The vector f must be of the same range as those vectors as it will
  --   be used to make keys for every vector added to the table.
  --   The "re" and "im" are two random floats used as multipliers in
  --   the hash function.

-- CONSTRUCTOR :

  procedure Add ( v : in Vector; tol : in double_float );

  -- DESCRIPTION :
  --   Add one entry in the hash table for the given vector, provided
  --   the key for the vector does not already belong to the table,
  --   within the given tolerance.

-- SELECTORS :

  function Number return natural;

  -- DESCRIPTION :
  --   Returns the number of slots currently filled in the table.

  function Key ( v : Vector ) return double_float;

  -- DESCRIPTION :
  --   Returns the key of the vector v.

  function Member ( v : Vector; tol : double_float ) return natural;

  -- DESCRIPTION :
  --   If Key(v) matches one of the keys in the table within the given
  --   tolerance, then the index to that key is returned; otherwise,
  --   if there is no matching key for v, 0 is returned.

-- DESTRUCTOR : 

  procedure Clear;

  -- DESCRIPTION :
  --   The hash table is descroyed.

end Hash_Table;
