with Standard_Integer_Matrices;          use Standard_Integer_Matrices;

package Smith_Normal_Form is

-- DESCRIPTION :
--   This package provides a diagonalization routine for a matrix of
--   standard integer numbers, along with some utilities.

  function Identity ( n : natural ) return Matrix;

  -- DESCRIPTION :
  --   Returns the n-by-n identity matrix.

  function Diagonal ( mat : Matrix ) return boolean;

  -- DESCRIPTION :
  --   Returns true if the matrix is diagonal, returns false otherwise.

  procedure Diagonalize ( u,a,v : in out Matrix );

  -- DESCRIPTION :
  --   Returns the Smith normal form of the matrix, a := u*a*v.

  -- ON ENTRY :
  --   u        identity matrix of range a'range(1);
  --   a        matrix that is to diagonalize;
  --   v        identity matrix of range a'range(2).

  -- ON RETURN :
  --   u        unimodular matrix of left multiplications;
  --   a        diagonal matrix, equals u*a*v;
  --   v        unimodular matrix of right multiplications.

end Smith_Normal_Form;
