with text_io,integer_io;                 use text_io,integer_io;
with Standard_Integer_Matrices;          use Standard_Integer_Matrices;
with Standard_Integer_Matrices_io;       use Standard_Integer_Matrices_io;
with Standard_Random_Matrices;           use Standard_Random_Matrices;
with Standard_Common_Divisors;           use Standard_Common_Divisors;
with Standard_Integer_Linear_Solvers;    use Standard_Integer_Linear_Solvers;  
with Smith_Normal_Form;                  use Smith_Normal_Form;

procedure ts_smith is

-- DESCRIPTION :
--   Development of Smith Normal Form of an integer matrix.

  procedure Test_Hermite_Normal_Form ( mat : in Matrix ) is

  -- DESCRIPTION :
  --   Test on the Hermite normal form of a matrix.

    wrk : Matrix(mat'range(1),mat'range(2));
    u : Matrix(mat'range(1),mat'range(1));
    v : Matrix(mat'range(2),mat'range(2));

  begin
    wrk := mat;
    Upper_Triangulate(u,wrk);
    put_line("The left U-matrix used in Hermite normal form : "); put(u);
    put_line("The matrix in upper triangular form : "); put(wrk);
    put_line("The left U-matrix times original matrix : "); put(u*mat);
    wrk := mat;
    Lower_Triangulate(wrk,v);
    put_line("The right U-matrix used in Hermite normal form : "); put(v);
    put_line("The matrix in lower triangular form : "); put(wrk);
    put_line("The right U-matrix times original matrix : "); put(mat*v);
  end Test_Hermite_Normal_Form;

  procedure Interactive_Test_Hermite_Normal_Form ( n,m : in natural ) is

  -- DESCRIPTION :
  --   Test Smith normal form on a given matrix.

    mat : Matrix(1..n,1..m);

  begin
    put("Give "); put(n,1); put("-by-"); put(m,1);
    put_line(" matrix : "); get(mat);
    put_line("Your matrix :"); put(mat);
    Test_Hermite_Normal_Form(mat);
  end Interactive_Test_Hermite_Normal_Form;

  procedure Test_Smith_Normal_Form
              ( mat : in Matrix; output : in boolean; bug : out boolean ) is

  -- DESCRIPTION :
  --   Test on the Smith normal form of a matrix.
  --   If output is switched on, then all intermediate information is printed,
  --   otherwise this is only the case when a bug is detected;

    wrk : Matrix(mat'range(1),mat'range(2)) := mat;
    u : Matrix(mat'range(1),mat'range(1)) := Identity(mat'last(1));
    v : Matrix(mat'range(2),mat'range(2)) := Identity(mat'last(2));
    multwrk : Matrix(mat'range(1),mat'range(2));

  begin
    Diagonalize(u,wrk,v);
    multwrk := u*mat;
    multwrk := multwrk*v;  
    bug := not ((multwrk = wrk) and Diagonal(wrk));
    if output or bug
     then put_line("The matrix to diagonalize : "); put(mat);
          put_line("The left U-matrix used in Smith normal form : "); put(u);
          put_line("The right U-matrix used in Smith normal form : "); put(v);
          put_line("The matrix in diagonal form : "); put(wrk);
          put_line("The left U times original times right U : "); put(multwrk);
    end if;
    if not bug
     then put_line("The test was successful.");
     else put_line("Bug detected!");
    end if;
  end Test_Smith_Normal_Form;

  procedure Interactive_Test_Smith_Normal_Form ( n,m : in natural ) is

  -- DESCRIPTION :
  --   Test Smith normal form on a given matrix.

    mat : Matrix(1..n,1..m);
    bug : boolean;

  begin
    put("Give "); put(n,1); put("-by-"); put(m,1);
    put_line(" matrix : "); get(mat);
    Test_Smith_Normal_Form(mat,true,bug);
  end Interactive_Test_Smith_Normal_Form;

  procedure Random_Test_Smith_Normal_Form ( n,m : in natural ) is

  -- DESCRIPTION :
  --   Test Smith normal form on randomly generated matrices.

    low,upp : integer;
    nbs : natural;
    mat : Matrix(1..n,1..m);
    bug : boolean;

  begin
    put("Give lower bound on numbers : "); get(low);
    put("Give upper bound on numbers : "); get(upp);
    put("Give the number of samples : "); get(nbs);
    for i in 1..nbs loop
      mat := Random_Matrix(n,m,low,upp);
      Test_Smith_Normal_Form(mat,false,bug);
      exit when bug;
    end loop;
    if not bug
     then put("Successfully tested "); put(nbs,1); put_line(" samples.");
     else put_line("Generated a case that shows there is a bug.");
    end if;
  end Random_Test_Smith_Normal_Form;

  procedure Main is

    n,m : natural;
    ans : character;

  begin
    loop
      new_line;
      put_line("MENU for testing Hermite and Smith normal forms : ");
      put_line("  0. Exit this program.");
      put_line("  1. Hermite normal form on a given matrix.");
      put_line("  2. Smith normal form on a given matrix.");
      put_line("  3. Smith normal form on a random matrix.");
      put("Type 0, 1, 2, or 3 : "); get(ans);
      exit when ans = '0';
      new_line;
      put("Give number of rows : "); get(n);
      put("Give number of columns : "); get(m); 
      case ans is
        when '1' => Interactive_Test_Hermite_Normal_Form(n,m);
        when '2' => Interactive_Test_Smith_Normal_Form(n,m);
        when '3' => Random_Test_Smith_Normal_Form(n,m);
        when others => null;
      end case;
    end loop;
  end Main;

begin
  new_line;
  put_line("Test on Hermite and Smith normal form of integer matrices.");
  Main;
end ts_smith;
