with text_io;                            use text_io;
with Standard_Floating_Numbers;          use Standard_Floating_Numbers;

package Standard_Floating_Numbers_io is

-- DESCRIPTION :
--   This package defines output of standard floating numbers.

  procedure get ( f : in out single_float );
  procedure get ( file : in file_type; f : in out single_float );
  procedure get ( f : in out double_float );
  procedure get ( file : in file_type; f : in out double_float );

  -- DESCRIPTION :
  --   Reads a floating-point number from standard input or from file.

  procedure put ( f : in single_float );
  procedure put ( file : in file_type; f : in single_float );
  procedure put ( f : in double_float );
  procedure put ( file : in file_type; f : in double_float );
  procedure put ( f : in double_float; fore,aft,exp : in natural );
  procedure put ( file : in file_type;
                  f : in double_float; fore,aft,exp : in natural );

  -- DESCRIPTION :
  --   Writes a floating-point number on standard output or on file.
  --   The triplet (fore,aft,exp) determines the output format, i.e.:
  --   they indicate the number of decimal places before and after
  --   the decimal point and the number of decimal places in the exponent.

  procedure put ( f : in double_float; dp : in natural );
  procedure put ( file : in file_type; f : in double_float; dp : in natural );

  -- DESCRIPTION : put(f,dp) = put(f,dp,dp,dp).

-- FLOATS in C format :

  procedure printf ( s : in string; f : in double_float );
  pragma Import(C,printf,"printf");

  procedure scanf ( s : in string; f : in out double_float );
  pragma Import(C,scanf,"scanf");

  -- DESCRIPTION :
  --   scanf is similar to the C scanf, s is a control string;
  --   printf writes a float in scientific format, using the C convention.

end Standard_Floating_Numbers_io;
