with integer_io;                         use integer_io;
with Characters_and_Numbers;             use Characters_and_Numbers;
with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Multprec_Integer_Numbers;           use Multprec_Integer_Numbers;
with Multprec_Floating_Numbers;          use Multprec_Floating_Numbers;
with Multprec_Floating_Numbers_io;       use Multprec_Floating_Numbers_io;
with Multprec_Complex_Numbers;           use Multprec_Complex_Numbers;
with Multprec_Complex_Numbers_io;        use Multprec_Complex_Numbers_io;
with Standard_Natural_Vectors;           use Standard_Natural_Vectors;
with Standard_Complex_Polynomials_io;
with Symbol_Table_io;

package body Multprec_Complex_Polynomials_io is

-- AUXILIARIES FOR THE INPUT ROUTINES :

  procedure Build_Exponent ( file : in file_type;
                             char : in out character; i : out integer;
                             ni : out natural; sign : out character ) is
  -- DESCRIPTION :
  --   characters are read from the input and a number is build up;
  --   the result is the number i.
  
  -- ON ENTRY :
  --   file         file type of a file that must be opened for input;
  --   char         first character to be analized.
 
  -- ON RETURN :
  --   char         first character that is not a digit;
  --   i1           digits read;
  --   ni           number of digits in i;
  --   sign         sign of the number.

    res : integer := 0;
    min : boolean := false;
    cnt,temp : natural := 0;

  begin
    sign := '+';
    while (char = ' ') loop get(file,char); end loop;
    if (char = '+') or (char = '-')
     then min := (char = '-');
          sign := char;
          get(file,char); 
    end if;
    while (char = ' ') loop get(file,char); end loop;
    loop
      temp := Convert(char);
      if temp < 10
       then if cnt < 9
             then res := res*10 + temp;
                  cnt := cnt + 1;
             else null;
            end if;
            get(file,char);
       else exit;
      end if;
    end loop;
    if min
     then i := -res;
     else i := res;
    end if;
    ni := cnt;
  end Build_Exponent;

  procedure Build_Number ( file : in file_type; char : in out character;
                           i : out Integer_Number;
                           n : out natural; sign : out character ) is

  -- DESCRIPTION :
  --   characters are read from the input and a number is build up
  
  -- ON ENTRY :
  --   file         file type of a file that must be opened for input;
  --   char         first character to be analized.
 
  -- ON RETURN :
  --   char         first character that is not a digit;
  --   i            integer number read;
  --   n            number of digits in i;
  --   sign         sign of the number.

    res : Integer_Number := Create(0);
    min : boolean := false;
    cnt,temp : natural := 0;

  begin
    sign := '+';
    while (char = ' ') loop get(file,char); end loop;
    if (char = '+') or (char = '-')
     then min := (char = '-');
          sign := char;
          get(file,char); 
    end if;
    while (char = ' ') loop get(file,char); end loop;
    loop
      temp := Convert(char);
      if temp < 10
       then Mul(res,10);
            Add(res,temp);
            cnt := cnt + 1;
            get(file,char);
       else exit;
      end if;
    end loop;
    if min
     then Multprec_Integer_Numbers.Min(res);
    end if;
    i := res;
    n := cnt;
  end Build_Number;

  procedure Build_Number ( file : in file_type; char : in out character;
                           f : out Floating_Number ) is

  -- DESCRIPTION :
  --   a floating point number is read

    int,quot,expo : Integer_Number;
    f_int,f_quot,acc : Floating_Number;
    nq,np,ne,temp : natural := 0;
    sign : character;
    min : boolean;

  begin
    Build_Number(file,char,int,np,sign);
    f_int := Create(int);
    min := (sign = '-');
    case char is
      when '.'    => get(file,char);       -- skip the point
                     temp := Convert(char);
                     if temp < 10
                      then Build_Number(file,char,quot,nq,sign);
                           f_quot := Create(quot);
                     end if;
                     if char = 'E'
                      then get(file,char); -- skip the 'E'
                           Build_Number(file,char,expo,ne,sign);
                     end if;
      when 'E'    => if char = 'E'
                      then get(file,char); -- skip the 'E'
                           Build_Number(file,char,expo,ne,sign);
                     end if;
      when others => null;
    end case; 
    if min
     then if Equal(f_int,0.0) and Equal(f_quot,0.0) and (nq = 0) and (np = 0)
           then f := Create(-1);   --  "-x" = -1*x 
           else --f := ( f_int - f_quot*10.0**(-nq) )*10.0**expo ;
                acc := f_quot*10.0**(-nq);
                Sub(f_int,acc); Clear(acc);
                acc := 10.0**expo;
                Mul(f_int,acc); Clear(acc);
                f := f_int;
          end if;
     else --f := ( f_int + f_quot*10.0**(-nq) )*10.0**expo ;
          acc := f_quot*10.0**(-nq);
          Add(f_int,acc); Clear(acc);
          acc := 10.0**expo;
          Mul(f_int,acc); Clear(acc);
          f := f_int;
    end if;
  end Build_Number;

  procedure Build_Number ( file : in file_type;
                           char : in out character; c : out Complex_Number ) is
  -- DESCRIPTION :
  --   a floating point number is read and converted into a complex number;
  --   the number may be the quotient of two floating point numbers
 
    f1,f2 : Floating_Number;

  begin
    Build_Number(file,char,f1);
    if char = '/'
     then get(file,char);            -- skip the '/'
          Build_Number(file,char,f2);
          c := Create(f1/f2);
     else c := Create(f1);
    end if;
  exception
    when numeric_error => raise INFINITE_NUMBER;
  end Build_Number;

  procedure Read_Term ( file : in file_type; char : in out character;
                        n : in natural; termp : in out Poly );
  -- DESCRIPTION :
  --   Reads a term from file, char is the first character of the term.

  procedure Read_Factor ( file : in file_type;
                          char : in out character; n : in natural;
                          d : in out Degrees; pb : in out Poly );
  -- DESCRIPTION :
  --   Reads a factor from file, char is the first character of the factor.

  procedure Read_Factor ( file : in file_type;
                          char : in out character; n : in natural;
                          d : in out Degrees; pb : in out Poly ) is

    sb : symbol;
    i : positive := 1;
    k,ne : natural := 0;
    expo : integer := 1;
    sign : character;
 
  begin
    sb := (sb'range => ' ');
    while (char = ' ') loop get(file,char); end loop;
    if char = '('
     then get(file,pb);
          get(file,char);       -- get a new symbol, skip '('
          return;
    end if;
   -- read the symbol :
    loop
      case char is
        when '+' | '-' | '*' | '^' => exit;
        when delimiter | ' ' | ')' => exit;
        when '('                   => raise ILLEGAL_SYMBOL;
        when others                => sb(i) := char;
                                      i := i+1; get(file,char);
      end case;
    end loop;
   -- check for legality of the symbol :
    if Convert(sb(1)) < 10
     then raise ILLEGAL_SYMBOL;
     else for j in 2..3 loop
            case sb(j) is
              when '*' | '+' | '-' | '^' | '/' | ';' | '(' | ')'
                => raise ILLEGAL_SYMBOL;
              when others => null;
            end case;
          end loop;
    end if;
   -- search for the number of the symbol :
    k := Symbol_Table.get(sb);
    if k = 0
     then declare
          begin
            Symbol_Table.add(sb,k);
          exception
            when OVERFLOW_IN_THE_SYMBOL_TABLE => raise OVERFLOW_OF_UNKNOWNS;
          end;
    end if;
    if k > n
     then raise OVERFLOW_OF_UNKNOWNS;
    end if;
   -- read further :
    while (char = ' ') loop get(file,char); end loop;
    if char = '^'
     then get(file,char);                                    -- skip the '^'
          Build_Exponent(file,char,expo,ne,sign);
          d(k) := d(k) + natural(expo);
          while char = ' ' loop get(file,char); end loop;
          if char /= '*'                            -- the case x^2*...
           then return;                             -- end of factor
           else get(file,char);                     -- skip the '*'
          end if; 
     elsif char = '*'
         then get(file,char);
              if char = '*'
               then get(file,char);                 -- the case " x ** expo "
                    Build_Exponent(file,char,expo,ne,sign);
                    d(k) := d(k) + natural(expo);
                    while (char = ' ') loop get(file,char); end loop;
                    if char /= '*'
                     then return;                   -- end of factor
                     else get(file,char);           -- skip the '*'
                    end if;
               else d(k) := d(k) + 1;               -- the case " x * ? "
              end if;
         else -- the case " x ?", with ? /= '*' or ' ' or '^' :
              d(k) := d(k) + 1;
              return;
    end if;
    while (char = ' ') loop get(file,char); end loop;
    if (char = '-') or (char = '+') 
     then return;
    end if;
    if Convert(char) < 10
     then -- the case " x * c " or " x ** c * c " :
          Read_Term(file,char,n,pb);
     else -- the case " x * y " :
          Read_Factor(file,char,n,d,pb);
    end if;
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end Read_Factor;

  function Create ( f : double_float ) return Complex_Number is

    ff : Floating_Number := Create(f);

  begin
    return Create(ff);
  end Create;

  function Create ( re,im : double_float ) return Complex_Number is

    f_re : Floating_Number := Create(re);
    f_im : Floating_Number := Create(im);

  begin
    return Create(f_re,f_im);
  end Create;
 
  procedure Read_Term ( file : in file_type; char : in out character;
                        n : in natural; termp : in out Poly ) is

    c : Complex_Number;
    d : Degrees := new Standard_Natural_Vectors.Vector'(1..n => 0);
    pb,res,temp : Poly;
    tmp : Term;
    realzero : Floating_Number := Create(0);
    realmin1 : Floating_Number := Create(-1);
    compzero : Complex_Number := Create(realzero);
    compmin1 : Complex_Number := Create(realmin1);

    procedure Collect_Factor_Polynomial is
    begin
      if pb  /= Null_Poly
       then if res = Null_Poly
             then Copy(pb,res); Clear(pb);
             else Mul(res,pb); Clear(pb);
            end if;
      end if;
    end Collect_Factor_Polynomial;

  begin
    Build_Number(file,char,c);
   
   -- look for 'i' :

    while (char = ' ') loop get(file,char); end loop;
  
    if ( Equal(c,compzero) ) and then (char = 'i')
     then -- the case "+ i" :
          c := Create(0.0,1.0); 
          get(file,char);        -- skip 'i'
     elsif ( Equal(c,compmin1) ) and then (char = 'i')
         then -- the case "- i" :
              c := Create(0.0,-1.0);
              get(file,char);    -- skip 'i'
         elsif char = '*'
             then -- the case ".. c *.." :
                  while (char = ' ') loop get(file,char); end loop;
                  get(file,char);  -- skip '*'
                  while (char = ' ') loop get(file,char); end loop;
                  if char = 'i'
                   then -- the case ".. c * i.." :
                        c := Create(realzero,REAL_PART(c));
                        get(file,char);    -- skip 'i'
                   else -- the case ".. c * x.." :
                        Read_Factor(file,char,n,d,pb);
                        if pb /= Null_Poly
                         then Clear(res); Copy(pb,res); Clear(pb);
                        end if;
                  end if;
             else -- the case ".. c ?" :
                  -- will be treated in the loop
                  null;
    end if;
 
    loop
      case char is
        when ' '       => get(file,char);
        when '*'       => get(file,char); Read_Factor(file,char,n,d,pb);
                          Collect_Factor_Polynomial;
        when '+' | '-' => if Equal(c,compzero)
                           then raise ILLEGAL_CHARACTER;
                           else exit;
                          end if;
        when delimiter => exit;
        when '('       => if Equal(c,compzero)
                             or else Equal(c,compmin1)
                           then -- the case "+ (" or "- (" :
                                c := Create(0.0);
                                exit;
                           else -- the case "c  (" :
                                raise BAD_BRACKET;
                          end if;
        when ')'       => exit;
        when others    => if Equal(c,compzero)
                           then c := Create(1.0);
                                Read_Factor(file,char,n,d,pb);
                           elsif c = Create(-1.0)
                               then Read_Factor(file,char,n,d,pb);
                               else raise ILLEGAL_CHARACTER;
                          end if;
                          Collect_Factor_Polynomial;
      end case;
    end loop;
    tmp.cf := c;
    tmp.dg := d;
    termp := create(tmp);
    if Number_Of_Unknowns(res) > 0
     then Mul(termp,res); Clear(res);
    end if;
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end Read_Term;

-- THE INPUT OPERATIONS :

  procedure get ( n : in out natural; p : in out Poly ) is
  begin
    get(Standard_Input,n,p);
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end get;

  procedure get ( file : in file_type; n : in out natural; p : in out Poly ) is
  begin
    get(file,n);
    if Symbol_Table.Empty
     then Symbol_Table.Init(n);
    end if;
    get(file,p);
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end get;

  procedure get ( p : in out Poly ) is
  begin
    get(Standard_Input,p);
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end get;

  procedure get ( file : in file_type; p : in out Poly ) is

    n : constant natural := Symbol_Table.Maximal_Size;
    char,oper : character;
    term,res,acc : Poly;

  begin
    oper := '+';
    get(file,char);
    while (char = ' ') loop get(file,char); end loop;
    if char = '-'
     then oper := '-';
    end if;
                                    -- the first term can have no sign
    Read_Term(file,char,n,res);     -- therefore read it first
    loop
      case char is
        when ' '       => get(file,char);    -- skip blanks
        when '+' | '-' => oper := char;
                          Read_Term(file,char,n,term);
                          Add(res,term); Clear(term);
        when delimiter => exit;
        when '('       => get(file,term);
                          case oper is
                            when '+' => Add(acc,res); Clear(res);
                                        Copy(term,res);
                            when '-' => Add(acc,res);Clear(res);
                                        Copy(term,res); Min(res);
                            when '*' => Mul(res,term);
                            when others => raise ILLEGAL_OPERATION;
                          end case;
                          Clear(term);
                          get(file,char);   -- get new character
        when ')'       => exit;
        when '*'       => if res = Null_Poly
                           then raise ILLEGAL_CHARACTER;
                           else -- the case " ) * " :
                                oper := char; get(file,char);  -- skip '*'
                                Read_Term(file,char,n,term);
                                if char /= '('
                                 then case oper is
                                        when '+' => Add(res,term);
                                        when '-' => Sub(res,term);
                                        when '*' => Mul(res,term);
                                        when others => raise ILLEGAL_OPERATION;
                                      end case;
                                end if;
                                Clear(term);
                          end if;
        when others    => raise ILLEGAL_CHARACTER;
      end case;
    end loop;
    p := acc + res;
    Clear(acc); Clear(res);
  exception
    when ILLEGAL_CHARACTER    => raise ILLEGAL_CHARACTER;
    when ILLEGAL_SYMBOL       => raise ILLEGAL_SYMBOL;
    when ILLEGAL_OPERATION    => raise ILLEGAL_OPERATION;
    when INFINITE_NUMBER      => raise INFINITE_NUMBER;
    when OVERFLOW_OF_UNKNOWNS => raise OVERFLOW_OF_UNKNOWNS;
    when BAD_BRACKET          => raise BAD_BRACKET;
  end get;

-- THE OUTPUT OPERATIONS FOR TERMS :

  procedure Write ( file : in file_type; c : in Complex_Number ) is

    cre : Floating_Number := REAL_PART(c);
    cim : Floating_Number := IMAG_PART(c);

  begin
   -- if Equal(cim,0.0)
   --  then if not (cre < 0.0)
   --        then put(file,"+");
   --       end if;
   --       put(file,cre);
   --  elsif Equal(cre,0.0)
   --      then if not (cim < 0.0)
   --            then put(file,"+");
   --           end if;
   --           put(file,cim);
   --           put(file,"*i");
   --      else put(file,"+(");
   --           put(file,cre);
   --           if not (cim < 0.0)
   --            then put(file,"+");
   --           end if;
   --           put(file,cim);
   --           put(file,"*i)");
   -- end if;
    put(file,"+("); put(file,cre);
    put(file," + "); put(file,cim);
    put(file,"*i)");
    Clear(cre); Clear(cim);
  end Write;

  procedure Write ( file : in file_type; d : in Degrees;
                    standard : in boolean; pow : in Power ) is

    sb : Symbol;

  begin
    for i in d'range loop
      if d(i) /= 0
       then put(file,"*");
            if standard
             then put(file,'x');
                  put(file,i,1);
             else sb := Symbol_Table.get(i);
                  Symbol_Table_io.put(file,sb);
            end if;
            if d(i) /= 1
             then if pow = '^'
                   then put(file,"^");
                   else put(file,"**");
                  end if;
                  put(file,d(i),1);
            end if;
      end if;
    end loop;
  end Write;

  procedure Write ( file : in file_type; d : in Degrees;
                    s : in Array_of_Symbols; pow : in Power ) is
  begin
    for i in d'range loop
      if d(i) /= 0
       then put(file,"*");
            Symbol_Table_io.put(file,s(i));
            if d(i) /= 1
             then if pow = '^'
                   then put(file,"^");
                   else put(file,"**");
                  end if;
                  put(file,d(i),1);
            end if;
      end if;
    end loop;
  end Write;

  procedure put ( file : in file_type; t : in Term;
                  standard : in boolean; pow : in Power ) is
  begin
    Write(file,t.cf);
    if Sum(t.dg) /= 0
     then Write(file,t.dg,standard,pow);
    end if;
  end put;

  procedure put ( file : in file_type; t : in Term;
                  s : in Array_of_Symbols; pow : in Power ) is
  begin
    Write(file,t.cf);
    if Sum(t.dg) /= 0
     then Write(file,t.dg,s,pow);
    end if;
  end put;

-- THE OUTPUT OPERATIONS :

  procedure put ( p : in Poly; pow : in Power ) is
  begin
    put(Standard_Output,p,pow);
  end put;

  procedure put ( file : in file_type; p : in Poly; pow : in Power ) is

    n : constant natural := Number_of_Unknowns(p);
    standard : constant boolean := (Symbol_Table.Number < n);

    procedure Write_Term ( t : in Term; continue : out boolean ) is
    begin
      put(file,t,standard,pow);
      continue := true;
    end Write_Term;
    procedure Write_Terms is new Visiting_Iterator(Write_Term);

  begin
    Write_Terms(p);
    put(file,delimiter);
  end put;

  procedure put ( n : in natural; p : in Poly; pow : in Power ) is
  begin
    put(Standard_Output,n,p,pow);
  end put;

  procedure put ( file : in file_type;
                  n : in natural; p : in Poly; pow : in Power ) is
  begin
    put(file,n,3); new_line(file);
    put(file,p,pow);
  end put;

  procedure put ( p : in Poly ) is
  begin
    put(Standard_Output,p);
  end put;

  procedure put ( file : in file_type; p : in Poly ) is
  begin
    put(file,p,'*');
  end put;

  procedure put ( p : in Poly; s : in Array_of_Symbols ) is
  begin
    put(Standard_Output,p,s);
  end put;

  procedure put ( file : in file_type;
                  p : in Poly; s : in Array_of_Symbols ) is

    procedure Write_Term ( t : in Term; continue : out boolean ) is
    begin
      put(file,t,s,'*');
      continue := true;
    end Write_Term;
    procedure Write_Terms is new Visiting_Iterator(Write_Term);

  begin
    Write_Terms(p);
    put(file,delimiter);
  end put;

  procedure put ( n : in natural; p : in Poly; s : in Array_of_Symbols ) is
  begin
    put(Standard_Output,n,p,s);
  end put;

  procedure put ( file : in file_type;
                  n : in natural; p : in Poly; s : in Array_of_Symbols ) is
  begin
    put(file,n,3); new_line(file);
    put(file,p,s);
  end put;

  procedure put_line ( p : in Poly ) is
  begin
    put_line(Standard_Output,p);
  end put_line;

  procedure put_line ( file : in file_type; p : in Poly ) is
  begin
    put_line(file,p,'*');
  end put_line;

  procedure put_line ( p : in Poly; pow : in Power ) is
  begin
    put_line(Standard_Output,p,pow);
  end put_line;

  procedure put_line ( file : in file_type; p : in Poly; pow : in Power ) is

    n : constant natural := Number_of_Unknowns(p);
    standard : constant boolean := (Symbol_Table.Number < n);

    procedure Write_Term ( t : in Term; continue : out boolean ) is
    begin
      put(file,t,standard,pow);
      new_line(file);
      continue := true;
    end Write_Term;
    procedure Write_Terms is new Visiting_Iterator(Write_Term);

  begin
    Write_Terms(p);
    put(file,delimiter);
  end put_line;

  procedure put_line ( p : in Poly; s : in Array_of_Symbols ) is
  begin
    put_line(Standard_Output,p,s);
  end put_line;

  procedure put_line ( file : in file_type;
                       p : in Poly; s : in Array_of_Symbols ) is
  begin
    put_line(file,p,s,'*');
  end put_line;

  procedure put_line
              ( p : in Poly; s : in Array_of_Symbols; pow : in Power ) is
  begin
    put_line(Standard_Output,p,s,pow);
  end put_line;

  procedure put_line ( file : in file_type; p : in Poly;
                       s : in Array_of_Symbols; pow : in Power ) is

    procedure Write_Term ( t : in Term; continue : out boolean ) is
    begin
      put(file,t,s,pow);
      new_line(file);
      continue := true;
    end Write_Term;
    procedure Write_Terms is new Visiting_Iterator(Write_Term);

  begin
    Write_Terms(p);
    put(file,delimiter);
  end put_line;

  procedure Display_Format is
  begin
    Standard_Complex_Polynomials_io.Display_Format;
  end Display_Format;

end Multprec_Complex_Polynomials_io;
