with text_io,integer_io;                 use text_io,integer_io;
--with Standard_Floating_Numbers_io;       use Standard_Floating_Numbers_io;

with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Characters_and_Numbers;             use Characters_and_Numbers;
with Standard_Natural_Vectors;           use Standard_Natural_Vectors;
with Symbol_Table;                       use Symbol_Table;

package body Standard_Complex_Poly_Strings is

-- AUXILIARIES FOR INPUT :

  procedure Skip_Spaces ( s : in string; k : in out natural ) is

  -- DESCRIPTION :
  --   Moves the position counter k ahead to skip white spaces.

  begin
    while (k <= s'last and then s(k) = ' ') loop
      k := k+1;
    end loop;
  end Skip_Spaces;

  procedure Read_Natural ( s : in string; k : in out natural;
                           e : out natural ) is

  -- DESCRIPTION :
  --   Starts reading a natural number in the string at position k.

  -- ON ENTRY :
  --   s          string where s(k) is a digit;
  --   k          position to start the parsing.

  -- ON RETURN :
  --   k          position in s just after last digit read;
  --   e          value of the number read.

    exp : string(1..16);
    cnt : natural := 0;

  begin
    while Convert(s(k)) < 10 loop
      cnt := cnt+1;
      exp(cnt) := s(k);
      k := k+1;
    end loop;
    e := Convert(exp(1..cnt));
  end Read_Natural;

  procedure Read_Natural ( s : in string; k : in out natural;
                           e : out double_float ) is

  -- DESCRIPTION :
  --   Starts reading a natural number in the string at position k.

  -- ON ENTRY :
  --   s          string where s(k) is a digit;
  --   k          position to start the parsing.

  -- ON RETURN :
  --   k          position in s just after last digit read;
  --   e          value of the number read.

    exp : string(1..16);
    cnt : natural := 0;

  begin
    while Convert(s(k)) < 10 loop
      cnt := cnt+1;
      exp(cnt) := s(k);
      k := k+1;
    end loop;
    e := Convert(exp(1..cnt));
  end Read_Natural;

  procedure Read_Coefficient ( s : in string; k : in out natural;
                               c : out Complex_Number ) is

  -- DESCRIPTION :
  --   Reads an unsigned coefficient.

    ind : natural;
    f,a : double_float;

  begin
    if s(k) = '.'
     then k := k+1;
          ind := k;
          Read_Natural(s,k,f);
          f := f/10.0**(k-ind);
     else Read_Natural(s,k,f);
          if s(k) = '.'
           then k := k+1;
                ind := k;
                Read_Natural(s,k,a);
                f := f + a/10.0**(k-ind); 
          end if;
    end if;
    c := Create(f);
  end Read_Coefficient;

  procedure Read_Factor ( s : in string; k : in out natural;
                          n : in natural;
                          dg : in out Standard_Natural_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Starts reading a factor of a polynomial in n variables
  --   in the string s at position k.

    sb : Symbol;
    ind,pos : natural;

  begin
    loop
      sb := (sb'range => ' ');
      ind := sb'first - 1;
      Skip_Spaces(s,k);
      if k > s'last
       then return;
      end if;
      loop                             -- read symbol
        case s(k) is
          when '+' | '-' | '*' | '^' | ';' => exit;
          when others => ind := ind + 1;
                         sb(ind) := s(k);
        end case;
        k := k+1;
      end loop;
      exit when (ind < sb'first);
      pos := Symbol_Table.get(sb);     -- lookup symbol
      if pos = 0
       then Symbol_Table.Add(sb,pos);
      end if;
      if s(k) = '^'
       then k := k+1;
            Read_Natural(s,k,dg(pos));
       elsif s(k) = '*' and then s(k+1) = '*'
           then k := k+2;
                Read_Natural(s,k,dg(pos));
           else dg(pos) := 1;
      end if;
      Skip_Spaces(s,k);
      exit when s(k) /= '*';
      k := k+1;
    end loop;
  end Read_Factor;

  procedure Read_Coefficient_and_Term
              ( s : in string; k : in out natural;
                n : in natural; t : in out Term ) is

  -- DESCRIPTION :
  --   Reads a coefficient and exponent vector for the term.

  -- REQUIRED :
  --   memory is allocated for t.dg and s(k) is a number.

  begin
    Read_Coefficient(s,k,t.cf);
    if s(k) = '*'
     then k := k+1;
          Read_Factor(s,k,n,t.dg.all);
    end if;
  end Read_Coefficient_and_Term;

  procedure Read_Term ( s : in string; k : in out natural; 
                        n : in natural; t : out Term ) is

  -- DESCRIPTION :
  --   Starts reading a term of a polynomial in n variables
  --   in the string s at position k.
  --   The starting position can either be white space,
  --   a sign, a number, or the beginning of a factor.

  begin
    Skip_Spaces(s,k);
    t.dg := new Standard_Natural_Vectors.Vector'(1..n => 0);
    if Convert(s(k)) < 10 or s(k) = '.'
     then Read_Coefficient_and_Term(s,k,n,t);
     else if s(k) = '+'
           then k := k+1;
                Skip_Spaces(s,k);
                if Convert(s(k)) < 10 or s(k) = '.'
                 then Read_Coefficient_and_Term(s,k,n,t);
                 else t.cf := Create(1.0);
                      Read_Factor(s,k,n,t.dg.all);
                end if;
           elsif s(k) = '-'
               then k := k+1;
                    Skip_Spaces(s,k);
                    if Convert(s(k)) < 10 or s(k) = '.'
                     then Read_Coefficient_and_Term(s,k,n,t);
                          t.cf := -t.cf;
                     else t.cf := Create(-1.0);
                          Read_Factor(s,k,n,t.dg.all);
                    end if;
               else t.cf := Create(1.0);
                    Read_Factor(s,k,n,t.dg.all);
          end if;
    end if;
  end Read_Term;

  function Delimiters ( n : natural; s : string )
                      return Standard_Natural_Vectors.Vector is

  -- DESCRIPTION :
  --   Searches the string s for n semicolons and returns the positions
  --   of those semicolons.

    res : Standard_Natural_Vectors.Vector(1..n);
    ind : natural := 1;

  begin
    for i in s'range loop
      if s(i) = ';'
       then res(ind) := i;
            ind := ind+1;
      end if;    end loop;
    return res;
  end Delimiters;

-- AUXILIARIES FOR OUTPUT :

  function Is_Imag ( c : Complex_Number ) return boolean is

  -- DESCRIPTION :
  --   Returns true if the number is pure imaginary, false otherwise.

  begin
    return ( REAL_PART(c) = 0.0 );
  end is_imag;

  function Is_Real ( c : Complex_Number ) return boolean is

  -- DESCRIPTION :
  --   Returns true if the complex number is real, false otherwise.

  begin
    return ( IMAG_PART(c) = 0.0 );
  end is_real;

  function Is_Integer ( f : double_float ) return boolean is

  -- DESCRIPTION :
  --   Returns true if the floating number is an integer, false otherwise.

  begin
    return ( (f - double_float(integer(f))) = 0.0 );
  exception
    when numeric_error => return false;
  end is_integer;

  function Is_Unit ( c : Complex_Number ) return boolean is

  -- DESCRIPTION :
  --   Returns true if c is +1 or -1.

  begin
    if not Is_Real(c)
     then return false;
     elsif not Is_Integer(REAL_PART(c))
         then return false;
         else declare
                i : integer := integer(REAL_PART(c));
              begin
                if (i = 1) or (i = -1)
                 then return true;
                 else return false;
                end if;
              end;
    end if;     
  end Is_Unit;

  function Sign ( c : Complex_Number ) return integer is

  -- DESCRIPTION :
  --   This routine only make sense when Is_Unit(c).
  --   It returns +1 or -1, depending whether c = +1 or -1.

  begin
    return integer(REAL_PART(c));
  end Sign;

  function Truncate ( f : double_float ) return integer is

    tol : constant double_float := 1.0E-8;
    i : integer := integer(f);

  begin
    if i >= 0
     then if double_float(i) > f + tol
           then i := i-1;
          end if;
 --    else if double_float(i) < f - tol --          then i := i+1;
 --         end if;
    end if;
    return i;
  end Truncate;

  function Trim_Zeros ( s : string ) return string is

  -- DESCRIPTION :
  --   Returns the string s with trailing zeros removed.

  begin
    for i in reverse s'range loop
      if s(i) /= '0'
       then return s(1..i);
      end if;
    end loop;
    return s(1..1);
  end Trim_Zeros;

  function Convert ( f : double_float ) return string is

  -- DESCRIPTION :
  --   Converts a positive float into fixed point format string.

    a : integer := Truncate(f);
    r1 : double_float := f - double_float(a);
    re8a : double_float := r1*1.0E+8;
    b1 : integer := Truncate(re8a);
    sb1 : constant string := Convert(b1); 
    s8b1 : string(1..8);
    dif1 : constant natural := 8 - sb1'last;
    r2 : double_float := re8a - double_float(b1);
    re8b : double_float := r2*1.0E+8;
    b2 : integer := Truncate(re8b);

  begin
    if dif1 = 0
     then s8b1 := sb1;
     else s8b1(1..dif1) := (1..dif1 => '0');
          for i in sb1'range loop
            s8b1(dif1+i) := sb1(i);
          end loop;
    end if;
    if b2 <= 0
     then return Convert(a) & "." & Trim_Zeros(s8b1);
     else declare
            sb2 : constant string := Convert(b2);
            s8b2 : string(1..8);
            dif2 : constant natural := 8 - sb2'last;
          begin
            if dif2 = 0
             then return Convert(a) & "." & s8b1 & Trim_Zeros(sb2);
             else s8b2(1..dif2) := (1..dif2 => '0');
                  for i in sb2'range loop
                    s8b2(dif2+i) := sb2(i);
                  end loop;
                  return Convert(a) & "." & s8b1 & Trim_Zeros(s8b2);
            end if;
          end;
    end if;
  end Convert;

  function Signed_Constant ( c : Complex_Number ) return string is

  -- DESCRIPTION :
  --   Returns the string representation of the constant term of
  --   a polynomial, with its sign.

    re : double_float;

  begin
    if Is_Real(c)
     then re := REAL_PART(c);
          if Is_Integer(re)
           then declare
                  i : integer := integer(re);
                begin
                  if i >= 0
                   then return " + " & Convert(i);
                   else return " - " & Convert(-i);
                  end if;
                end;
           else if re >= 0.0
                 then return " + " & Convert(re);
                 else return " - " & Convert(abs(re));
                end if;
          end if;
    end if;
    return "";
  end Signed_Constant;

  function Unsigned_Constant ( c : Complex_Number ) return string is

  -- DESCRIPTION :
  --   This routine converts the constant term of the polynomial into
  --   a string.  A positive integer is returned without the sign.

    re : double_float;

  begin
    if Is_Real(c)
     then re := REAL_PART(c);
          if Is_Integer(re)
           then declare
                  i : integer := integer(re);
                begin
                  if i >= 0
                   then return Convert(i);
                   else return " - " & Convert(-i);
                  end if;
                end;
           else if re >= 0.0
                 then return Convert(re);
                 else return " - " & Convert(abs(re));
                end if;
          end if;
    end if;
    return "";
  end Unsigned_Constant;

  function Signed_Coefficient ( c : Complex_Number ) return string is

  -- DESCRIPTION :
  --   Returns the coefficient with its sign.  The +1 and -1 are
  --   treated differently.

    re : double_float;

  begin
    if Is_Real(c)
     then re := REAL_PART(c);
          if Is_Integer(re)
           then declare
                  i : integer := integer(re);
                begin
                  if i >= 0
                   then if i = 1
                         then return " + ";
                         else return " + " & Convert(i);
                        end if;
                   else if i = -1
                         then return " - ";
                         else return " - " & Convert(-i);
                        end if;
                  end if;
                end;
           else if re >= 0.0
                 then return " + " & Convert(re);
                 else return " - " & Convert(abs(re));
                end if;
          end if;
    end if;
    return "";
  end Signed_Coefficient;

  function Unsigned_Coefficient ( c : Complex_Number ) return string is

  -- DESCRIPTION :
  --   Returns the coefficient without sign, except if it is negative.

    re : double_float;

  begin
    if Is_Real(c)
     then re := REAL_PART(c);
          if Is_Integer(re)
           then declare
                  i : integer := integer(re);
                begin
                  if i >= 0
                   then if i = 1
                         then return "";
                         else return Convert(i);
                        end if;
                   else if i = -1
                         then return " - ";
                         else return " - " & Convert(-i);
                        end if;
                  end if;
                end;
           else if re >= 0.0
                 then return Convert(re);
                 else return " - " & Convert(abs(re));
                end if;
          end if;
    end if;
    return "";
  end Unsigned_Coefficient;

  function Concat_Symbol0 ( s : string; sb : Symbol ) return string is

  -- DESCRIPTION :
  --   Returns the result of concatenating the symbol to the string s.
  --   There is no '*' between the string and the symbol.

    nb : natural := 0;

  begin
    for i in sb'range loop
      if sb(i) = ' '
       then nb := i-1; exit;
      end if;
    end loop;
    declare
      res : string(1..s'last+nb);
    begin
      res(s'range) := s;
      for i in 1..nb loop
        res(s'last+i) := sb(i);
      end loop;
      return res;
    end;
  end Concat_Symbol0;

  function Concat_Symbol1 ( s : string; sb : Symbol ) return string is

  -- DESCRIPTION :
  --   Returns the result of concatenating the symbol to the string s.
  --   There is a '*' between the string and the symbol.

    nb : natural := 0;

  begin
    for i in sb'range loop
      if sb(i) = ' '
       then nb := i-1; exit;
      end if;
    end loop;
    declare
      res : string(1..s'last+1+nb);
    begin
      res(s'range) := s;
      res(s'last+1) := '*';
      for i in 1..nb loop
        res(s'last+1+i) := sb(i);
      end loop;
      return res;
    end;
  end Concat_Symbol1;

  function Write ( t : Term; lead : boolean ) return string is

  -- DESCRIPTION :
  --   Returns the string representation of the term.
  --   When the term is the leading term of the polynomial,
  --   then lead is true.

    first : boolean := true;

    function Rec_Write ( k : natural; accu : string; first : boolean )
                       return string is
    begin
      if k > t.dg'last
       then return accu;
       elsif t.dg(k) = 0
           then return Rec_Write(k+1,accu,first);
           else declare
                  sb : Symbol := Symbol_Table.Get(k);
                begin
                  if t.dg(k) > 1
                   then if first
                         then declare
                                new_accu : string := Concat_Symbol0(accu,sb)
                                               & '^' & Convert(t.dg(k));
                              begin
                                return Rec_Write(k+1,new_accu,false);
                              end;
                         else declare
                                new_accu : string := Concat_Symbol1(accu,sb)
                                               & '^' & Convert(t.dg(k));
                              begin
                                return Rec_Write(k+1,new_accu,first);
                              end;
                        end if;
                   else if first
                         then declare
                                new_accu : string := Concat_Symbol0(accu,sb);
                              begin
                                return Rec_Write(k+1,new_accu,false);
                              end;
                         else declare
                                new_accu : string := Concat_Symbol1(accu,sb);
                              begin
                                return Rec_Write(k+1,new_accu,first);
                              end;
                        end if;
                  end if;
                end;
      end if;
    end Rec_Write;
 
  begin
    declare
      factor : string := Rec_Write(1,"",true);
    begin
      if factor = ""    -- then we have constant term
       then if lead
             then return Unsigned_Constant(t.cf);
             else return Signed_Constant(t.cf);
            end if;
       else if lead
             then if Is_Unit(t.cf)
                   then if Sign(t.cf) = +1
                         then return factor;
                         else return " - " & factor;
                        end if;
                   else return Unsigned_Coefficient(t.cf) & "*" & factor;
                  end if;
             else if Is_Unit(t.cf)
                   then if Sign(t.cf) = +1
                         then return " + " & factor;
                         else return " - " & factor;
                        end if;
                   else return Signed_Coefficient(t.cf) & "*" & factor;
                  end if;
            end if;
      end if;
    end;
  end Write;

  function Write ( p : Poly_Sys ) return string is
  begin
    if p'first = p'last
     then return Write(p(p'first));
     else return Write(p(p'first)) & Write(p(p'first+1..p'last));
    end if;
  end Write;

-- TARGET FUNCTIONS :

  function Parse ( n : natural; s : string ) return Poly is

    res : Poly;
    ind : natural := s'first;
    t : Term;

  begin
    loop
      Read_Term(s,ind,n,t);
      Add(res,t);
      Clear(t);
      exit when (ind = s'last or s(ind) = ';');
    end loop;
    return res;
  end Parse;

  function Parse ( n,m : natural; s : string ) return Poly_Sys is

    res : Poly_Sys(1..n);
    ind : Standard_Natural_Vectors.Vector(1..n) := Delimiters(n,s);

  begin
    res(1) := Parse(m,s(s'first..ind(1)));
    for i in 2..n loop
      res(i) := Parse(m,s(ind(i-1)+1..ind(i)));
    end loop;
    return res;
  end Parse;

  function Write ( p : Poly ) return string is

    nb : constant natural := Number_of_Terms(p);
    terms : array(1..nb) of Term;
    ind : natural := 0;

    procedure Store_Term ( t : in Term; continue : out boolean ) is
    begin
      ind := ind + 1;
      terms(ind) := t;
      continue := true;
    end Store_Term;
    procedure Store_Terms is new Visiting_Iterator(Store_Term);
 
    function Write_Terms ( k : natural; accu : string ) return string is
    begin
      if k > nb
       then return accu;
       else if k = 1
             then declare
                    new_accu : string := accu & Write(terms(k),true);
                  begin
                    return Write_Terms(k+1,new_accu);
                  end;
             else declare
                    new_accu : string := accu & Write(terms(k),false);
                  begin
                    return Write_Terms(k+1,new_accu);
                  end;
            end if;
      end if;
    end Write_Terms;

  begin
    Store_Terms(p);
    return Write_Terms(1,"") & ";";
  end Write;

end Standard_Complex_Poly_Strings;
