with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Random_Numbers;           use Standard_Random_Numbers;

package body Standard_Random_Polynomials is

  function Random_Coefficient ( c : natural ) return Complex_Number is
 
    res : Complex_Number;
    ranflt : double_float;

  begin
    case c is
      when 1 => res := Create(1.0);
      when 2 => ranflt := Random;
                res := Create(ranflt);
      when others => res := Random1;
    end case;
    return res;
  end Random_Coefficient;

  function Random_Monomial ( n,d : natural ) return Term is

    res : Term;
    deg,pos : natural;

  begin
    res.cf := Create(1.0);
    res.dg := new Vector'(1..n => 0);
    for i in 1..d loop
      deg := Random(0,1);
      pos := Random(1,n);
      res.dg(pos) := res.dg(pos) + deg;
    end loop;
    return res;
  end Random_Monomial;

  function Random_Term ( n,d,c : natural ) return Term is

    res : Term := Random_Monomial(n,d);

  begin
    res.cf := Random_Coefficient(c);
    return res;
  end Random_Term;

  function Random_Dense_Poly ( n,d,c : natural ) return Poly is

    res : Poly := Null_Poly;
    t : Term;

    procedure Generate_Monomials
                ( accu : in out Term; k,sum : in natural ) is

    -- DESCRIPTION :
    --   Accumulating procedure to generate all monomials up to degree d.

    -- ON ENTRY :
    --   accu     accumulator contains the current exponent vector;
    --   k        current component;
    --   sum      sum of current entries in the accumulator.

    -- ON RETURN :
    --   accu     accumulator determined up to k component, k included.

    begin
      if k > n
       then accu.cf := Random_Coefficient(c);
            Add(res,accu);
       else for i in 0..d loop
              if sum + i <= d
               then accu.dg(k) := i;
                    Generate_Monomials(accu,k+1,sum+i);
                    accu.dg(k) := 0;
              end if;
            end loop;
      end if;
    end Generate_Monomials;

  begin
    t.dg := new Vector'(1..n => 0);
    Generate_Monomials(t,1,0);
    return res;
  end Random_Dense_Poly;

  function Random_Sparse_Poly ( n,d,m,c : natural ) return Poly is

    res : Poly := Null_Poly;

  begin
    for i in 1..m loop
      declare
        rt : Term := Random_Term(n,d,c);
      begin
        Add(res,rt);
        Clear(rt);
      end;
    end loop;
    return res;
  end Random_Sparse_Poly;

end Standard_Random_Polynomials;
