package body Monomial_Hashing is

  procedure Enumerate_Monomials ( d,n : in natural ) is

    accu : Vector(1..n) := (1..n => 0);
    sum : natural := 0;

    procedure Enumerate ( i,dg : natural ) is

    -- DESCRIPTION :
    --   Enumerates the distribution of degree dg over entries i to n.

    begin
      if sum = d then
        Monomial(accu);
      elsif i = n then
        accu(n) := dg;
        Monomial(accu);
        accu(n) := 0;
      else
        for j in reverse 0..dg loop
          accu(i) := j;
          sum := sum + j;
          Enumerate(i+1,d-sum);
          sum := sum - j;
          accu(i) := 0;
        end loop;
      end if;
    end Enumerate;

  begin
    Enumerate(1,d);
  end Enumerate_Monomials;

  procedure Enumerate_Leaves_of_Monomial_Tree ( d,n : in natural ) is

    accu : Vector(1..n) := (1..n => 0);

    procedure Enumerate ( level : in natural ) is

    -- DESCRIPTION :
    --   Enumerates all monomials at the current level <= d.

    begin
      for i in 1..n loop
        accu(i) := accu(i) + 1;
        if level = d
         then Monomial(accu);
         else Enumerate(level+1);
        end if;
        accu(i) := accu(i) - 1;
      end loop;
    end Enumerate;

  begin
    if d = 0
     then Monomial(accu);
     else Enumerate(1);
    end if;
  end Enumerate_Leaves_of_Monomial_Tree;

  function Monomial_Count ( d,n : natural ) return natural is

    res : natural := 0;

    procedure Count ( m : Vector ) is
    begin
      res := res + 1;
    end Count;
    procedure Count_Monomials is new Enumerate_Monomials(Count);

  begin
    Count_Monomials(d,n);
    return res;
  end Monomial_Count;

  function Monomial_Code ( d : natural; m : Vector ) return natural is

    res : natural := 0;

  begin
    for i in m'range loop
      res := res*d + m(i);
    end loop;
    return res;
  end Monomial_Code;

  function Monomial_Keys ( k,n : natural ) return VecVec is

    res : VecVec(1..k);
    ind,code_ind : natural := 0;

    procedure Label ( m : in Vector ) is
    begin
      code_ind := code_ind + 1;
      res(ind)(code_ind) := Monomial_Code(k+1,m);
    end Label;
    procedure Enumerate is new Enumerate_Monomials(Label);

  begin
    for i in 1..k loop
      res(i) := new Vector(1..Monomial_Count(i,n));
      code_ind := 0; ind := i;
      Enumerate(i,n);
    end loop;
    return res;
  end Monomial_Keys;

  function Search ( monkeys : VecVec; c,s : natural ) return natural is

    keys : Link_to_Vector;

  begin
    if s <= monkeys'last then
      keys := monkeys(s);
      for i in keys'range loop   -- linear search is not most efficient!
        if keys(i) = c
         then return i;
        end if;
      end loop;
    end if;
    return 0;
  end Search;

  function Search ( monkeys : VecVec; m : Vector;
                    s : natural ) return natural is

    k : constant natural := monkeys'last;
    c : constant natural := Monomial_Code(k+1,m);

  begin
    return Search(monkeys,c,s);
  end Search;

  function Search ( monkeys : VecVec; m : Vector ) return natural is

    s : constant natural := Sum(m);

  begin
    return Search(monkeys,m,s);
  end Search;

end Monomial_Hashing;
