with integer_io;                        use integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;
with Standard_Random_Numbers;
with Standard_Natural_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_Matrices;
with Standard_Complex_Norms_Equals;     use Standard_Complex_Norms_Equals;
with Standard_Complex_QR_Least_Squares; use Standard_Complex_QR_Least_Squares;
with Standard_Complex_Singular_Values;  use Standard_Complex_Singular_Values;
with Standard_Complex_Polynomials;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Jaco_Matrices;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Standard_Newton_Steps;
with Standard_Deflate_Singularities;
with Standard_Deflation_Trees_io;       use Standard_Deflation_Trees_io;
with Monomial_Hashing;                  use Monomial_Hashing;

package body Standard_Deflation_Methods is

-- INTERACTIVE NEWTON with DEFLATION :

  procedure Display_Results_of_One_Newton_Step
              ( file : in file_type; z : in Standard_Complex_Vectors.Vector;
                tol,err,rco,res : in double_float; rank,cnt : in natural ) is

    use Standard_Complex_Solutions;

    n : constant natural := z'length;
    s : Solution(n);

  begin
    put(file,"The new approximation at step ");
    put(file,cnt,1); put_line(file," : "); 
    s.v := z;
    put_vector(file,s);
    put(file,"error :"); put(file,err,3);
    put(file,"  rcond :"); put(file,rco,3);
    put(file,"  resid :"); put(file,res,3); new_line(file);
    put(file,"The numerical rank : "); put(file,rank,1);
    if rank > n
     then put(file," > "); put(file,n,1); 
          put_line(file," number of variables, BUG !??");
     else if rank = n
           then put(file," = ");
           else put(file," < ");
          end if;
          put(file,n,1);
          put(file,"  Corank is "); put(file,n-rank,1);
          put(file,"  w.r.t. tol :"); put(file,tol,3);  new_line(file);
    end if;
  end Display_Results_of_One_Newton_Step;

  procedure One_Symbolic_Newton_Step
              ( file : in file_type;
                ep : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                ejm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                z : in out Standard_Complex_Vectors.Vector;
                tol : in double_float; err,rco,res : out double_float;
                s : out Standard_Complex_Vectors.Vector;
                rank : out natural ) is

    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Newton_Steps;

    function f ( x : Vector ) return Vector is
    begin
      return Eval(ep,x);
    end f;

    function jmf ( x : Vector ) return Matrix is
    begin
      return Eval(ejm,x);
    end jmf;

    procedure Newton is new Reporting_Newton_Step_with_Singular_Values(f,jmf);

  begin
    Newton(file,ep'last,z,tol,err,rco,res,s,rank);
  end One_Symbolic_Newton_Step;

  procedure One_Symbolic_Newton_Step
              ( ep : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                ejm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                z : in out Standard_Complex_Vectors.Vector;
                tol : in double_float; err,rco,res : out double_float;
                s : out Standard_Complex_Vectors.Vector;
                rank : out natural ) is

    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Newton_Steps;

    function f ( x : Vector ) return Vector is
    begin
      return Eval(ep,x);
    end f;

    function jmf ( x : Vector ) return Matrix is
    begin
      return Eval(ejm,x);
    end jmf;

    procedure Newton is new Silent_Newton_Step_with_Singular_Values(f,jmf);

  begin
    Newton(ep'last,z,tol,err,rco,res,s,rank);
  end One_Symbolic_Newton_Step;

  function Staple ( n : natural; x : Standard_Complex_VecVecs.VecVec )
                  return Standard_Complex_Vectors.Vector is

  -- DESCRIPTION :
  --   Returns the values of the vector x in one n-vector.

  -- REQUIRED : number of elements in x is n.

    res : Standard_Complex_Vectors.Vector(1..n);
    cnt : natural := 0;

  begin
    for i in x'range loop
      for j in x(i)'range loop
        cnt := cnt + 1;
        res(cnt) := x(i)(j);
      end loop;
    end loop;
    return res;
  end Staple;

  procedure Split ( z : in Standard_Complex_Vectors.Vector;
                    x : in out Standard_Complex_VecVecs.VecVec ) is

  -- DESCRIPTION :
  --   Splits the values of the vector z over the vectors in x.

  -- REQUIRED :
  --   The number of elements in x equals the number of elements in z.

    cnt : natural := 0;

  begin
    for i in x'range loop
      for j in x(i)'range loop
        cnt := cnt + 1;
        x(i)(j) := z(cnt);
      end loop;
    end loop;
  end Split;

  procedure One_Algorithmic_Newton_Step
              ( file : in file_type;
                ep : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                ejm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                evt : in Standard_Evaluate_Deflation.Link_to_Eval_Tree;
                nd : in Standard_Jacobian_Trees.Link_to_Eval_Node;
                monkeys : in Standard_Natural_VecVecs.VecVec;
                k : in natural;
                nv,nq,R1 : in Standard_Natural_Vectors.Vector;
                B : in Standard_Complex_VecMats.VecMat;
                h : in Standard_Complex_VecVecs.VecVec;
                x : in out Standard_Complex_VecVecs.VecVec;
                z : in out Standard_Complex_Vectors.Vector;
                tol : in double_float; err,rco,res : out double_float;
                s : out Standard_Complex_Vectors.Vector;
                rank : out natural ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Evaluate_Deflation;
    use Standard_Newton_Steps;

    function f ( xx : Vector ) return Vector is
    begin
      Split(xx,x);
      return Eval(ep,ejm,evt,nd,monkeys,k,nv,nq,R1,B,h,x);
    end f;

    function jmf ( xx : Vector ) return Matrix is
    begin
      Split(xx,x);
      return Eval(evt,nd,monkeys,k,nv,nq,R1,B,h,x);
    end jmf;

    procedure Newton is new Reporting_Newton_Step_with_Singular_Values(f,jmf);

  begin
    Newton(file,nq(k),z,tol,err,rco,res,s,rank);
    Split(z,x);
  end One_Algorithmic_Newton_Step;

  procedure One_Algorithmic_Newton_Step
              ( ep : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                ejm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                evt : in Standard_Evaluate_Deflation.Link_to_Eval_Tree;
                nd : in Standard_Jacobian_Trees.Link_to_Eval_Node;
                monkeys : in Standard_Natural_VecVecs.VecVec;
                k : in natural;
                nv,nq,R1 : in Standard_Natural_Vectors.Vector;
                B : in Standard_Complex_VecMats.VecMat;
                h : in Standard_Complex_VecVecs.VecVec;
                x : in out Standard_Complex_VecVecs.VecVec;
                z : in out Standard_Complex_Vectors.Vector;
                tol : in double_float; err,rco,res : out double_float;
                s : out Standard_Complex_Vectors.Vector;
                rank : out natural ) is

    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Evaluate_Deflation;
    use Standard_Newton_Steps;

    function f ( xx : Vector ) return Vector is
    begin
      return Eval(ep,xx);
    end f;

    function jmf ( xx : Vector ) return Matrix is
    begin
      Split(xx,x);
      return Eval(evt,nd,monkeys,k,nv,nq,R1,B,h,x);
    end jmf;

    procedure Newton is new Silent_Newton_Step_with_Singular_Values(f,jmf);

  begin
    Newton(ep'last,z,tol,err,rco,res,s,rank);
    Split(z,x);
  end One_Algorithmic_Newton_Step;

  procedure Add_Multipliers
               ( z : in out Standard_Complex_Vectors.Link_to_Vector;
                 f : in Standard_Complex_Poly_Systems.Poly_Sys;
                 m : in natural ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Vectors;
    use Standard_Deflate_Singularities;

    la : Vector(1..m);
    res : double_float;
    nz : Vector(1..z'last+m);

  begin
    Multipliers(f,z.all,m,la,res);
    put_line("Values computed for the multipliers :");
    put_line(la);
    put("Residual of multiplier computation : ");
    put(res,3); new_line;
    nz(z'range) := z.all;
    for i in la'range loop
      nz(z'last+i) := la(i);
    end loop;
    Clear(z);
    z := new Vector'(nz);
  end Add_Multipliers;

  procedure Add_Multipliers
              ( file : in file_type;
                f : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                jm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                nd : in Standard_Jacobian_Trees.Link_to_Eval_Node;
                monkeys : in Standard_Natural_VecVecs.VecVec;
                k : in natural;
                nv,nq,R1 : in Standard_Natural_Vectors.Vector;
                B : in Standard_Complex_VecMats.VecMat;
                h : in Standard_Complex_VecVecs.VecVec;
                x : in out Standard_Complex_VecVecs.VecVec ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Evaluate_Deflation;

    k1 : constant natural := k-1;
    t1 : Link_to_Eval_Tree := Create(k1);
    n : constant natural := nq(k1)+1;
    m : constant natural := R1(k);
    A1 : Matrix(1..nq(k1),1..nv(k1))
       := Eval(t1,nd,monkeys,k1,nv(0..k1),nq(0..k1),R1(1..k1),
               B(1..k1),h(1..k1),x(0..k1));
    AB1 : Matrix(1..nq(k1),1..R1(k)) := A1*B(k).all;
    A,wrk : Matrix(1..nq(k1)+1,1..R1(k));
    rhs : Vector(1..nq(k1)+1) := (1..nq(k1)+1 => Create(0.0));
    lambda : Vector(1..R1(k));
    qraux : Vector(A'range(2)) := (A'range(2) => Create(0.0));
    jpvt : Standard_Natural_Vectors.Vector(A'range(2))
         := (A'range(2) => 0);
    rsd,dum : Vector(A'range(1));
    info : integer;
    res : double_float;

  begin
    for i in AB1'range(1) loop
      for j in AB1'range(2) loop
        A(i,j) := AB1(i,j);
      end loop;
    end loop;
    for j in 1..R1(k) loop
      A(A'last(1),j) := h(k)(j);
    end loop;
    rhs(rhs'last) := Create(1.0);
    wrk := A;
    QRD(wrk,qraux,jpvt,false);
    QRLS(wrk,n,n,m,qraux,rhs,dum,dum,lambda,rsd,dum,110,info);
    dum := rhs - A*lambda;
    res := Max_Norm(dum);
    put_line(file,"Values computed for the multipliers :");
    put_line(file,lambda);
    put(file,"Residual of multiplier computation : ");
    put(file,res,3); new_line(file);
    x(k) := new Standard_Complex_Vectors.Vector'(lambda);
  end Add_Multipliers;

  procedure Interactive_Symbolic_Newton
              ( file : in file_type;
                p : in Standard_Complex_Poly_Systems.Poly_Sys;
                z : in out Standard_Complex_Vectors.Vector;
                err,rco,res : out double_float;
                tol : in double_float; rank : out natural ) is

    use Standard_Complex_Vectors;
    use Standard_Complex_Poly_Systems;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;

    ep : Eval_Poly_Sys(p'range) := Create(p);
    wp : Link_to_Poly_Sys := new Poly_Sys'(p);
    jm : Jaco_Mat(p'range,z'range) := Create(p);
    ejm : Eval_Jaco_Mat(jm'range(1),jm'range(2)) := Create(jm);
    ans : character;
    cnt : natural := 0;
    k : natural := 0;
    n : constant natural := p'length;
    q : constant natural := z'length;
    m : constant natural := Min0(n+1,q);
    s : Vector(1..m);

  begin
    loop
      cnt := cnt + 1;
      One_Symbolic_Newton_Step(file,ep,ejm,z,tol,err,rco,res,s,rank);
      put_line("The singular values : "); put_line(s);
      Display_Results_of_One_Newton_Step
        (standard_output,z,tol,err,rco,res,rank,cnt);
      Display_Results_of_One_Newton_Step(file,z,tol,err,rco,res,rank,cnt);
      put("Do you want another iteration ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
    end loop;
  end Interactive_Symbolic_Newton;

  procedure Interactive_Algorithmic_Newton
              ( file : in file_type;
                f : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                jm : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                evt : in Standard_Evaluate_Deflation.Link_to_Eval_Tree;
                nd : in Standard_Jacobian_Trees.Link_to_Eval_Node;
                monkeys : in Standard_Natural_VecVecs.VecVec;
                k : in natural;
                nv,nq,R1 : in Standard_Natural_Vectors.Vector;
                B : in Standard_Complex_VecMats.VecMat;
                h : in Standard_Complex_VecVecs.VecVec;
                x : in out Standard_Complex_VecVecs.VecVec;
                err,rco,res : out double_float;
                tol : in double_float; rank : out natural ) is

    ans : character;
    cnt : natural := 0;
    m : constant natural := Min0(nq(k)+1,nv(k));
    s : Standard_Complex_Vectors.Vector(1..m);
    z : Standard_Complex_Vectors.Vector(1..nv(k));

  begin
    loop
      cnt := cnt + 1;
      z := Staple(nv(k),x);
      One_Algorithmic_Newton_Step
        (file,f,jm,evt,nd,monkeys,k,nv,nq,R1,B,h,x,z,tol,err,rco,res,s,rank);
      put_line("The singular values : "); put_line(s);
      Display_Results_of_One_Newton_Step
        (standard_output,z,tol,err,rco,res,rank,cnt);
      Display_Results_of_One_Newton_Step(file,z,tol,err,rco,res,rank,cnt);
      put("Do you want another iteration ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
    end loop;
  end Interactive_Algorithmic_Newton;

  procedure Deflate
               ( p : in out Standard_Complex_Poly_Systems.Link_to_Poly_Sys;
                 m : in natural ) is

    use Standard_Complex_Poly_Systems;
    dp : constant Poly_Sys := Standard_Deflate_Singularities.Deflate(p.all,m);

  begin
    Clear(p);
    p := new Poly_Sys'(dp);
  end Deflate;

  procedure Write_Results
               ( file : in file_type; i : natural;
                 p,dp : in Standard_Complex_Poly_Systems.Poly_Sys;
                 z : in Standard_Complex_Vectors.Vector;
                 err,rco,res : in double_float ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Solutions;

    n : constant natural := z'length;
    sols,sols_last : Solution_List;
    s : Solution(n);

  begin
    new_line(file);
    put(file,"DEFLATED SYSTEM #");
    put(file,i,1); put_line(file," :");
    Write_Deflated_System(file,p,dp);
    s.t := Create(1.0);
    s.m := 1;
    s.v := z;
    s.err := err; s.rco := rco; s.res := res;
    Append(sols,sols_last,s);
    new_line(file);
    put_line(file,"THE SOLUTIONS :");
    put(file,"1  "); put(file,n,1); new_line(file);
    put(file,sols);
  end Write_Results;

  procedure Interactive_Symbolic_Deflation
               ( file : in file_type;
                 p : in Standard_Complex_Poly_Systems.Poly_Sys;
                 sol : in Standard_Complex_Vectors.Vector;
                 tol : in double_float ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Vectors;
    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_Systems;
    use Standard_Complex_Solutions;

    n : constant natural := Number_of_Unknowns(p(p'first));
    wz : Link_to_Vector := new Vector'(sol);
    err,rco,res : double_float;
    wp : Link_to_Poly_Sys;
    m,k,rank : natural;
    ans : character;

  begin
    wp := new Poly_Sys(p'range);
    Copy(p,wp.all);
    k := 0;
    loop
      Interactive_Symbolic_Newton(file,wp.all,wz.all,err,rco,res,tol,rank);
      put("Do you want to deflate ? (y/n) ");
      Ask_Yes_or_No(ans);
      if ans = 'y'
       then m := rank+1;
            put("The number of multipliers : "); put(m,1); new_line; 
            Deflate(wp,m);
            k := k+1;
            Add_Multiplier_Symbols(k,m);     
            Add_Multipliers(wz,wp.all,m);
       else put("Do you want to continue ? (y/n) ");
            Ask_Yes_or_No(ans);
      end if;
      exit when (ans /= 'y');
    end loop;
    Write_Results(file,1,p,wp.all,wz.all,err,rco,res);
  end Interactive_Symbolic_Deflation;

  procedure Interactive_Algorithmic_Deflation
               ( file : in file_type;
                 p : in Standard_Complex_Poly_Systems.Poly_Sys;
                 sol : in Standard_Complex_Vectors.Vector;
                 tol : in double_float ) is

    use Standard_Complex_Numbers;
    use Standard_Random_Numbers;
    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Complex_Solutions;
    use Standard_Jacobian_Trees;
    use Standard_Evaluate_Deflation;

    max_order : constant natural := 3;
    n : constant natural := Number_of_Unknowns(p(p'first));
    nv : Standard_Natural_Vectors.Vector(0..max_order);
    nq : Standard_Natural_Vectors.Vector(0..max_order);
    R1 : Standard_Natural_Vectors.Vector(1..max_order);
    B : Standard_Complex_VecMats.VecMat(1..max_order);
    h : Standard_Complex_VecVecs.VecVec(1..max_order);
    x : Standard_Complex_VecVecs.VecVec(0..max_order);
    ep : Eval_Poly_Sys(p'range) := Create(p);
    jm : Jaco_Mat(p'range,1..n) := Create(p);
    ejm : Eval_Jaco_Mat(p'range,1..n) := Create(jm);
    monkeys : Standard_Natural_VecVecs.VecVec(1..max_order);
    nd : Link_to_Eval_Node;
    evt : Link_to_Eval_Tree;
    err,rco,res : double_float;
    m,k,rank : natural;
    ans : character;

  begin
    nv(0) := n;
    nq(0) := p'last;
    R1(1) := 0;
    put("creating a remember table for Jacobian matrices up to order ");
    put(max_order,1); put_line("...");
    Create_Remember_Derivatives(jm,max_order,nd);
    monkeys := Monomial_Keys(max_order,nv(0));
    k := 0;
    x(0) := new Standard_Complex_Vectors.Vector'(sol);
    loop
      if k = 0 then
        Interactive_Symbolic_Newton(file,p,x(0).all,err,rco,res,tol,rank);
      else
        Interactive_Algorithmic_Newton
          (file,ep,ejm,evt,nd,monkeys,k,nv(0..k),nq(0..k),R1(1..k),
           B(1..k),h(1..k),x(0..k),err,rco,res,tol,rank);
      end if;
      put("Do you want to deflate ? (y/n) ");
      Ask_Yes_or_No(ans);
      if ans = 'y' then
        m := rank+1;
        put("The number of multipliers : "); put(m,1); new_line; 
        k := k+1;
       -- Standard_Evaluate_Deflation.Clear(evt); OOPS!!
        evt := Create(k);
        Add_Multiplier_Symbols(k,m);     
        R1(k) := m;
        nv(k) := nv(k-1) + m;
        nq(k) := 2*nq(k-1) + 1;
        declare
          hk : Standard_Complex_Vectors.Vector(1..m);
          Bk : Standard_Complex_Matrices.Matrix(1..nv(k-1),1..m);
        begin
          for j in 1..m loop
            hk(j) := Random1;
          end loop;
          h(k) := new Standard_Complex_Vectors.Vector'(hk);
          for j1 in Bk'range(1) loop
            for j2 in 1..m loop
              Bk(j1,j2) := Random1;
            end loop;
          end loop;
          B(k) := new Standard_Complex_Matrices.Matrix'(Bk);
        end;
        Add_Multipliers(standard_output,ep,ejm,nd,monkeys,k,nv(0..k),
                        nq(0..k),R1(1..k),B(1..k),h(1..k),x(0..k));
      else
        put("Do you want to continue ? (y/n) ");
        Ask_Yes_or_No(ans);
      end if;
      exit when (ans /= 'y');
    end loop;
  end Interactive_Algorithmic_Deflation;

  procedure Interactive_Symbolic_Deflation
               ( file : in file_type;
                 p : in Standard_Complex_Poly_Systems.Poly_Sys;
                 sols : in Standard_Complex_Solutions.Solution_List;
                 tol : in double_float ) is

    use Standard_Complex_Numbers;
    use Standard_Complex_Vectors;
    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_Systems;
    use Standard_Complex_Solutions;

    n : constant natural := Number_of_Unknowns(p(p'first));
    z : Vector(1..n);
    wz : Link_to_Vector;
    err,rco,res : double_float;
    wp : Link_to_Poly_Sys;
    m,k,rank,len : natural;
    tmp : Solution_List := sols;
    ans : character;

  begin
    if Is_Null(sols)
     then put("Give "); put(n,1);
          put_line(" complex numbers to start at :");
          get(z);
          len := 1;
     else z := Head_Of(sols).v;
          len := Length_Of(sols);
    end if;
    for i in 1..len loop
      wp := new Poly_Sys(p'range);
      Copy(p,wp.all);
      k := 0;
      wz := new Vector'(z);
      loop
        Interactive_Symbolic_Newton(file,wp.all,wz.all,err,rco,res,tol,rank);
        put("Do you want to deflate ? (y/n) ");
        Ask_Yes_or_No(ans);
        if ans = 'y'
         then m := rank+1;
              put("The number of multipliers : "); put(m,1); new_line; 
              Deflate(wp,m);
              k := k+1;
              Add_Multiplier_Symbols(k,m);     
              Add_Multipliers(wz,wp.all,m);
         else put("Do you want to continue on this root ? (y/n) ");
              Ask_Yes_or_No(ans);
        end if;
        exit when (ans /= 'y');
      end loop;
      Write_Results(file,i,p,wp.all,wz.all,err,rco,res);
      if i < len
       then put("Move on to root #"); put(i+1,1); put(" ? (y/n) ");
            Ask_Yes_or_No(ans);
            exit when (ans /= 'y');
            tmp := Tail_Of(tmp);
            z := Head_Of(tmp).v;
      end if;
    end loop;
  end Interactive_Symbolic_Deflation;

  procedure Interactive_Algorithmic_Deflation
               ( file : in file_type;
                 p : in Standard_Complex_Poly_Systems.Poly_Sys;
                 sols : in Standard_Complex_Solutions.Solution_List;
                 tol : in double_float ) is

    use Standard_Complex_Numbers;
    use Standard_Random_Numbers;
    use Standard_Complex_Polynomials;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Complex_Solutions;
    use Standard_Jacobian_Trees;
    use Standard_Evaluate_Deflation;

    max_order : constant natural := 3;
    n : constant natural := Number_of_Unknowns(p(p'first));
    nv : Standard_Natural_Vectors.Vector(0..max_order);
    nq : Standard_Natural_Vectors.Vector(0..max_order);
    R1 : Standard_Natural_Vectors.Vector(1..max_order);
    B : Standard_Complex_VecMats.VecMat(1..max_order);
    h : Standard_Complex_VecVecs.VecVec(1..max_order);
    x : Standard_Complex_VecVecs.VecVec(0..max_order);
    ep : Eval_Poly_Sys(p'range) := Create(p);
    jm : Jaco_Mat(p'range,1..n) := Create(p);
    ejm : Eval_Jaco_Mat(p'range,1..n) := Create(jm);
    monkeys : Standard_Natural_VecVecs.VecVec(1..max_order);
    nd : Link_to_Eval_Node;
    evt : Link_to_Eval_Tree;
    z : Standard_Complex_Vectors.Vector(1..n);
    err,rco,res : double_float;
    m,k,rank,len : natural;
    tmp : Solution_List := sols;
    ans : character;

  begin
    nv(0) := n;
    nq(0) := p'last;
    R1(1) := 0;
    put("creating a remember table for Jacobian matrices up to order ");
    put(max_order,1); put_line("...");
    Create_Remember_Derivatives(jm,max_order,nd);
    monkeys := Monomial_Keys(max_order,nv(0));
    if Is_Null(sols)
     then put("Give "); put(n,1);
          put_line(" complex numbers to start at :");
          get(z);
          len := 1;
     else z := Head_Of(sols).v;
          len := Length_Of(sols);
    end if;
    for i in 1..len loop
      k := 0;
      x(0) := new Standard_Complex_Vectors.Vector'(z);
      loop
        if k = 0 then
          Interactive_Symbolic_Newton(file,p,x(0).all,err,rco,res,tol,rank);
        else
          Interactive_Algorithmic_Newton
            (file,ep,ejm,evt,nd,monkeys,k,nv(0..k),nq(0..k),R1(1..k),
             B(1..k),h(1..k),x(0..k),err,rco,res,tol,rank);
        end if;
        put("Do you want to deflate ? (y/n) ");
        Ask_Yes_or_No(ans);
        if ans = 'y' then
          m := rank+1;
          put("The number of multipliers : "); put(m,1); new_line; 
          k := k+1;
         -- Standard_Evaluate_Deflation.Clear(evt); OOPS!!
          evt := Create(k);
          Add_Multiplier_Symbols(k,m);     
          R1(k) := m;
          nv(k) := nv(k-1) + m;
          nq(k) := 2*nq(k-1) + 1;
          declare
            hk : Standard_Complex_Vectors.Vector(1..m);
            Bk : Standard_Complex_Matrices.Matrix(1..nv(k-1),1..m);
          begin
            for j in 1..m loop
              hk(j) := Random1;
            end loop;
            h(k) := new Standard_Complex_Vectors.Vector'(hk);
            for j1 in Bk'range(1) loop
              for j2 in 1..m loop
                Bk(j1,j2) := Random1;
              end loop;
            end loop;
            B(k) := new Standard_Complex_Matrices.Matrix'(Bk);
          end;
          Add_Multipliers(standard_output,ep,ejm,nd,monkeys,k,nv(0..k),
                          nq(0..k),R1(1..k),B(1..k),h(1..k),x(0..k));
        else
          put("Do you want to continue on this root ? (y/n) ");
          Ask_Yes_or_No(ans);
        end if;
        exit when (ans /= 'y');
      end loop;
     -- Write_Results(file,i,p,wp.all,wz.all,err,rco,res);
      if i < len
       then put("Move on to root #"); put(i+1,1); put(" ? (y/n) ");
            Ask_Yes_or_No(ans);
            exit when (ans /= 'y');
            tmp := Tail_Of(tmp);
            z := Head_Of(tmp).v;
      end if;
    end loop;
  end Interactive_Algorithmic_Deflation;

-- NEWTON with DEFLATION and CLUSTERING : 

  function Equal ( n : natural; tol : double_float;
                   s1,s2 : Standard_Complex_Vectors.Vector )
                 return boolean is

    use Standard_Complex_Numbers;

  begin
    for i in 1..n loop
      if AbsVal(s1(i)-s2(i)) > tol
       then return false;
      end if;
    end loop;
    return true;
  end Equal;

  procedure Set_Multiplicity
              ( sols : in out Standard_Complex_Solutions.Solution_List;
                s : in Standard_Complex_Solutions.Solution;
                tol : in double_float; n,m : in natural ) is

    use Standard_Complex_Solutions;

    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Equal(n,tol,ls.v,s.v)
       then ls.m := m;
            Set_Head(tmp,ls);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
  end Set_Multiplicity;

  function Number_of_Occurrences 
              ( sols : Standard_Complex_Solutions.Solution_List;
                s : Standard_Complex_Solutions.Solution;
                tol : in double_float; n : in natural ) return natural is

    use Standard_Complex_Solutions;

    res : natural := 0;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Equal(n,tol,ls.v,s.v)
       then res := res+1;
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return res;
  end Number_of_Occurrences;

  function Is_In ( sols : Standard_Complex_Solutions.Solution_List;
                   v : Standard_Complex_Vectors.Vector;
                   tol : double_float; n : natural ) return boolean is

    use Standard_Complex_Solutions;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if Equal(n,tol,ls.v,v)
       then return true;
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    return false;
  end Is_In;

  procedure Remove_Duplicates
              ( sols : in out Standard_Complex_Solutions.Solution_List;
                tol : in double_float; n : in natural ) is

    use Standard_Complex_Solutions;
    res,res_last : Solution_List;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      if not Is_In(res,ls.v,tol,n)
       then Append(res,res_last,ls.all);
      end if;
      tmp := Tail_Of(tmp);
    end loop;
    Clear(sols);
    sols := res;
  end Remove_Duplicates;

  procedure Compute_Multiplicities
              ( sols : in out Standard_Complex_Solutions.Solution_List;
                tol : in double_float; n : in natural ) is

    use Standard_Complex_Solutions;

    tmp : Solution_List := sols;
    ls : Link_to_Solution;
    m : natural;

  begin
    while not Is_Null(tmp) loop
      ls := Head_Of(tmp);
      m := Number_of_Occurrences(sols,ls.all,tol,n);
      Set_Multiplicity(sols,ls.all,tol,n,m);
      tmp := Tail_Of(tmp);
    end loop;
  end Compute_Multiplicities;

  procedure Compute_Multiplicities
              ( nd : in out Standard_Deflation_Trees.Node;
                tol : in double_float; n : in natural ) is

    use Standard_Complex_Solutions;
    use Standard_Deflation_Trees;

  begin
    if not Is_Null(nd.sols)
     then Compute_Multiplicities(nd.sols,tol,n);
          Remove_Duplicates(nd.sols,tol,n);
    end if;
    for i in nd.children'range loop
      if nd.children(i) /= null
       then Compute_Multiplicities(nd.children(i).all,tol,n);
      end if;
    end loop;
  end Compute_Multiplicities;

  procedure Deflate_Solution
              ( file : in file_type; m : in natural; output : in boolean;
                df : in Standard_Complex_Poly_Systems.Poly_Sys;
                ls : in out Standard_Complex_Solutions.Link_to_Solution ) is

    use Standard_Complex_Vectors;
    use Standard_Complex_Solutions;
    use Standard_Deflate_Singularities;

    ns : Solution(ls.n+m);
    la : Vector(1..m);
    res : double_float;

  begin
    Multipliers(df,ls.v,m,la,res);
    if output
     then put_line(file,"Values computed for the multipliers :");
          put_line(file,la);
          put(file,"Residual of multiplier computation :");
          put(file,res,3); new_line(file);
    end if;
    ns.v(ls.v'range) := ls.v;
    for i in la'range loop
      ns.v(ls.v'last+i) := la(i);
    end loop;
    ns.t := ls.t; ns.m := ls.m;
    ns.err := ls.err; ns.rco := ls.rco; ns.res := ls.res;
    Clear(ls);
    ls := new Solution'(ns);
  end Deflate_Solution;

  procedure Apply_Newton
              ( file : in file_type; output : in boolean; cnt : in natural;
                f : in Standard_Complex_Poly_SysFun.Eval_Poly_Sys;
                jf : in Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                ls : in out Standard_Complex_Solutions.Link_to_Solution;
                tolrnk : in double_float; rank : out natural ) is

    sv : Standard_Complex_Vectors.Vector(ls.v'range);

  begin
    if output then
      One_Symbolic_Newton_Step
        (file,f,jf,ls.v,tolrnk,ls.err,ls.rco,ls.res,sv,rank);
      Display_Results_of_One_Newton_Step
        (file,ls.v,tolrnk,ls.err,ls.rco,ls.res,rank,cnt);
    else
      One_Symbolic_Newton_Step(f,jf,ls.v,tolrnk,ls.err,ls.rco,ls.res,sv,rank);
    end if;
  end Apply_Newton;

  procedure Write_Conclusion
              ( file : in file_type; fail : in boolean; i : in natural) is
  begin
    put(file,"Convergence criteria");
    if not fail
     then put(file," are reached for solution ");
     else put(file," NOT reached for solution ");
    end if;
    put(file,i,1); put_line(file,".");
  end Write_Conclusion;

  procedure Symbolic_Deflation_and_Clustering
              ( file : in file_type; outfilename : in string;
                p : in Standard_Complex_Poly_Systems.Poly_Sys;
                sols : in Standard_Complex_Solutions.Solution_List;
                output : in boolean; maxitr,maxdef : in natural;
	        tolerr,tolres,tolrnk : in double_float ) is

    use Standard_Complex_Polynomials;
    use Standard_Complex_Solutions;
    use Standard_Deflation_Trees;

    k,rank,m,nit : natural := 0;
    tmp : Solution_List := sols;
    ls : Link_to_Solution;
    fail : boolean;
    ne : constant natural := p'last;
    nv : constant natural := Number_of_Unknowns(p(p'first));
    t : Node(ne,nv) := Create_Root(p);
    lt : Link_to_Node := new Node'(t);
    nd : Link_to_Node;

  begin
    for i in 1..Length_Of(sols) loop
      ls := Head_Of(tmp);
      if output
       then put(file,"SOLUTION #"); put(file,i,1); put_line(file," :");
      end if;
      nd := lt; k := 0; nit := 0; fail := true;
      loop
        nit := nit + 1;
        exit when (nit > maxitr);
        Apply_Newton(file,output,nit,nd.f,nd.jf,ls,tolrnk,rank);
        if rank < ls.n and k < maxdef then
          k := k + 1;
          if output
           then put(file,"Deflation #"); put(file,k,1);
                put(file," for solution "); put(file,i,1); new_line(file);
          end if;
          m := rank + 1;
          if nd.children(m) = null then
            if output then Add_Multiplier_Symbols(k,m); end if;
            Create_Child(nd.all,m);
          end if;
          nd := nd.children(m);
          Deflate_Solution(file,m,output,nd.s,ls);
        else
          if (ls.err <= tolerr) or (ls.res <= tolres) then
            fail := false; exit;
          end if;
        end if;
      end loop;
      if output then Write_Conclusion(file,fail,i); end if;
      Append(nd.sols,nd.last,ls.all);
      tmp := Tail_Of(tmp);
    end loop;
    Compute_Multiplicities(lt.all,1.0E-8,nv);
    Write(file,outfilename,lt.all);
    Clear(lt);
  end Symbolic_Deflation_and_Clustering;

end Standard_Deflation_Methods;
