with integer_io;                        use integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Complex_QR_Least_Squares; use Standard_Complex_QR_Least_Squares;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Functions;   use Standard_Complex_Poly_Functions;
with Standard_Complex_Poly_SysFun;      use Standard_Complex_Poly_SysFun;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Monomial_Hashing;                  use Monomial_Hashing;
with Standard_Numerical_Rank;           use Standard_Numerical_Rank;
 
package body Standard_Multiplicity_Structure is

-- AUXILIARY OPERATIONS :

  function Derivative ( p : Poly;
                        m : Standard_Natural_Vectors.Vector ) return Poly is

  -- DESCRIPTION :
  --   Returns the derivative of p with respect to the variables in m.

    res : Poly;

  begin
    Copy(p,res);
    for i in m'range loop
      if m(i) > 0 then
        for j in 1..m(i) loop
          Diff(res,i);
          exit when (res = Null_Poly);
        end loop; 
      end if;
    end loop;
    return res;
  end Derivative;

  function Factorial ( m : Standard_Natural_Vectors.Vector ) return natural is

  -- DESCRIPTION :
  --   Returns the product of all factorials of the elements in m.

    res : natural := 1;

  begin
    for i in m'range loop
      if m(i) > 1
       then for j in 2..m(i) loop
              res := res*j;
            end loop;
      end if;
    end loop;
    return res;
  end Factorial;

  function Monomial_Multiple
              ( m : Standard_Natural_Vectors.Vector;
                f : Poly_Sys ) return Poly_Sys is

  -- DESCRIPTION :
  --   Returns x^m*f.

    res : Poly_Sys(f'range);
    t : Term;

  begin
    t.cf := Create(1.0);
    t.dg := new Standard_Natural_Vectors.Vector'(m);
    for i in f'range loop
      res(i) := t*f(i);
    end loop;
    Clear(t);
    return res;
  end Monomial_Multiple;

  procedure Evaluate_Derivatives
              ( a : in out Matrix; r : in natural; c : in out natural;
                nq,nv,k : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates all derivatives up to order k of the system f at z,
  --   in the rows of the matrix a, starting at given row r and column c.

    procedure Monomial ( m : in Standard_Natural_Vectors.Vector ) is

      dp : Poly;
      row : natural := r;
      fac : constant double_float := double_float(Factorial(m));

    begin
      for i in 1..nq loop
        dp := Derivative(f(i),m);
        if dp = Null_Poly
         then a(row,c) := Create(0.0);
         else a(row,c) := Eval(dp,z)/fac;
              Clear(dp);
        end if;
        row := row + 1;
      end loop;       
      c := c + 1;
    end Monomial;
    procedure Enumerate is new Enumerate_Monomials(Monomial);

  begin
    Enumerate(k,nv);
  end Evaluate_Derivatives;

  procedure Evaluate_Derivatives
              ( file : in file_type;
                a : in out Matrix; r : in natural; c : in out natural;
                nq,nv,k : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates all derivatives up to order k of the system f at z,
  --   in the rows of the matrix a, starting at given row r and column c.

    procedure Monomial ( m : in Standard_Natural_Vectors.Vector ) is

      dp : Poly;
      row : natural := r;
      fac : constant double_float := double_float(Factorial(m));

    begin
      for i in 1..nq loop
        dp := Derivative(f(i),m);
        a(row,c) := Eval(dp,z)/fac;
       -- put(file,"row = "); put(file,row,1);
       -- put(file,"  column = "); put(file,c,1); new_line(file);
        Clear(dp);
        row := row + 1;
      end loop;       
      c := c + 1;
    end Monomial;
    procedure Enumerate is new Enumerate_Monomials(Monomial);

  begin
    Enumerate(k,nv);
  end Evaluate_Derivatives;

  procedure Evaluate_All_Derivatives
              ( a : in out Matrix; r,c : in natural; nq,nv,k : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates all derivates of multiples of f with all monomials
  --   of degree k-1.  The results are places in the matrix a, 
  --   starting at row r and column c.

    wr : natural := r;

    procedure Monomial ( m : in Standard_Natural_Vectors.Vector ) is
 
    -- DESCRIPTION :
    --   Evaluates all derivatives up to order k of m*f.

      mf : Poly_Sys(f'range) := Monomial_Multiple(m,f);
      row : natural := wr;
      wc : natural;
 
    begin
      for i in 1..nq loop
        a(row,c) := Eval(mf(i),z);
        row := row + 1;
      end loop;
      wc := c+1;
      for i in 1..k loop
        Evaluate_Derivatives(a,wr,wc,nq,nv,i,mf,z);
      end loop;
      wr := wr + nq;
      Clear(mf);
    end Monomial;
    procedure Enumerate is new Enumerate_Monomials(Monomial);

  begin
    Enumerate(k-1,nv);
  end Evaluate_All_Derivatives;

  procedure Evaluate_Highest_Order
              ( a : in out Matrix; r,c : in natural; nq,nv,k : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates the k-th order derivative of f at all monomial multiples of
  --   degree 1 to k-1, placing the results in a, starting at row r and 
  --   starting at row r and column c.

    wr : natural := r;

    procedure Highest_Order ( m : in Standard_Natural_Vectors.Vector ) is

    -- DESCRIPTION :
    --   Evaluates the highest order derivative of m*f at z.

      mf : Poly_Sys(f'range) := Monomial_Multiple(m,f);
      wc : natural := c;

    begin
      Evaluate_Derivatives(a,wr,wc,nq,nv,k,mf,z);
      wr := wr+nq;
    end Highest_Order;
    procedure Enumerate is new Enumerate_Monomials(Highest_Order);

  begin
    for i in 1..k-1 loop
      Enumerate(i,nv);
    end loop;
  end Evaluate_Highest_Order;

  procedure Evaluate_Monomial_Multiples
              ( a : in out Matrix; r,c,nq,nv,k,nc1 : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates monomial multiples of the degree k with the system f
  --   in the rows of the matrix a, starting at given row r and column c.

    wr,wc : natural;

  begin
    wc := nc1+1;
    Evaluate_Derivatives(a,1,wc,nq,nv,k,f,z);
    wr := nq+1;
    wc := nc1+1;
    Evaluate_Highest_Order(a,wr,wc,nq,nv,k,f,z);
    wr := r;
    Evaluate_All_Derivatives(a,r,1,nq,nv,k,f,z);
  end Evaluate_Monomial_Multiples;

  procedure Evaluate_Monomial_Multiples
              ( file : in file_type;
                a : in out Matrix; r,c,nq,nv,k,nc1 : in natural;
                f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Evaluates monomial multiples of the degree k with the system f
  --   in the rows of the matrix a, starting at given row r and column c.

    wr,wc : natural;

    procedure Monomial ( m : in Standard_Natural_Vectors.Vector ) is
 
    -- DESCRIPTION :
    --   Evaluates all derivatives up to order k of m*f.

      mf : Poly_Sys(f'range) := Monomial_Multiple(m,f);
      row : natural := wr;
 
    begin
      put(file,"multiply f with monomial "); put(file,m); new_line(file);
      for i in 1..nq loop
        put(file,"row = "); put(file,row,1);
        put(file,"  column = "); put(file,c,1); new_line(file);
        a(row,c) := Eval(mf(i),z);
        row := row + 1;
      end loop;
      wc := c+1;
      for i in 1..k loop
        Evaluate_Derivatives(file,a,wr,wc,nq,nv,i,mf,z);
      end loop;
      wr := wr + nq;
      Clear(mf);
    end Monomial;
    procedure Evaluate_All_Derivatives is new Enumerate_Monomials(Monomial);

    procedure Highest_Order ( m : in Standard_Natural_Vectors.Vector ) is

    -- DESCRIPTION :
    --   Evaluates the highest order derivative of m*f at z.

      mf : Poly_Sys(f'range) := Monomial_Multiple(m,f);

    begin
      put(file,"multiply f with monomial "); put(file,m); new_line(file);
      wc := nc1+1;
      Evaluate_Derivatives(file,a,wr,wc,nq,nv,k,mf,z);
      wr := wr+nq;
    end Highest_Order;
    procedure Evaluate_Highest_Order is new Enumerate_Monomials(Highest_Order);

  begin
    wc := nc1+1;
    put_line(file,"evaluating highest order for i = 0");
    Evaluate_Derivatives(file,a,1,wc,nq,nv,k,f,z);
    wr := nq+1;
    for i in 1..k-1 loop
      put(file,"evaluating highest order for i = ");
      put(file,i,1); new_line(file);
      Evaluate_Highest_Order(i,nv);
    end loop;
    wr := r;
    Evaluate_All_Derivatives(k-1,nv);
  end Evaluate_Monomial_Multiples;

-- TARGET OPERATIONS :

  procedure Dimensions_of_Nullity_Matrix
              ( nq,nv : in natural; k : in positive; nr,nc : out natural ) is

    nb0,nb1,nr1,nc1 : natural;

  begin
    if k = 1 
     then nr := nq;
          nc := nv+1;
     else nb0 := Monomial_Count(k,nv);
          nb1 := Monomial_Count(k-1,nv);
          Dimensions_of_Nullity_Matrix(nq,nv,k-1,nr1,nc1);
          nr := nr1 + nb1*nq;
          nc := nc1 + nb0;
    end if;
  end Dimensions_of_Nullity_Matrix;

  function Evaluate_Nullity_Matrix
             ( nq,nv,nr,nc : natural; k : positive; f : Poly_Sys;
               z : Standard_Complex_Vectors.Vector ) return Matrix is

    res : Matrix(1..nr,1..nc);
    nr1,nc1 : natural;
    y : Standard_Complex_Vectors.Vector(1..nq);
    dp : Poly;

  begin
    if k = 1 then
      y := Eval(f,z);
      for i in y'range loop
        res(i,1) := y(i);
        for j in 1..nv loop
          dp := Diff(f(i),j);
          res(i,j+1) := Eval(dp,z);
          Clear(dp);
        end loop;
      end loop;
    else
      Dimensions_of_Nullity_Matrix(nq,nv,k-1,nr1,nc1);
      declare
        eva : Matrix(1..nr1,1..nc1)
            := Evaluate_Nullity_Matrix(nq,nv,nr1,nc1,k-1,f,z);
      begin
        for i in 1..nr1 loop
          for j in 1..nc1 loop
            res(i,j) := eva(i,j);
          end loop;
        end loop;
      end;
      Evaluate_Monomial_Multiples(res,nr1+1,1,nq,nv,k,nc1,f,z);
    end if;
    return res;
  end Evaluate_Nullity_Matrix;

  function Evaluate_Nullity_Matrix
             ( nq,nv,nr,nc : natural; k : positive; a1 : Matrix;
               f : Poly_Sys; z : Standard_Complex_Vectors.Vector )
             return Matrix is

    res : Matrix(1..nr,1..nc);
    nr1,nc1 : natural;
    y : Standard_Complex_Vectors.Vector(1..nq);

  begin
    if k = 1 then
      y := Eval(f,z);
      for i in y'range loop
        res(i,1) := y(i);
        for j in a1'range(2) loop
          res(i,j+1) := a1(i,j);
        end loop;
      end loop;
    else
      nr1 := a1'last(1);
      nc1 := a1'last(2);
      for i in 1..nr1 loop
        for j in 1..nc1 loop
          res(i,j) := a1(i,j);
        end loop;
      end loop;
      Evaluate_Monomial_Multiples(res,nr1+1,1,nq,nv,k,nc1,f,z);
    end if;
    return res;
  end Evaluate_Nullity_Matrix;

  function Evaluate_Nullity_Matrix
             ( file : file_type;
               nq,nv,nr,nc : natural; k : positive; f : Poly_Sys;
               z : Standard_Complex_Vectors.Vector ) return Matrix is

    res : Matrix(1..nr,1..nc);
    nr1,nc1 : natural;
    y : Standard_Complex_Vectors.Vector(1..nq);
    dp : Poly;

  begin
    put(file,"evaluating a "); put(file,nr,1); put(file,"-by-");
    put(file,nc,1); put(file," nullity matrix for differential order ");
    put(file,k,1); put_line(file,"...");
    if k = 1 then
      y := Eval(f,z);
      for i in y'range loop
        res(i,1) := y(i);
        for j in 1..nv loop
          dp := Diff(f(i),j);
          res(i,j+1) := Eval(dp,z);
          Clear(dp);
        end loop;
      end loop;
    else
      Dimensions_of_Nullity_Matrix(nq,nv,k-1,nr1,nc1);
      declare
        eva : Matrix(1..nr1,1..nc1)
            := Evaluate_Nullity_Matrix(file,nq,nv,nr1,nc1,k-1,f,z);
      begin
        for i in 1..nr1 loop
          for j in 1..nc1 loop
            res(i,j) := eva(i,j);
          end loop;
        end loop;
      end;
      Evaluate_Monomial_Multiples(file,res,nr1+1,1,nq,nv,k,nc1,f,z);
    end if;
    return res;
  end Evaluate_Nullity_Matrix;

  procedure Orthogonal_Basis
              ( nr,nc : in natural; a : in Matrix; b : out Matrix ) is

    qraux : Standard_Complex_Vectors.Vector(1..nc) := (1..nc => Create(0.0));
    jpvt : Standard_Natural_Vectors.Vector(1..nc) := (1..nc => 0);

  begin
    b := a;
    QRD(b,qraux,jpvt,false);
  end Orthogonal_Basis;

  procedure Multiplicity_Structure
             ( f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector;
               tol : in double_float; max_h : in natural;
               h : out Standard_Natural_Vectors.Vector; m : out natural ) is

    nq : constant natural := f'last;
    nv : constant natural := z'last;
    nr,nc : natural;
    nullity : Standard_Natural_Vectors.Vector(0..max_h);

  begin
    h := (0..max_h => 0);
    h(0) := 1;
    nullity(0) := 0;          -- makes a nice exit in loop below
    for i in 1..max_h loop
      Dimensions_of_Nullity_Matrix(nq,nv,i,nr,nc);
      declare
        eva : Matrix(1..nr,1..nc)
            := Evaluate_Nullity_Matrix(nq,nv,nr,nc,i,f,z);
        rnk : constant natural := Numerical_Rank(eva,tol);
      begin
        nullity(i) := nc - rnk;
      end;
      exit when (nullity(i) = nullity(i-1));
      if i = 1
       then h(i) := nullity(i) - 1;
       else h(i) := nullity(i) - nullity(i-1);
      end if;
    end loop;
    m := Standard_Natural_Vectors.Sum(h);
  end Multiplicity_Structure;

  procedure Multiplicity_Structure
             ( file : in file_type;
               f : in Poly_Sys; z : in Standard_Complex_Vectors.Vector;
               tol : in double_float; max_h : in natural;
               h : out Standard_Natural_Vectors.Vector; m : out natural ) is

    nq : constant natural := f'last;
    nv : constant natural := z'last;
    nr,nc : natural;
    nullity : Standard_Natural_Vectors.Vector(0..max_h);

  begin
    h := (0..max_h => 0);
    h(0) := 1;
    nullity(0) := 0;          -- makes a nice exit in loop below
    for i in 1..max_h loop
      Dimensions_of_Nullity_Matrix(nq,nv,i,nr,nc);
      declare
        eva : Matrix(1..nr,1..nc)
            := Evaluate_Nullity_Matrix(file,nq,nv,nr,nc,i,f,z);
        rnk : constant natural := Numerical_Rank(eva,tol);
      begin
        nullity(i) := nc - rnk;
        put(file,"nullity("); put(file,i,1); put(file,") = ");
        put(file,nullity(i),1);
      end;
      exit when (nullity(i) = nullity(i-1));
      if i = 1
       then h(i) := nullity(i) - 1;
       else h(i) := nullity(i) - nullity(i-1);
      end if;
      put(file,"  h("); put(file,i,1); put(") = ");
      put(file,h(i),1); new_line(file);
    end loop;
    new_line(file);
    m := Standard_Natural_Vectors.Sum(h);
  end Multiplicity_Structure;

  procedure Multiplicity_Structure
             ( file : in file_type; output : in boolean;
               f : in Poly_Sys; sols : in out Solution_List;
               tol : in double_float; max_h : in natural ) is

    nv : constant natural := Number_of_Unknowns(f(f'first));
    tmp : Solution_List := sols;
    ls : Link_to_Solution;

  begin
    new_line(file);
    for i in 1..Length_Of(sols) loop
      ls := Head_Of(tmp);
      declare
        z : Standard_Complex_Vectors.Vector(1..nv) := ls.v;
        h : Standard_Natural_Vectors.Vector(0..max_h);
        m : natural;
      begin
        if output
         then Multiplicity_Structure(file,f,z,tol,max_h,h,m);
         else Multiplicity_Structure(f,z,tol,max_h,h,m);
        end if;
        put(file,"Multiplicity Structure of solution ");
        put(file,i,1); put_line(file," :");
        put(file,"  values of the Hilbert function : ");
        put(file,h); new_line(file);
        put(file,"  the multiplicity : "); put(file,m,1); new_line(file);
        ls.m := m;
      end;
      Set_Head(tmp,ls);
      tmp := Tail_Of(tmp);
    end loop;
  end Multiplicity_Structure;

  procedure Driver_to_Multiplicity_Structure
             ( file : in file_type;
               f : in Poly_Sys; sols : in out Solution_List ) is

    max_h : natural := 3;
    tol : double_float := 1.0E-8;
    ans : character;
    first,output : boolean;

  begin
    new_line;
    first := true;
    loop
      if first then
        put("Default value for maximal differential order is ");
        first := false;
      else
        put("Current value for maximal differential order is ");
      end if;
      put(max_h,1); new_line;
      put("Do you want to change this value ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when ans = 'n';
      put("Give new value for the maximal differential order : ");
      get(max_h);
    end loop;
    first := true;
    loop
      if first then
        put("Default tolerance for numerical rank is ");
        first := false;
      else
        put("Current tolerance for numerical rank is ");
      end if;
      put(tol,3); new_line;
      put("Do you want to change this value ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when ans = 'n';
      put("Give new value for tolerance : "); get(tol);
    end loop;
    put("Do you want intermediate output ? (y/n) ");
    Ask_Yes_or_No(ans);
    output := (ans = 'y');
    Multiplicity_Structure(file,output,f,sols,tol,max_h);
    new_line(file);
    put_line(file,"THE SOLUTIONS :");
    put(file,Length_Of(sols),Head_Of(sols).n,sols);
  end Driver_to_Multiplicity_Structure;

end Standard_Multiplicity_Structure;
