with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Polynomials_io;   use Standard_Complex_Polynomials_io;
with Standard_Complex_Poly_Systems;     use Standard_Complex_Poly_Systems;
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Jaco_Matrices;    use Standard_Complex_Jaco_Matrices;
with Standard_Derivative_Trees; 
with Standard_Jacobian_Trees;

procedure ts_dertrees is

-- DESCRIPTION :
--   Computes all derivatives of a polynomial system.

  procedure Show_all_Derivatives ( p : in Poly_Sys );

  procedure Show_all_Derivatives ( p : in Poly ) is

    n : constant natural := Number_of_Unknowns(p);
    dp : Poly_Sys(1..n);

  begin
    for i in 1..n loop
      put("Derivative w.r.t. "); put(i,1); put(" : ");
      dp(i) := Diff(p,i);
      put(dp(i)); new_line;
    end loop;
    Show_all_Derivatives(dp);
  end Show_all_Derivatives;

  procedure Show_all_Derivatives ( p : in Poly_Sys ) is
  begin
    for i in p'range loop
      if p(i) /= Null_Poly then
        put("All derivatives of polynomial ");
        put(i,1); put_line(" : ");
        Show_all_Derivatives(p(i));
      end if;
    end loop;
  end Show_all_Derivatives;

  procedure Write_Derivative_Nodes
              ( rnd : in Standard_Derivative_Trees.Array_of_Nodes ) is

    use Standard_Derivative_Trees;

    procedure Write ( nd : in Node; cont : out boolean ) is
    begin
      put("Polynomial "); put(nd.p); put_line(" has derivatives : ");
      for i in nd.d'range loop
        put("  w.r.t. "); put(i,1); put(" : ");
        if nd.d(i) = null
         then put_line("0");
         else put(nd.d(i).p); new_line;
        end if;
      end loop;
      cont := true;
    end Write;
    procedure Write_all_Derivatives is new Enumerate_Nodes(Write);

  begin
    Write_all_Derivatives(rnd);
  end Write_Derivative_Nodes;

  procedure Write ( jm : in Jaco_Mat ) is
  begin
    for i in jm'range(1) loop
      for j in jm'range(2) loop
        put("  "); put(jm(i,j));
      end loop;
      new_line;
    end loop;
  end Write;

  procedure Write_Jacobian_Nodes
              ( nd : in Standard_Jacobian_Trees.Node ) is

    use Standard_Jacobian_Trees;

    procedure Write ( nd : Node; cont : out boolean ) is
    begin
      put_line("Jacobian matrix"); Write(nd.a.all); 
      put_line("has derivatives");
      for i in nd.d'range loop
        put("  w.r.t. "); put(i,1); put(" : ");
        if nd.d(i) = null
         then put_line("0");
         else new_line; Write(nd.d(i).a.all);
        end if;
      end loop;
      cont := true;
    end Write;
    procedure Write_all_Jacobian_Matrices is new Enumerate(Write);

  begin
    Write_all_Jacobian_Matrices(nd);
  end Write_Jacobian_Nodes;

  procedure Enumerate_all_Derivatives ( p : in Poly_Sys ) is

    use Standard_Derivative_Trees;

    dp : Array_of_Nodes(p'range) := Create(p);

  begin
    Write_Derivative_Nodes(dp);
  end Enumerate_all_Derivatives;

  procedure Enumerate_all_Jacobian_Matrices ( p : in Poly_Sys ) is

    use Standard_Jacobian_Trees;

    n : constant natural := Number_of_Unknowns(p(p'first));
    jm : Jaco_Mat(p'range(1),1..n) := Create(p);
    dp : Node(n) := Create(jm);

  begin
    Write_Jacobian_Nodes(dp);
  end Enumerate_all_Jacobian_Matrices;

  procedure Read_Vector ( v : in out Vector )is
  begin
    for i in v'range loop
      put("How many times to differentiate w.r.t. ");
      put(i,1); put(" ? ");
      get(v(i));
    end loop;
  end Read_Vector;

  procedure Interactive_Derivatives ( p : in Poly_Sys ) is

    use Standard_Derivative_Trees;

    k : natural;
    dp : Array_of_Nodes(p'range) := Initialize(p);
    n : constant natural := dp(dp'first).n;
    v : Vector(1..n);
    d : Poly;
    ans : character;

  begin
    loop
      put("Give current polynomial in the system to differentiate : ");
      get(k);
      Read_Vector(v);
      Derivative(dp(k).all,v,d);
      put("The derivative of "); put(dp(k).p);
      put(" w.r.t. "); put(v); put(" is "); put(d); new_line;
      Clear(d);
      put_line("The current tree of derivatives : ");
      Write_Derivative_Nodes(dp);
      put("Do you wish to continue ? (y/n) "); 
      Ask_Yes_or_No(ans);
      exit when ans /= 'y';
    end loop;
  end Interactive_Derivatives;

  procedure Interactive_Jacobians ( p : in Poly_Sys ) is

    use Standard_Jacobian_Trees;

    n : constant natural := Number_of_Unknowns(p(p'first));
    jm : Jaco_Mat(p'range,1..n) := Create(p);
    nd : Node(n) := Initialize(jm);
    v : Vector(1..n);
    da : Link_to_Jaco_Mat;
    ans : character;

  begin
    loop
      Read_Vector(v);
      Derivative(nd,v,da);
      put_line("The derivative of "); Write(nd.a.all);
      put(" w.r.t. "); put(v); put(" is ");
      if da = null
       then put_line("0");
       else new_line; Write(da.all);
      end if;
      put_line("The current tree of derivatives : ");
      Write_Jacobian_Nodes(nd);
      put("Do you wish to continue ? (y/n) "); 
      Ask_Yes_or_No(ans);
      exit when ans /= 'y';
    end loop;
  end Interactive_Jacobians;

  procedure Main is

    lp : Link_to_Poly_Sys;
    ans : character;

  begin
    new_line;
    put_line("Testing the calculation of trees of derivatives ...");
    new_line;
    get(lp);
    new_line;
    put_line("MENU to Test Operations in Trees of Derivatives :");
    put_line("  0. show all derivatives in recursive calculation;");
    put_line("  1. complete tree calculation of system and printing;");
    put_line("  2. interactive selective calculations for system;");
    put_line("  3. complete tree calculation of all Jacobian matrices;");
    put_line("  4. interactive selective calculations of Jacobian matrices.");
    put("Type 0, 1, 2, 3, or 4 to select : ");
    Ask_Alternative(ans,"01234");
    new_line;
    case ans is
      when '0' => Show_all_Derivatives(lp.all);
      when '1' => Enumerate_all_Derivatives(lp.all);
      when '2' => Interactive_Derivatives(lp.all);
      when '3' => Enumerate_all_Jacobian_Matrices(lp.all);
      when '4' => Interactive_Jacobians(lp.all);
      when others => null;
    end case;
  end Main;

begin
  Main;
end ts_dertrees;
