with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with File_Scanning;                     use File_Scanning;
with Standard_Floating_Numbers;         use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;      use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;          use Standard_Complex_Numbers;
with Multprec_Floating_Numbers;         use Multprec_Floating_Numbers;
with Multprec_Floating_Numbers_io;      use Multprec_Floating_Numbers_io;
with Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;       use Standard_Complex_Vectors_io;
with Standard_Complex_Norms_Equals;     use Standard_Complex_Norms_Equals;
with Multprec_Complex_Vectors;
with Multprec_Complex_Vector_Tools;     use Multprec_Complex_Vector_Tools;
with Standard_Integer_Matrices;
with Standard_Integer_Matrices_io;      use Standard_Integer_Matrices_io;
with Standard_Complex_Matrices;
with Multprec_Complex_Matrices;
with Symbol_Table;
with Standard_Complex_Polynomials;      use Standard_Complex_Polynomials;
with Standard_Complex_Poly_Functions;   use Standard_Complex_Poly_Functions;
with Standard_Complex_Poly_Systems; 
with Standard_Complex_Poly_Systems_io;  use Standard_Complex_Poly_Systems_io;
with Standard_Complex_Poly_SysFun;
with Standard_Complex_Jaco_Matrices;
with Standard_to_Multprec_Convertors;   use Standard_to_Multprec_Convertors;
with Multprec_Complex_Poly_Systems;
with Multprec_Complex_Poly_SysFun;
with Multprec_Complex_Jaco_Matrices;
with Standard_Complex_Solutions;
with Standard_Complex_Solutions_io;     use Standard_Complex_Solutions_io;
with Multprec_Complex_Solutions;
with Multprec_Complex_Solutions_io;     use Multprec_Complex_Solutions_io;
with Standard_Newton_Steps;
with Multprec_Newton_Steps;

procedure ts_newton is

-- DESCRIPTION :
--   Interactive testing of Newton with SVD.

  function Classify ( jm : Standard_Complex_Jaco_Matrices.Jaco_Mat;
                      ejm : Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                      z0,z1 : Standard_Complex_Vectors.Vector;
                      gamma : double_float )
                    return Standard_Integer_Matrices.Matrix is

  -- DESCRIPTION :
  --   Classifies the elements in the Jacobian matrix.
  --   The (i,j)-th entry of the matrix on return is
  --     -1 for a numerical zero element;
  --      0 for an algebraic zero element; and
  --     +1 for a numerical or algebraic nonzero element.

  -- ON ENTRY :
  --   jm        matrix with all partial derivatives of a system;
  --   ejm       evaluable form of the Jacobian matrix;
  --   z0        k-th iteration of Newton's method (for some k);
  --   z1        (k+1)-th iteration of Newton's method;
  --   gamma     constant to measure convergence rate.

    res : Standard_Integer_Matrices.Matrix(jm'range(1),jm'range(2));

  begin
    for i in jm'range(1) loop
      for j in jm'range(2) loop
        if jm(i,j) = Null_Poly then
          res(i,j) := 0;
        elsif Degree(jm(i,j)) = 0 then
          res(i,j) := 1;
        else declare
               y0 : Complex_Number := Eval(ejm(i,j),z0);
               y1 : Complex_Number := Eval(ejm(i,j),z1);
             begin
               if AbsVal(y1)/AbsVal(y0) <= gamma + 1.0E-4
                then res(i,j) := -1;
                else res(i,j) := 1;
               end if;
             end;
        end if;
      end loop;
    end loop;
    return res;
  end Classify;

  procedure Call_Classify
              ( jm : Standard_Complex_Jaco_Matrices.Jaco_Mat;
                ejm : Standard_Complex_Jaco_Matrices.Eval_Jaco_Mat;
                z0,z1 : Standard_Complex_Vectors.Vector;
                gamma : double_float ) is

  -- DESCRIPTION :
  --   Calls the classify routine with the given parameters
  --   and writes the matrix to screen.

    clasjm : Standard_Integer_Matrices.Matrix(jm'range(1),jm'range(2))
           := Classify(jm,ejm,z0,z1,gamma);

  begin
    put_line("The classified Jacobian elements :"); put(clasjm);
  end Call_Classify;

  function Compute_Gamma
              ( z0,z1,z2 : Standard_Complex_Vectors.Vector )
              return double_float is

  -- DESCRIPTION :
  --   Let z0, z1, and z2 be points generated by Newton's method,
  --   in a converging sequence to the root, then this function
  --   returns ||z2-z1||/||z1-z0||.

    use Standard_Complex_Vectors;

  begin
    return Max_Norm(z2-z1)/Max_Norm(z1-z0);
  end Compute_Gamma;

  procedure Call_Standard_Newton
              ( file : in file_type;
                p : in Standard_Complex_Poly_Systems.Poly_Sys;
                z : in out Standard_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Calls Newton's method to find a better approximation of a root
  --   of p, starting at the vector z.

    use Standard_Complex_Vectors;
    use Standard_Complex_Matrices;
    use Standard_Complex_Poly_SysFun;
    use Standard_Complex_Jaco_Matrices;
    use Standard_Complex_Solutions;

    ep : Eval_Poly_Sys(p'range) := Create(p);
    jm : Jaco_Mat(p'range,z'range) := Create(p);
    ejm : Eval_Jaco_Mat(jm'range(1),jm'range(2)) := Create(jm);
    ans : character;
    err,rco,res : double_float;
    tol : constant double_float := 1.0E-8;
    rank : natural;
    cnt : natural := 0;
    z0,z1 : Standard_Complex_Vectors.Vector(z'range) := z;

    function f ( x : Vector ) return Vector is
    begin
      return Eval(ep,x);
    end f;

    function jmf ( x : Vector ) return Matrix is

      res : Matrix(jm'range(1),jm'range(2)) := Eval(ejm,x);

    begin
      return res;
    end jmf;

    procedure Newton is
      new Standard_Newton_Steps.Reporting_Newton_Step(f,jmf);

  begin
    loop
      cnt := cnt + 1;
      z0 := z1;
      z1 := z;
      Newton(file,p'last,z,tol,err,rco,res,rank);
      put("The new approximation at step ");
      put(cnt,1); put_line(" : "); 
      declare
        s : Solution(z'length);
      begin
        s.v := z;
        put_vector(s);
      end;
      put("error :"); put(err,3);
      put("  rcond :"); put(rco,3);
      put("  resid :"); put(res,3); new_line;
      put("The numerical rank : "); put(rank,1); new_line;
     -- if cnt > 1
     --  then gamma := Compute_Gamma(z0,z1,z);
     --       put("gamma constant :"); put(gamma);
     --       if gamma > 1.0
     --        then put_line(" > 1.0, classification cannot be trusted!");
     --        else put_line(" <= 1.0, classification may be okay.");
     --       end if;
     --       Call_Classify(jm,ejm,z1,z,gamma);
     -- end if;
      put("Do you want another iteration ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
    end loop;
  end Call_Standard_Newton;

  procedure Call_Multprec_Newton
              ( file : in file_type; -- size : in natural;
                p : in Multprec_Complex_Poly_Systems.Poly_Sys;
                z : in out Multprec_Complex_Vectors.Vector ) is

  -- DESCRIPTION :
  --   Calls Newton's method to find a better approximation of a root
  --   of p, starting at the vector z.

  -- NOTE :
  --   One could think that doubling the working precision at each
  --   Newton step is a good idea, but it makes the execution time
  --   to grow exponentially as well, so it is commented out.
  --   The iterations happend in fixed precision.

    use Multprec_Complex_Vectors;
    use Multprec_Complex_Matrices;
    use Multprec_Complex_Poly_SysFun;
    use Multprec_Complex_Jaco_Matrices;
    use Multprec_Complex_Solutions;

    ep : Eval_Poly_Sys(p'range) := Create(p);
    jm : Jaco_Mat(p'range,z'range) := Create(p);
    ejm : Eval_Jaco_Mat(jm'range(1),jm'range(2)) := Create(jm);
    ans : character;
    tol : constant double_float := 1.0E-8;
    err,rco,res : Floating_Number;
    rank : natural;
    cnt : natural := 0;
   -- sz : natural := size;

    function f ( x : Vector ) return Vector is
    begin
      return Eval(ep,x);
    end f;

    function jmf ( x : Vector ) return Matrix is

      res : Matrix(jm'range(1),jm'range(2)) := Eval(ejm,x);

    begin
      return res;
    end jmf;

    procedure Newton is
      new Multprec_Newton_Steps.Reporting_Newton_Step(f,jmf);

  begin
    loop
      cnt := cnt + 1;
      Newton(file,p'last,z,tol,err,rco,res,rank);
      put("The new approximation at step ");
      put(cnt,1); put_line(" : "); 
      declare
        s : Solution(z'length);
      begin
        s.v := z;
        put_vector(s);
      end;
      put("error :"); put(err,3);
      put("  rcond :"); put(rco,3);
      put("  resid :"); put(res,3); new_line;
      put("The numerical rank : "); put(rank,1); new_line;
      put("Do you want another iteration ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
     -- sz := sz*2;    
     -- Set_Size(z,sz);
    end loop;
  end Call_Multprec_Newton;

  procedure Main is

    use Standard_Complex_Poly_Systems;
    use Standard_Complex_Solutions;

    lp : Link_to_Poly_Sys;
    infile : file_type;
    ans : character;
    found : boolean;
    sols : Solution_List;
    n,dgts : natural;

  begin
    new_line;
    put_line("Testing Newton's method ...");
    new_line;
    put("Is the polynomial system on file ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then new_line;
          put_line("Reading the name of the file for the system.");
          Read_Name_and_Open_File(infile);
          get(infile,lp);
          n := Number_of_Unknowns(lp(lp'first));
          Scan_and_Skip(infile,"SOLUTIONS",found);
          if found
           then get(infile,sols);
          end if;
     else new_line;
          put("Give the dimension of the system : "); get(n);
          Symbol_Table.Init(n);
          lp := new Poly_Sys(1..n);
          put("Give "); put(n,1); put_line(" polynomials :");
          get(lp.all); skip_line;
          found := false;
    end if;
    new_line;
    put("Give the number of digits (<16 is hardware float) : ");
    get(dgts);
   -- put("Do you want multiprecision arithmethic ? (y/n) ");
   -- Ask_Yes_or_No(ans);
   -- if ans = 'y'
   --  then dgts := 32;
   --  else dgts := 0;
   -- end if;
    new_line;
    declare
      z : Standard_Complex_Vectors.Vector(1..n);
    begin
      if found
       then z := Head_Of(sols).v;
       else put("Give "); put(n,1);
            put_line(" complex numbers to start at:");
            get(z);
      end if;
      if dgts < 16
       then Call_Standard_Newton(Standard_Output,lp.all,z);
       else declare
              mpz : Multprec_Complex_Vectors.Vector(1..n) := Create(z);
              mps : Multprec_Complex_Poly_Systems.Poly_Sys(lp'range)
                  := Convert(lp.all);
              size : constant natural := Decimal_to_Size(dgts);
            begin
              Set_Size(mps,size);
              Set_Size(mpz,size);
              Call_Multprec_Newton(Standard_Output,mps,mpz);
            end;
      end if;
    end;
  end Main;

begin
  Main;
end ts_newton;
