with integer_io;                       use integer_io;
with Timing_Package;                   use Timing_Package;
with Standard_Floating_Numbers;        use Standard_Floating_Numbers;
with Standard_Floating_Numbers_io;     use Standard_Floating_Numbers_io;
with Standard_Complex_Numbers;         use Standard_Complex_Numbers;
with Standard_Complex_Numbers_io;      use Standard_Complex_Numbers_io;
with Standard_Complex_Vectors;
with Standard_Complex_Vectors_io;      use Standard_Complex_Vectors_io;
with Standard_Random_Vectors;          use Standard_Random_Vectors;
with Standard_Complex_Polynomials_io;  use Standard_Complex_Polynomials_io;
with Standard_Durand_Kerner;           use Standard_Durand_Kerner;

procedure Black_Box_Durand_Kerner ( file : in file_type; p : in Poly ) is

  d : constant natural := Degree(p);

  procedure Write_Results
               ( step : in natural;
                 z,res : in Standard_Complex_Vectors.Vector ) is

    absres : double_float;

  begin
    new_line(file);
    put(file,"Results after "); put(file,step,1);
    put_line(file," iterations :");
    put_line(file,
    "------------------------------------------------------------------------");
    put_line(file,
    "| APPROXIMATED ROOTS (real and imaginary part) |     RESIDUALS         |");
    put_line(file,
    "------------------------------------------------------------------------");
    for i in z'range loop
      absres := AbsVal(res(i));
      put(file,"| "); put(file,z(i)); put(file," | "); 
      put(file,absres); put(file," |");
      new_line(file);
    end loop;
    put_line(file,
    "------------------------------------------------------------------------");
  end Write_Results;

  procedure Find_Roots ( cp : in Standard_Complex_Vectors.Vector ) is

    max : constant natural := 10*d;
    eps : constant double_float := 1.0E-13;
    z,res : Standard_Complex_Vectors.Vector(1..d);
    nb : natural;
    fail : boolean;

  begin
    z := Random_Vector(1,d);
    Silent_Durand_Kerner(cp,z,res,max,eps,nb,fail);
    if fail
     then put_line(file,"precision insufficient to reach results");
     else Write_Results(nb,z,res);
    end if;
  end Find_Roots;

  procedure Call_Durand_Kerner is

    timer : Timing_Widget;
    cff : Standard_Complex_Vectors.Vector(0..d) := (0..d => Create(0.0));

    procedure Scan_Coefficient ( t : in Term; cont : out boolean ) is
    begin
      cff(t.dg(1)) := t.cf;
      cont := true;
    end Scan_Coefficient;
    procedure Scan_Coefficients is new Visiting_Iterator(Scan_Coefficient);

  begin
    tstart(timer);
    Scan_Coefficients(p);
    new_line(file);
    put_line(file,"The coefficient vector :");
    put_line(file,cff);
    for i in 0..d-1 loop
      cff(i) := cff(i)/cff(d);
    end loop;
    Find_Roots(cff);
    tstop(timer);
    new_line(file);
    print_times(file,timer,"application of the method of Weierstrass");
  end Call_Durand_Kerner;

begin
  put_line(file,"1 1");
  put(file,p); new_line(file);
  if d = 0
   then new_line(file);
        put_line(file,"There are no roots to compute.");
        new_line(file);
   else Call_Durand_Kerner;
  end if;
end Black_Box_Durand_Kerner;
