with Standard_Floating_Numbers;          use Standard_Floating_Numbers;
with Standard_Complex_Laur_Systems;      use Standard_Complex_Laur_Systems;
with Standard_Poly_Laur_Convertors;      use Standard_Poly_Laur_Convertors;
with Standard_Root_Refiners;             use Standard_Root_Refiners;
with Transforming_Laurent_Systems;       use Transforming_Laurent_Systems;
with Fewnomial_System_Solvers;           use Fewnomial_System_Solvers;

procedure Black_Box_Fewnomial_Solver
            ( file : in file_type; p : in Poly_Sys;
              sols : out Solution_List; fail : out boolean ) is

  procedure Call_Root_Refiner
              ( file : in file_type; p_sys : in Poly_Sys;
                p_sols : in out Solution_List ) is

    epsxa,epsfa,tolsing : double_float;
    numit : natural;
    max : constant natural := 5;

  begin
    epsxa := 1.0E-14;
    epsfa := 1.0E-14;
    tolsing := 1.0E-08;
    Reporting_Root_Refiner
      (file,p_sys,p_sols,epsxa,epsfa,tolsing,numit,max,false,false);
  end Call_Root_Refiner;

  procedure Main is

    l : Laur_Sys(p'range) := Polynomial_to_Laurent_System(p);
    result : Solution_List;

  begin
    Shift(l);
    if not Is_Fewnomial_System(l) then
      fail := true;
    else
      Solve(l,result,fail);
      if not fail then
        if Is_Null(result) then
          new_line(file);
          put_line(file,"No isolated solutions found.  Rank deficiency of the");
          put_line(file,"exponent matrix may allow sets of solutions.");
        else
          Call_Root_Refiner(file,p,result);
        end if;
      end if;
      sols := result;
    end if;
    Clear(l);
  end Main;

begin
  Main;
end Black_Box_Fewnomial_Solver;
