with Standard_Complex_Numbers;           use Standard_Complex_Numbers;
with Standard_Complex_Norms_Equals;      use Standard_Complex_Norms_Equals;

package body Standard_Durand_Kerner is

-- AUXILIARIES :

  function Horner ( p : Vector; x : Complex_Number ) return Complex_Number is

  -- DESCRIPTION :
  --   Returns (..((a[n]*x + a[n-1])*x + a[n-2])*x + .. + a[1])*x + a[0].

    res : Complex_Number;

  begin
    if p'first = p'last
     then res := p(p'first);
     else res := p(p'last);
          for i in reverse p'first..(p'last-1) loop
            res := res*x + p(i);
          end loop;
    end if;
    return res;
  end Horner;

  procedure DK ( p : in Vector; z,res : in out Vector ) is

  -- DESCRIPTION :
  --   Computes one step in the Durand-Kerner iteration.

    function Compute_q ( i : integer; a : Vector ) return Complex_Number is

    -- DESCRIPTION :
    --   Computes the quotient needed in the Durand-Kerner step.

      res : Complex_Number;

    begin
      res := Create(1.0);
      for j in a'range loop
        if j /= i
         then res := res*(a(i)-a(j));
        end if;
      end loop;
      return res;
    end Compute_q;

  begin
    for i in z'range loop
      z(i) := z(i) - Horner(p,z(i))/Compute_q(i,z);
      res(i) := Horner(p,z(i));
    end loop;
  end DK;

-- TARGET ROUTINES :

  procedure Reporting_Durand_Kerner
              ( p : in Vector; z,res : in out Vector;
                maxsteps : in natural; eps : in double_float;
                nb : out natural; fail : out boolean ) is

    pp : Vector(p'range);

  begin
    if p(p'last) /= Create(1.0)
     then for i in p'range loop
            pp(i) := p(i) / p(p'last);
          end loop;
     else for i in p'range loop
            pp(i) := p(i);
          end loop;
    end if;
    fail := true;
    for k in 1..maxsteps loop
      nb := k;
      DK(pp,z,res);  
      Write(k,z,res);
      if Max_Norm(res) <= eps
       then fail := false;
            exit;
      end if;
    end loop;
  exception
    when others => fail := true;
  end Reporting_Durand_Kerner;

  procedure Silent_Durand_Kerner
              ( p : in Vector; z,res : in out Vector;
                maxsteps : in natural; eps : in double_float;
                nb : out natural; fail : out boolean ) is

    pp : Vector(p'range);

  begin
    if p(p'last) /= Create(1.0)
     then for i in p'range loop
            pp(i) := p(i) / p(p'last);
          end loop;
     else for i in p'range loop
            pp(i) := p(i);
          end loop;
    end if;
    fail := true;
    for k in 1..maxsteps loop
      nb := k;
      DK(pp,z,res);  
      if Max_Norm(res) <= eps
       then fail := false;
            exit;
      end if;
    end loop;
  exception
    when others => fail := true;
  end Silent_Durand_Kerner;

end Standard_Durand_Kerner;
