with text_io,integer_io;                use text_io,integer_io;

package body Checker_Moves is

-- PART I : Moves of the Black Checkers

  function Identity_Permutation ( n : natural ) return Vector is

    res : Vector(1..n);
 
  begin
    for i in 1..n loop
      res(i) := i;
    end loop;
    return res;
  end Identity_Permutation;

  function Descending_Checker ( p : Vector ) return natural is
  begin
    for i in reverse p'first..p'last-1 loop
      if p(i+1) > p(i)
       then return i;
      end if;
    end loop;
    return 0;
  end Descending_Checker;

  function Rising_Checker ( p : Vector; i : natural ) return natural is
  begin
    if i /= 0
     then for j in i+1..p'last loop
            if p(j) = p(i)+1
             then return j;
            end if;
          end loop;
    end if;
    return 0;
  end Rising_Checker;

  procedure Move ( p : in out Vector; down,up : in natural ) is
  begin
    p(down) := p(down) + 1;
    p(up) := p(up) - 1;
  end Move;

  function Number_of_Moves ( n : natural ) return natural is

    res : natural := (n*n - n)/2 + 1;

  begin
    return res;
  end Number_of_Moves;

-- PART II : Moves of the White Checkers

  function Critical_Row 
              ( r,c : natural; rows,cols : Vector ) return natural is
  begin
    for i in rows'range loop
      if (rows(i) = r) and (cols(cols'last-i+1) >= c)
       then return i;
      end if; 
    end loop;
    return 0;
  end Critical_Row;

  function In_Critical_Diagonal 
              ( dr,dc,n,row,column : natural ) return boolean is

    k : natural := 0;

  begin
    while ((dr + k <= n) and (dc + k <= n)) loop
      if ((dr + k = row) and (dc + k = column))
       then return true;
       else k := k+1;
      end if;
    end loop;
    return false;
  end In_Critical_Diagonal;

  function Top_White_Checker
              ( dr,dc,n : natural; rows,cols : Vector ) return natural is

    res,c : natural;

  begin
    res := 0;
    for i in rows'range loop
      c := cols(cols'last-i+1);
      if In_Critical_Diagonal(dr,dc,n,rows(i),c)
       then if res = 0
             then res := i;
             elsif rows(i) < rows(res)
                 then res := i;
            end if;
      end if;
    end loop;
    return res;
  end Top_White_Checker;

  function Between ( a,b,x : natural ) return boolean is
  begin
    if a <= b
     then return (x >= a) and (x <= b);
     else return (x >= b) and (x <= a);
    end if;
  end Between;

  function Blocker ( rows,cols : Vector; t,r : natural ) return boolean is
  begin
    for i in rows'range loop
       if i /= r and i /= t
       then if Between(rows(t),rows(r),rows(i)) 
              and Between(cols(cols'last-t+1),
                          cols(cols'last-r+1),cols(cols'last-i+1))
             then return true;
            end if;
      end if;
    end loop;
    return false;
  end Blocker;

  function Central_Choice
              ( p : Vector; d,r : natural;
                rows,cols : Vector; cr,cd : natural; debug : boolean ) 
              return natural is
  begin
    if debug
     then put("Descending black checker at (");
          put(p(d),1); put(","); put(p'last-d+1,1); put_line(").");
          put("White checker in critical row at (");
          put(rows(cr),1); put(",");
          put(cols(cols'last-cr+1),1); put_line(").");
    end if;
   -- is white checker in critical row in descending checker's square?
    if p(d) = rows(cr) and p'last-d+1 = cols(cols'last-cr+1)
     then if debug
           then put_line("No choice because in descending's checker's square.");
          end if;
          return 1;   -- we must swap
     else if debug 
           then put_line("White checker not in descending's checker's square.");
          end if;
    end if;
    if debug
     then put("Rising black checker at (");
          put(p(r),1); put(","); put(p'last-r+1,1); put_line(").");
          put("Top white checker in critical diagonal at (");
          put(rows(cd),1); put(",");
          put(cols(cols'last-cd+1),1); put_line(").");
    end if;
   -- is top white checker in critical row in rising checker's square?
    if p(r) = rows(cd) and p'last-r+1 = cols(cols'last-cd+1)
     then if debug
           then put_line("No choice because in rising's checker's square.");
          end if;
          return 1;   -- we must swap
     else if debug
           then put_line("White checker not in rising's checker's square.");
          end if;
    end if;
   -- is there a blocker?
    if Blocker(rows,cols,cr,cd)
     then if debug
           then put_line("No choice because of a blocker.");
          end if;
          return 0;   -- we must stay
     else if debug
           then put_line("There is not a blocker, so there is choice.");
          end if;
          return 2;
    end if;
  end Central_Choice;

  procedure Swap ( rows : in out Vector; i,j : in natural ) is

    tmp : natural;

  begin
    if i /= 0
     then tmp := rows(j);
          rows(j) := rows(i);
          rows(i) := tmp;
    end if;
  end Swap;

  function Happy_in_Row ( p : Vector; r,c : natural ) return boolean is
  begin
    for k in p'range loop 
      if p(k) = r                 -- black checker in row r ?
       then if p'last+1-k <= c    -- and column <= c ?
             then return true;
            end if;
      end if;
    end loop;
    return false;
  end Happy_in_Row;

  function Happy_in_Column ( p : Vector; r,c : natural ) return boolean is

    k : constant natural := p'last+1-c;

  begin
    return (p(k) <= r);
  end Happy_in_Column;

  procedure Make_Happy ( p : in Vector; rows,cols : in out Vector ) is

    r,c : natural;

  begin
    for i in rows'range loop
      r := rows(i);
      c := cols(cols'last+1-i);
     -- put("White checker at (");
     -- put(r,1); put(",");
     -- put(c,1); put(")");
      if not Happy_in_Row(p,r,c)
       then -- put_line(" is not happy in its row.");
            loop
              r := r - 1;
              exit when Happy_in_Row(p,r,c);
            end loop;
            rows(i) := r;
       -- else put_line(" is happy in its row.");
      end if;
     -- put("White checker at (");
     -- put(r,1); put(",");
     -- put(c,1); put(")");
      if not Happy_in_Column(p,r,c)
       then -- put_line(" is not happy in its column.");
            loop
              c := c - 1;
              exit when Happy_in_Column(p,r,c);
            end loop;
            cols(cols'last+1-i) := c;
       -- else put_line(" is happy in its column.");
      end if;
    end loop;
  end Make_Happy;

end Checker_Moves;
