with Checker_Moves;                     use Checker_Moves;

package body Checker_Posets is

  function Generate_Moves ( n : natural ) return VecVec is

    res : VecVec(1..Number_of_Moves(n));
    mvp : Vector(1..n) := Identity_Permutation(n);
    d,r : natural;

  begin
    res(1) := new Vector'(mvp);
    for i in 2..res'last loop
      d := Descending_Checker(mvp);
      r := Rising_Checker(mvp,d);
      Move(mvp,d,r);
      res(i) := new Vector'(mvp);
    end loop;
    return res;
  end Generate_Moves;

  function Create ( r,c : Vector ) return Node is

    res : Node(r'last-r'first+1);

  begin
    res.coeff := 1;
    res.rows := r;
    res.cols := c;
    res.stay_child := null;
    res.swap_child := null;
    res.next_sibling := null;
    return res;
  end Create;

  function Equal ( nd1,nd2 : Node ) return boolean is
  begin
    if not Equal(nd1.rows,nd2.rows)
     then return false;
     else if not Equal(nd1.cols,nd2.cols)
           then return false;
           else return true;
          end if;
    end if;
  end Equal;

  function Copy_Child ( nd : Node ) return Node is

  -- DESCRIPTION :
  --   Returns a copy from the node (except for the links!).

    res : Node(nd.k);

  begin
    res.coeff := nd.coeff;
    res.rows := nd.rows;
    res.cols := nd.cols;
    res.stay_child := null;
    res.swap_child := null;
    res.next_sibling := null;
    return res;
  end Copy_Child;

  function Stay_Child ( nd : Node; np : Vector ) return Node is

  -- DESCRIPTION :
  --   Returns a copy from the node (except for the links!)
  --   after making it happy according to the next location
  --   of the black checkers in np.

    res : Node(nd.k) := Copy_Child(nd);

  begin
    Make_Happy(np,res.rows,res.cols);
    return res;
  end Stay_Child;

  function Swap_Child ( nd : Node; np : Vector; cr,cd : natural )
                      return Node is

  -- DESCRIPTION :
  --   Returns the node obtained from swapping rows and after
  --   making it happy according to the next location of the
  --   black checkers in np.

    res : Node(nd.k) := Copy_Child(nd);

  begin
    Swap(res.rows,cr,cd);
    Make_Happy(np,res.rows,res.cols);
    return res;
  end Swap_Child;

  procedure Update_Children ( p : in out Poset; level : in natural;
                              nd : in Node; lnd : out Link_to_Node ) is

    tmp : Link_to_Node := p.white(level);

  begin
    if level <= p.white'last
     then declare
            tmp : Link_to_Node := p.white(level);
            previous : Link_to_Node;
          begin
            while tmp /= null loop
              if Equal(tmp.all,nd)
               then tmp.coeff := tmp.coeff + nd.coeff;
                    lnd := tmp;
                    return;
               else previous := tmp;
                    tmp := tmp.next_sibling;
              end if;
            end loop;
            lnd := new Node'(nd);
            if p.white(level) = null
             then p.white(level) := lnd;
             else previous.next_sibling := lnd;
            end if;
          end;
     else lnd := null;
    end if;
  end Update_Children;

  procedure White_Moves ( p : in out Poset; level,n,d,r : in natural;
                          pb : in Vector; lnd : in out Link_to_Node ) is

    cr : constant natural := Critical_Row(pb(d),n-d+1,lnd.rows,lnd.cols);
    cd : natural;
    swap,stay : boolean;

  begin
    if cr = 0
     then stay := true;
          swap := false;
     else cd := Top_White_Checker(pb(r),n-r+1,n,lnd.rows,lnd.cols);
          if cd = 0
           then stay := true;
                swap := false;
           else case Central_Choice(pb,d,r,lnd.rows,lnd.cols,cr,cd,true) is
                  when 0 => stay := true; swap := false;
                  when 1 => stay := false; swap := true;
                  when others => stay := true; swap := true;
                end case;
          end if;
    end if;
    if stay 
     then declare
            stay_nd : Node(lnd.k)
                    := Stay_Child(lnd.all,p.black(level+1).all);
            lk_stay : Link_to_Node;
          begin
            Update_Children(p,level+1,stay_nd,lk_stay); 
            lnd.stay_child := lk_stay;
          end;
    end if;
    if swap
     then declare
            swap_nd : Node(lnd.k)
                    := Swap_Child(lnd.all,p.black(level+1).all,cr,cd);
            lk_swap : Link_to_Node;
          begin
            Update_Children(p,level+1,swap_nd,lk_swap); 
            lnd.swap_child := lk_swap;
          end;
    end if;
  end White_Moves;

  procedure Black_Moves ( p : in out Poset; n,level : in natural ) is

    pb : constant Vector := p.black(level).all;
    d : constant natural := Descending_Checker(pb);
    r : constant natural := Rising_Checker(pb,d);

  begin
    if r /= 0
     then declare
            lnd : Link_to_Node := p.white(level);
          begin
            while lnd /= null loop
              White_Moves(p,level,n,d,r,pb,lnd);
              lnd := lnd.next_sibling;
            end loop;
          end;
    end if;
  end Black_Moves;

  function Create ( n : natural; r,c : Vector ) return Poset is

    res : Poset;
    k : constant natural := r'last-r'first+1;
    root : Node(k) := Create(r,c);

  begin
    res.black := new VecVec'(Generate_Moves(n));
    res.white := new Array_of_Nodes(res.black'range);
    res.white(1) := new Node'(root);
    for i in res.white'first+1..res.white'last loop
      res.white(i) := null;
    end loop;
    for i in 1..res.black'last-1 loop
      Black_Moves(res,n,i);
    end loop;
    return res;
  end Create;

-- DESTRUCTORS :

  procedure Clear ( lnd : in out Link_to_Node ) is
  begin
    null;
  end Clear;

  procedure Clear ( arrlnd : in out Array_of_Nodes ) is
  begin
    null;
  end Clear;

  procedure Clear ( arrlnd : in out Link_to_Array_of_Nodes ) is
  begin
    null;
  end Clear;

  procedure Clear ( p : in out Poset ) is
  begin
    null;
  end Clear;

end Checker_Posets;
