with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Natural_VecVecs;          use Standard_Natural_VecVecs;

package Checker_Posets is

-- DESCRIPTION :
--   Littlewood-Richardson coefficients count "checker games".

-- DATA STRUCTURES :

  type Node;
  type Link_to_Node is access Node;

  type Node ( k : natural ) is record
    coeff : natural;                    -- Littlewood-Richardson coefficient
    rows : Vector(1..k);                -- row positions on board
    cols : Vector(1..k);                -- column positions
    stay_child : Link_to_Node;          -- child with same location
    swap_child : Link_to_Node;          -- child with swapped rows
    next_sibling : Link_to_Node;        -- next sibling at same level
  end record;

  type Array_of_Nodes is array ( integer range <> ) of Link_to_Node;
  type Link_to_Array_of_Nodes is access Array_of_Nodes;

  type Poset is record
    black : Link_to_VecVec;             -- location of black checkers
    white : Link_to_Array_of_Nodes;     -- location of white checkers
  end record;

-- OPERATIONS :

  function Generate_Moves ( n : natural ) return VecVec;

  -- DESCRIPTION :
  --   Returns a vector describing the moves of the black checkers,
  --   of range 1..Checker_Moves.Number_of_Moves(n),
  --   starting with the identity permutation
  --   and ending at its reverse permutation.

  function Create ( r,c : Vector ) return Node;

  -- DESCRIPTION :
  --   Creates the node with rows and columns in r and c respectively. 

  function Create ( n : natural; r,c : Vector ) return Poset;

  -- DESCRIPTION :
  --   Creates the poset for a board of n checkers,
  --   with the black checkers on the diagonal and white checkers
  --   in rows r and columns indicated by c.

-- DESTRUCTORS :

  procedure Clear ( lnd : in out Link_to_Node );
  procedure Clear ( arrlnd : in out Array_of_Nodes );
  procedure Clear ( arrlnd : in out Link_to_Array_of_Nodes );
  procedure Clear ( p : in out Poset );

  -- DESCRIPTION :
  --   Deallocation of memory.

end Checker_Posets;
