with text_io,integer_io;                use text_io,integer_io;
with Communications_with_User;          use Communications_with_User;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Natural_Vectors_io;       use Standard_Natural_Vectors_io;
with Standard_Natural_VecVecs;          use Standard_Natural_VecVecs;
with Standard_Natural_VecVecs_io;       use Standard_Natural_VecVecs_io;
with Standard_Natural_Matrices;         use Standard_Natural_Matrices;
with Standard_Natural_Matrices_io;      use Standard_Natural_Matrices_io;
with Checker_Boards;                    use Checker_Boards;
with Checker_Boards_io;                 use Checker_Boards_io;
with Checker_Moves;                     use Checker_Moves;
with Checker_Posets;                    use Checker_Posets;

procedure ts_checkers is

-- DESCRIPTION :
--   Simple test on the operations in Checker_Posets.

-- CREATION OF LOCALIZATION PATTERN :

  procedure Sort_Rows ( r,c : in out Vector ) is

  -- DESCRIPTION :
  --   Sorts the position of the white checkers so that the
  --   vector r is sorted.

    min,tmp : natural;

  begin
    for i in r'first..r'last-1 loop
      min := i;
      for j in i+1..r'last loop
        if r(j) < r(min)
         then min := j;
        end if;
      end loop;
      if min /= i
       then tmp := r(min);
            r(min) := r(i);
            r(i) := tmp;
            tmp := c(c'last+1-min);
            c(c'last+1-min) := c(c'last+1-i);
            c(c'last+1-i) := tmp;
      end if;
    end loop;
  end Sort_Rows;

  function Localization_Pattern ( p,r,c : Vector ) return Matrix is

  -- DESCRIPTION :
  --   Returns a localization pattern for a k-plane in n-space.

    res : Matrix(r'range,p'range);
    sr : Vector(r'range) := r;
    sc : Vector(c'range) := c;
    col : natural;

  begin
    for i in res'range(1) loop
      for j in res'range(2) loop
        res(i,j) := 0;
      end loop;
    end loop;
    Sort_Rows(sr,sc);
   -- put("["); put(r); put("]");
   -- put("["); put(c); put("]");
   -- put(" -> ");
   -- put("["); put(sr); put("]");
   -- put("["); put(sc); put("]");
   -- new_line;
    for i in sr'range loop  -- white at (sr(i),sc(sc'last+1-i))
      res(i,sr(i)) := 1;
      for j in p'range loop -- black at (p(j),p'last+1-j) 
        col := p'last+1-j;
        if (p(j) < sr(i)) and (col <= sc(sc'last+1-i))
         then res(i,p(j)) := 2;
              for k in res'first(1)..i-1 loop  -- eliminate if 1 above
                if res(k,p(j)) = 1
                 then res(i,p(j)) := 0;
		      for j2 in res'first(2)..p(j)-1 loop
                        if res(k,j2) = 2
                         then res(i,j2) := 2;
                        end if;
                      end loop;
                      exit;
                end if;
              end loop;
        end if;
      end loop;
    end loop;
    return res;
  end Localization_Pattern;

-- TARGET TEST ROUTINES :

  procedure Interactive_Test ( n : in natural ) is

    p : Vector(1..n);
    ans : character;

  begin
    loop
      Read_Permutation(p);
      Write_Permutation(p);
      put("index of descending checker : ");
      put(Descending_Checker(p),1); new_line;
      put("index of rising checker : ");
      put(Rising_Checker(p,Descending_Checker(p)),1); new_line;
      put("Want another permutation ? (y/n) ");
      get(ans);
      exit when (ans /= 'y');
    end loop;
  end Interactive_Test;

  procedure Specialization_Order ( n : in natural ) is

    p : Vector(1..n) := Identity_Permutation(n);
    ans : character;
    desc,rise,cnt : natural := 0;

  begin
    put("Number of moves : "); 
    put(Number_of_Moves(n),1); new_line;
    loop
      cnt := cnt + 1;
      put("permutation no ");
      put(cnt,1); put_line(" : ");
      Write_Permutation(p);
      put("index of descending checker : ");
      desc := Descending_Checker(p);
      put(desc,1); new_line;
      exit when (desc = 0);
      rise := Rising_Checker(p,desc);
      put("index of rising checker : ");
      put(rise,1); new_line;
      exit when (rise = 0);
      put("Do we make the move ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
      Move(p,desc,rise);
    end loop;
  end Specialization_Order;

  procedure Show_All_Moves ( n : in natural ) is

    all_moves : VecVec(1..Number_of_Moves(n)) := Generate_Moves(n);
    ans : character;

  begin
    put("The number of moves : ");
    put(Number_of_Moves(n),1); new_line;
    put(all_moves);
    put("Do you wish to see all the boards ? (y/n) ");
    Ask_Yes_or_No(ans);
    if ans = 'y'
     then for i in all_moves'range loop
            put("Move #"); put(i,1); put_line(":");
            --Write(Configuration(all_moves(i).all));
            Write_Permutation(all_moves(i).all);
          end loop;
    end if;
  end Show_All_Moves;

  procedure Test_White_Moves ( k,n : in natural ) is

    p : Vector(1..n);
    b : Board(1..n,1..n);
    rows,cols : Vector(1..k);
    r,d,cr,cd : natural;
    ans : character;
    debug : boolean := true;
    we_swap : boolean;

  begin
    put_line("Reading coordinates for black checkers...");
    Read_Permutation(p);
    b := Configuration(p);
    put_line("The board with black checkers :");
    Write_Permutation(p);
    put_line("Reading coordinates for white checkers...");
    Read_Permutation(rows);
    Read_Permutation(cols);
    Place_White(b,rows,cols);
    put_line("The board with black and white checkers :");
    Write(b,p,rows,cols);
    loop
      d := Descending_Checker(p);
      exit when (d = 0);
      we_swap := false;
      if debug
       then put("Descending black checker is in row ");
            put(p(d),1); put(" and column "); put(p'last-d+1,1);
            put_line(".");
      end if;
      r := Rising_Checker(p,d);
      exit when (r = 0);
      if debug
       then put("Rising black checker is in row ");
            put(p(r),1); put(" and column "); put(p'last-r+1,1);
            put_line(".");
            put("The critical row is "); put(p(d),1);
            put_line(".");
      end if;
      cr := Critical_Row(p(d),p'last-d+1,rows,cols);
      if debug then
        if cr = 0 then
          put_line("There is no white checker in the critical row.");
          put_line("The white checkers stay at their current position.");
        else
          put("White checker at row "); put(rows(cr),1);
          put(" and column "); put(cols(cols'last-cr+1),1);
          put_line(" is in the critical row.");
          put("Start of critical diagonal : ");
          put(p(r),1); put(" "); put(n-r+1,1); new_line;
        end if;
      end if;
      if cr /= 0 then
        cd := Top_White_Checker(p(r),n-r+1,n,rows,cols);
        if debug then
          if cd = 0
           then put_line("No white checker is in the critical diagonal.");
                cr := 0;  -- forces a stay
           else put("Top white checker is in row ");
                put(rows(cd),1); put(" and column ");
                put(cols(cols'last-cd+1),1); new_line;
          end if;
        end if;
      end if;
      if cr > 0 and cd > 0 then
        case Central_Choice(p,d,r,rows,cols,cr,cd,true) is
          when 1 => we_swap := true;
                    put_line("There is no choice, we swap.");
          when 2 => put("Type 0 to stay, or 1 to swap : ");
                    Ask_Alternative(ans,"01");
                    we_swap := (ans = '1');
          when others => put_line("There is no choice, we stay.");
        end case;
      end if;
      if we_swap
       then Swap(rows,cr,cd);
      end if;
      Move(p,d,r);
      if debug
       then b := Configuration(p,rows,cols);
            put_line("The board after moves before making happy :");
            Write(b,p,rows,cols);
      end if;
      Make_Happy(p,rows,cols);
      b := Configuration(p,rows,cols);
      put_line("The board after all moves of black and white :");
      Write(b,p,rows,cols);
      put("Continue with next move ? (y/n) ");
      Ask_Yes_or_No(ans);
      exit when (ans /= 'y');
    end loop;
  end Test_White_Moves;

  procedure Write ( ps : in Poset ) is

  -- DESCRIPTION :
  --   Writes the poset of black and white checkers.

  begin
    for i in ps.black'range loop
      put(ps.black(i)); put(" : ");
      declare
        tmp : Link_to_Node := ps.white(i);
      begin
        while tmp /= null loop
          put(" + "); put(tmp.coeff,1);
          put("(["); put(tmp.rows); put("]");
          put(",["); put(tmp.cols); put("])");
          tmp := tmp.next_sibling;
        end loop;
        new_line;
      end;
    end loop;
  end Write;

  procedure Write_Patterns ( ps : in Poset ) is

  -- DESCRIPTION :
  --   Writes the poset of black and white checkers,
  --   as well as the localization patterns.

  begin
    for i in ps.black'range loop
      put(ps.black(i)); put_line(" : ");
      declare
        tmp : Link_to_Node := ps.white(i);
      begin
        while tmp /= null loop
          put(" + "); put(tmp.coeff,1); put("*");
          put("(["); put(tmp.rows); put("]");
          put(",["); put(tmp.cols); put("])");
          put_line(" with pattern :");
          put(Localization_Pattern(ps.black(i).all,tmp.rows,tmp.cols));
          tmp := tmp.next_sibling;
        end loop;
      end;
    end loop;
  end Write_Patterns;

  procedure Create_Poset ( k,n : in natural ) is

    p : Vector(1..n) := Identity_Permutation(n);
    b : Board(1..n,1..n) := Configuration(p);
    rows,cols : Vector(1..k);
    ps : Poset;

  begin
    put_line("The board with black checkers :");
    Write_Permutation(p);
    put_line("Reading coordinates for white checkers...");
    Read_Permutation(rows);
    Read_Permutation(cols);
    Place_White(b,rows,cols);
    put_line("The board with black and white checkers :");
    Write(b,p,rows,cols);
    ps := Create(n,rows,cols);
    for i in ps.white'range loop
      put("Formal sum at level "); put(i,1); put_line(" :");
      declare
        tmp : Link_to_Node := ps.white(i);
      begin
        while tmp /= null loop
          put("+"); put(tmp.coeff,1);
          put("["); put(tmp.cols); put("]");
          tmp := tmp.next_sibling;
        end loop;
        new_line;
      end;
    end loop;
    put_line("The complete poset : ");
    Write(ps);
    put_line("The localization patterns in the complete poset : ");
    Write_Patterns(ps);
  end Create_Poset;

  procedure Main is

    n,k : natural;
    ans : character;

  begin
    new_line;
    put_line("Littlewood-Richardson rule by checker games...");
    new_line;
    put_line("Choose one of the following : ");
    put_line("  1. test calculation of rising and descending checker");
    put_line("  2. interactive generation of the specialization order");
    put_line("  3. show all permutations of moving black checkers");
    put_line("  4. test calculation of moves of white checkers");
    put_line("  5. create the poset of all black and white moves");
    put("Type 1, 2, 3, 4, or 5 to make a choice : ");
    Ask_Alternative(ans,"12345");
    new_line;
    put("Give n : "); get(n);
    case ans is
      when '1' => Interactive_Test(n);
      when '2' => Specialization_Order(n);
      when '3' => Show_All_Moves(n);
      when '4' => put("Give k : "); get(k);
                  Test_White_Moves(k,n);
      when '5' => put("Give k : "); get(k);
                  Create_Poset(k,n);
      when others => null;
    end case;
  end Main;

begin
  Main;
end;
