with text_io,integer_io;                use text_io,integer_io;
with Standard_Natural_Vectors;          use Standard_Natural_Vectors;
with Standard_Natural_Matrices;         use Standard_Natural_Matrices;
with Standard_Natural_Matrices_io;      use Standard_Natural_Matrices_io;
with Checker_Boards,Checker_Boards_io;  use Checker_Boards,Checker_Boards_io;
with Checker_Moves;                     use Checker_Moves;

procedure ts_induce is

  procedure Sort_Rows ( r,c : in out Vector ) is

  -- DESCRIPTION :
  --   Sorts the position of the white checkers so that the
  --   vector r is sorted.

    min,tmp : natural;

  begin
    for i in r'first..r'last-1 loop
      min := i;
      for j in i+1..r'last loop
        if r(j) < r(min)
         then min := j;
        end if;
      end loop;
      if min /= i
       then tmp := r(min);
            r(min) := r(i);
            r(i) := tmp;
            tmp := c(c'last+1-min);
            c(c'last+1-min) := c(c'last+1-i);
            c(c'last+1-i) := tmp;
      end if;
    end loop;
  end Sort_Rows;

  function Coordinate_Matrix
             ( n,k : natural; p,r,c : Vector ) return Matrix is

  -- DESCRIPTION :
  --   Returns the pattern for the n-by-k coordinate matrix for a k-plane
  --   in n-space whose shape is determined by the position of black and
  --   white checkers.

  -- ON ENTRY :
  --   n     dimension of the ambient space;
  --   k     dimension of the plane;
  --   p     permutation determines the position of the black checkers;
  --   r     rows of the white checkers;
  --   c     columns of the white checkers (start index at end of c).

    res : Matrix(1..n,1..k);
    sr : Vector(r'range) := r;
    sc : Vector(r'range) := c;
    row,col : natural;
    white_on_black : boolean;

  begin
    for i in 1..n loop      -- initialize to zero
      for j in 1..k loop
        res(i,j) := 0;
      end loop;
    end loop;
    Sort_Rows(sr,sc);
    for i in 1..k loop      -- put one in rows of white checker
      res(sr(i),i) := 1;
    end loop;
    for i in 1..n loop      -- put two in free places
      row := p(i);          -- row of i-th black checker
      col := n-i+1;         -- column of i-th black checker
      for j in 1..k loop    -- white checker at (r(j),c(k-j+1))
        if (sr(j) >= row) and (sc(k-j+1) >= col) then
          if res(row,j) = 0 then
            res(row,j) := 2;
          end if;
        end if;
      end loop;
    end loop;
    for j in 1..k loop       -- zeroes where white on black
      white_on_black := false;
      for i in 1..n loop
        if sr(j) = p(i) and sc(k-j+1) = n-i+1
         then white_on_black := true;
        end if;
        exit when white_on_black;
      end loop;
      if white_on_black
       then for j2 in j+1..k loop
              res(sr(j),j2) := 0;
            end loop;
      end if;
    end loop;
    return res;
  end Coordinate_Matrix;

  procedure Define_Homotopy ( n,k : in natural; p,row,col : in Vector ) is

    d : constant natural := Descending_Checker(p);
    r,cr,cd : natural;
    np : Vector(p'range) := p;
    b : Board(1..n,1..n);
    stay,swap : boolean;

  begin
    if d = 0
     then put_line("there is no homotopy to define...");
     else put("The descending checker is at (");
          put(p(d),1); put(","); put(d,1); put_line(")");
          r := Rising_Checker(p,d);
          put("The rising checker is at (");
          put(p(r),1); put(","); put(p'last-r+1,1); put_line(")");
          Move(np,d,r);
          b := Configuration(np);
         -- put_line("The board after moving black checkers : "); Write(b);
          cr := Critical_Row(p(d),n-d+1,row,col);
          put("The critical row : "); put(cr,1); new_line;
          if cr = 0
           then put_line("There are no white checkers in critical row.");
                stay := true; swap := false;
           else cd := Top_White_Checker(p(r),n-r+1,n,row,col);
                put("The critical diagonal : "); put(cd,1); new_line;
                if cd = 0
                 then stay := true; swap := false;
                 else case Central_Choice(p,d,r,row,col,cr,cd,true) is
                        when 0 => stay := true; swap := false;
                        when 1 => stay := false; swap := true;
                        when others => stay := true; swap := true;
                      end case;
                end if;
          end if;
    end if;
  end Define_Homotopy;

  procedure Test_Coordinates ( n,k : in natural ) is
 
  -- DESCRIPTION :
  --   Reads in a checker board and sets up the coordinates
  --   associated with the positions of white and black checkers.

    p : Vector(1..n);
    r,c : Vector(1..k);
    b : Board(1..n,1..n);
    cm : Matrix(1..n,1..k);

  begin
    put_line("Reading the permutation for the black checkers...");
    Read_Permutation(p);
    Write_Permutation(p);
    put_line("Reading rows and columns for the white checkers...");
    Read_Permutation(r);
    Read_Permutation(c);
    b := Configuration(p);
    Place_White(b,r,c);
    put_line("The board with black and white checkers : ");
    Write(b,p,r,c);
    cm := Coordinate_Matrix(n,k,p,r,c);
    put_line("The pattern of the coordinate matrix : ");
    put(cm);
    Define_Homotopy(n,k,p,r,c);
  end Test_Coordinates;

  procedure Main is
 
    n,k : natural;

  begin
    put("Give n : "); get(n);
    put("Give k : "); get(k);
    Test_Coordinates(n,k);
  end Main;

begin
  Main;
end ts_induce;
