/* The following is a simple interface of MPI with PHCpack:
 * after prompting the user for a list of solutions,
 * the solution list is broadcasted to all the nodes. */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "mpi.h"

extern void adainit( void );
extern int _ada_use_solcon ( int task, int *a, int *b, double *c );
extern void adafinal( void );

void dimension_broadcast ( int myid, int *len, int *n );
/* gets the length of the solution list and the dimension n
 * from the root node, and broadcasts then to all the nodes */

void solutions_broadcast ( int myid, int len, int n );
/* broadcasts all solutions in the container at the root
 * to the solutions container at each node */

void print_broadcast ( int myid );
/* prints the result of the broadcast at the node */

void print_solutions ( void );
/* prints the solutions in the container */

int main ( int argc, char *argv[] )
{
   int myid,numprocs,len,n;
   MPI_Status status; 

   adainit();
   MPI_Init(&argc,&argv);
   MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
   MPI_Comm_rank(MPI_COMM_WORLD,&myid);

   dimension_broadcast(myid,&len,&n);
   solutions_broadcast(myid,len,n);
   print_broadcast(myid);

   MPI_Finalize();
   adafinal();

   return 0;
}

void dimension_broadcast ( int myid, int *len, int *n )
{
   int k,fail;
   double *c;

   if(myid == 0)
   {
      fail = _ada_use_solcon(0,&k,&k,c);   /* prompts user for solutions */
      /* printf("The solution list in the container :\n"); */
      /* prints the solution list on screen */
      /* fail = _ada_use_solcon(1,&k,&k,c); */
      fail = _ada_use_solcon(2,&k,len,c);  /* get #solutions */
      fail = _ada_use_solcon(3,&k,n,c);    /* get dimension */
      printf("Root read %d solutions of dimension %d.\n",*len,*n); 
   }
 
   MPI_Bcast(n, 1, MPI_INT, 0, MPI_COMM_WORLD);  
   MPI_Bcast(len, 1, MPI_INT, 0, MPI_COMM_WORLD);  

   if(myid != 0)
      printf("Node %d expects %d solutions of dimension %d\n",myid,*len,*n);
}

void solutions_broadcast ( int myid, int len, int n )
{
   int i,k,m[2],fail;
   double sol[2*n+5];

   m[0] = n;
   for(k=1; k<=len; k++)
   {
      if(myid == 0) fail = _ada_use_solcon(4,&k,&m[1],sol);
      MPI_Bcast(&m[1],1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(sol,2*n+5,MPI_DOUBLE,0,MPI_COMM_WORLD);
      if(myid != 0) fail = _ada_use_solcon(6,&k,m,sol);
   }
}

void print_broadcast ( int myid )
{
   if(myid == 1)
   {
      printf("The solutions at node %d:\n", myid);
      print_solutions();
   }
}

void print_solution ( int n, double rt, double it, int m, double *v,
                      double err, double rco, double res )
{
   int i;

   printf("t : %.15e  %.15e\n", rt, it);
   printf("m : %d\n", m);
   printf("The solution for t :\n");
   for(i=0; i<n; i++)
      printf(" %.15e  %.15e\n", v[2*i], v[2*i+1]);
   printf("== err : %.3e = rco : %.3e = res : %.3e ==\n",err,rco,res);
}

void print_solutions ( void ) 
{
   int k,len,n,m,fail;
   double *c;

   fail = _ada_use_solcon(2,&k,&len,c);
   fail = _ada_use_solcon(3,&k,&n,c);

   c = (double*)calloc(2*n+5,sizeof(double));

   for(k=1; k<=len; k++)
   {
      fail = _ada_use_solcon(4,&k,&m,c);
      printf("Solution %d : \n", k);
      print_solution(n,c[0],c[1],m,c+2,c[2*n+2],c[2*n+3],c[2*n+4]);
   }
}
