/* The following is a simple interface of MPI with PHCpack:
 * after prompting the user for a polynomial system,
 * the system is broadcasted to all the nodes. */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "mpi.h"

extern void adainit( void );
extern int _ada_use_syscon ( int task, int *a, int *b, double *c );
extern void adafinal( void );

void dimension_broadcast ( int myid, int *n );
/* gets the dimension n from the root node,
 * broadcasts the dimension then to all the nodes, and
 * initializes the systems container at each node */

void monomials_broadcast ( int myid, int n );
/* broadcasts all monomials in the system to the nodes,
 * adding the monomials to the containers at each node */

void print_broadcast ( int myid );
/* prints the result of the broadcast at the node */

void print_monomials ( void );
/* prints the monomials in the container */

void print_system ( void );
/* prints the polynomial system in the container */

int main ( int argc, char *argv[] )
{
   int myid,numprocs,n;
   MPI_Status status; 

   adainit();
   MPI_Init(&argc,&argv);
   MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
   MPI_Comm_rank(MPI_COMM_WORLD,&myid);

   dimension_broadcast(myid,&n);
   monomials_broadcast(myid,n);
   print_broadcast(myid); 

   MPI_Finalize();
   adafinal();

   return 0;
}

void dimension_broadcast ( int myid, int *n )
{
   int *d,fail,m;
   double *c;

   if(myid == 0)
   {
      fail = _ada_use_syscon(0,n,d,c);    /* read system */
      fail = _ada_use_syscon(1,n,d,c);    /* write system */
      fail = _ada_use_syscon(2,n,d,c);    /* get dimension */
      printf("The dimension is %d.\n", *n); 
   }
 
   MPI_Bcast(n, 1, MPI_INT, 0, MPI_COMM_WORLD);  

   if(myid != 0)
   {
      printf("Node %d knowns that the dimension is %d\n", myid, *n);
      fail = _ada_use_syscon(3,n,d,c);    /* initialize container */
      fail = _ada_use_syscon(2,&m,d,c);   /* get dimension as test */
      printf("  and initialized container with dimension %d.\n", m);
   }
}

void monomials_broadcast ( int myid, int n )
{
   double cff[2];
   int i,j,exp[n],m[3],mm,fail;

   for(i=1; i<=n; i++)
   {
      m[1] = i;
      if(myid == 0)
      {
         fail = _ada_use_syscon(4,m,exp,cff);    /* get #monomials */
         mm = m[0];
         printf("Polynomial %d has %d monomials.\n",i,mm);
      }

      MPI_Bcast(&mm, 1, MPI_INT, 0, MPI_COMM_WORLD);

      m[0] = n;
      for(j=1; j<=mm; j++)    /* broadcast j-th term of i-th polynomial */
      {
         m[2] = j;
         if(myid == 0) fail = _ada_use_syscon(5,m,exp,cff); 
         MPI_Bcast(cff,2,MPI_DOUBLE,0,MPI_COMM_WORLD); 
         MPI_Bcast(exp,n,MPI_INT,0,MPI_COMM_WORLD); 
         if(myid != 0) fail = _ada_use_syscon(6,m,exp,cff); 
      }
   }
}

void print_broadcast ( int myid )
{
   if(myid == 1)
   {
      printf("The monomials at node %d:\n", myid);
      print_monomials();
      printf("The polynomial system at node %d:\n", myid);
      print_system();
   }
}

void print_monomials ( void ) 
{
   int *d,i,j,k,n,m[3],mm,fail;
   double c[2];

   fail = _ada_use_syscon(2,&n,d,c);

   d = (int*)calloc(n,sizeof(int));

   for(i=1; i<=n; i++)
   {
      m[1] = i;
      fail = _ada_use_syscon(4,m,d,c);
      mm = m[0];
      m[0] = n;
      printf("Polynomial %d has %d monomials :\n",i,mm);
      for(j=1; j<=mm; j++)
      {
         m[2] = j;
         fail = _ada_use_syscon(5,m,d,c);
         printf(" %.15e  %.15e",c[0],c[1]);
         for(k=0; k<n; k++) printf(" %d",d[k]);
         printf("\n");
      }
   }
}

void print_system ( void ) 
{
   int *d,*n,fail;
   double *c;

   fail = _ada_use_syscon(1,n,d,c);
}
